! ************************************************************
! BFGS Minimization-f90 version of Spellucci's domin code
! Originally modified by Daresbury labs
! Updated by Paul Quinn for use in MEIS fitting

! 1) As recieved f90 version of Domin
!
! 2) Optional cholesky factorisation with 
!    modified cholesky factorisation
!    Just uncomment the relevent bits  ! modchl and solve
!
!    This ensures that an indefinite or semidefinite
!    matrix is altered to a positive definite matrix
!    and that reasonable desent direction is obtained
!    Method outline by Gill and Murray
!    "Practical Optimisation"
!    Method : ACM on Mathematical Software algorithm 685 
!    Although BFGS should always result in a positive definite
!    matrix in theory in practice rounding errors etc. can
!    stop this happening
!    Yhis really depends on how you chose to initialise
!    your search.
!    If you chose to initialise with the 2nd derivatives
!    then this can/could be useful
!    Or if you initialise with averaged absolute values then
!    you won't need it
!    This method sorts this out 
!    The problem with the modified cholesky method is it 
!    takes a bit longer to run
!  
! 3) Linesearch updated to include cubic interpolation
!
! 4) Replaced idea of smallest step size sigsm
!    with smallest physical step size
!    ||sig*d|| > 0.001
!    ||sig*d|| < 0.5
!    0.001 smallest physically realistic step 
!    Removed scaling of d to limit max step
!    Found that large steps occur for semi-definite or ill-chosen
!    initial hessians
!    This problem sorted by (2) 
!
! 5) Covergence criteria altered to that of 
!    Gill and Murray Practical Optimisation
!    Also included external subroutine rval to
!    check rfactor curves
!  
! 7) Counts number of failed steps n linesearch
!    If > 10 quit out with step not found
!
! 8) Removed the "flatness" check bfgs=-2
!    Causes the algorithm to quit prematurely
!    even on standard functions e,g. rosenbrock function
!
!
!  9) Optional parameter scaling 
!	  Vibrations and Structure are difficult to
!     fit togtether due to the "conditioning"
!     i.e. how the r-factor reacts to changes in a parameter
!     If you fit the two together the algorithm concentrates
!     on the vibrations and ignores the vibrations.
!     This problem can also occur in simulations involving
!     large mass differences.
!     E.g. Ru and O. The movement of oxygen 
!     has a very small effect on the r-factor and tends to be ignored
!     by the search procedure
!     Solution : Parameter Scaling 
!    
! 10) Changed initialisation routine
!     To suit 2 and 9
!
! 11) Increased output as per Spellucci 
!     and a bit more
!     
! ************************************************************

! 18/1/2000
MODULE numbers
  implicit none
  integer float
  parameter(float=4)
End module numbers

MODULE domindat
 use numbers
 implicit none
 logical test1,test2,test3,cold,test4
 integer n,analyt,idump
 integer icf,icgf,iup,inoup,ires,imod
 integer niter,iterma

 real (float)  fx,xnorm,dnorm
 real (float)  sig,fx0,x0norm
 real (float)  d0norm,sig0,ftest,dirder
 real (float)  stepte,sigsm,sigla,alpha
 real (float)  beta,delta,theta,dskal
 real (float)  epsx,epsg,epsf
 real (float)  epsmac,tolmac,deldif
 real (float)  tracea,traceb
 real (float)  eta,thet2,bfgste, gstep
 real (float) ,allocatable,dimension(:)    :: x,d,gradx,x0,d0,gradx0,difx,grad2x,grdtmp
 real (float) ,allocatable,dimension(:)    :: diaga,diaga0,y,z,xst,scal
 real (float) ,allocatable,dimension(:,:)  :: a,a0,accinf,r
 real         ,allocatable,dimension(:)    :: E,V,TEMP_AW,x1
 integer      ,allocatable,dimension(:)    :: P
 real									   :: tau1,tau2


End module domindat


MODULE search1
  ! ************************************************************
  ! BFGS Minimization-modified version of Spellucci's domin code
  ! ************************************************************
  use numbers
   public domin
  private
  integer  nsmall,maxit
  parameter(nsmall=10,maxit=3000)
contains
  !-------------------------------------------------------------------------------
  subroutine domin(nuser,uxst,ftn,gftn,rftn,rftol,iout,start_con,scaleopt)
    use domindat
    logical start_con
    integer j,k,nuser,iout,scaleopt
    real (float)  term,tol1,rftol
	real (float)  uxst(nuser)
    external ftn,gftn,rftn

	n    = nuser
    allocate(x(n));x = 0.0_float
    allocate(d(n));d = 0.0_float
    allocate(gradx(n));gradx = 0.0_float
    allocate(x0(n));x0 = 0.0_float
    allocate(d0(n));d0 = 0.0_float
    allocate(gradx0(n));gradx0 = 0.0_float
    allocate(grad2x(n));grad2x = 0.0_float
    allocate(difx(n));difx = 0.0_float
    allocate(diaga(n));diaga = 0.0_float
    allocate(diaga0(n));diaga0 = 0.0_float
    allocate(y(n)); y = 0.0_float
    allocate(z(n)); z = 0.0_float
    allocate(a(n,n)); a = 0.0_float
    allocate(r(n,n)); r = 0.0_float
    allocate(a0(n,n));a0 = 0.0_float
    allocate(accinf(maxit,14)); accinf = 0.0_float
    allocate(xst(n));xst = 0.0_float
	allocate(p(n));p=0.0
	allocate(e(n));e=0.0
	allocate(v(n));v=0.0
	allocate(x1(n));x1=0.0
    allocate(scal(n));x1=1.0
	allocate(grdtmp(n));grdtmp=0.0	
	allocate(temp_aw(n));temp_aw=0.0


    !
    ! Generate machine precision
    !
    epsmac			= 2._float**(-20)
    term			= 1._float+epsmac
    do while ( term .ne. 1._float )
     epsmac			= epsmac/2._float
     term			= 1._float+epsmac
    enddo
    epsmac			= epsmac+epsmac
    tolmac			= epsmac
    do while ( tolmac .ne. 0._float )
     tol1			= tolmac
     tolmac			= tolmac/16._float
    enddo
    tolmac			= tol1
    test1			= .false.
    epsx			= epsmac**(2._float/3._float)
    epsg			= epsx
    epsf			= epsmac*1.0e3_float
    eta				= epsmac**.5_float
    sigsm			= epsx
    sigla			= 2048._float
    beta			= 2._float

    alpha			= 0.1_float
    delta			= 0.01_float
    theta			= 0.99_float
    deldif			= exp(log(epsmac)/7._float)*.25_float
    iterma			= maxit

!
	tau1 = epsmac ** (1./3.)
    tau2 = epsmac ** (1./3.)
!
    !
    ! Previously in user defined subroutine setup
    !
    n			= nuser
    xst(1:n)    = uxst(1:n)
    cold		= start_con
    idump		= 0
    !
    icf			=0
    icgf		=0
    iup			=0
    inoup		=0
    ires		=0
    imod		=0


    open(10,file='domin.res',status='unknown')
	write(10,*) 'Started Search'
	close(10)
    call bfgs(cold,ftn,gftn,rftn,rftol,iout,scaleopt)
    write(iout,*)'termination reason',bfgste

    ! Termination reason
    ! -3  Errors in definition of f(x)
    ! -2  itmax steps exceeded (usually due to ill conditioning
    ! -1  No acceptable stepsize can be found. (usually due to in sufficient precsion in grad)
    !  0  norm(d) = 0 (grad is zero)
    !  1  sufficient precsion attained
    !  2  f is flat
    !  3  four steps have been taken without a change in f
    !  4  all variables are 0.0 
    !  5  exited using user defined criteria 1
    !
! CBI
!    write(iout,*)'optimum value of f',fx
!    write(iout,*)'optimum parameters',x(1:n)
!    write(iout,*)'gradients at minimum',gradx(1:n)
1969  format(/,' BEST FIT =',f18.10)
     write(iout,1969) fx
1970  format(/,' OPTIMIZED PARAMETERS:')
1971  format(f14.8)
      write(iout,1970) 
      write(iout,1971) x(1:n)
1972  format(/,' GRADIENTS AT MINIMUM:')
1973  format(f14.8)
      write(iout,1972) 
      write(iout,1973) gradx(1:n)
       write(iout,*) 
! CBF
	  write(iout,'(1X,A,I4,1X,A,I4)')'function calls',icf,    &
				  'gradient calls',icgf
      write(iout,'(1X,A,I4,1X,A,I4,1X,A,I4)')'steps',niter,    &
			'updates',iup,'noupdates',inoup
      write(iout,'(1X,A,I4,1X,A,I4)')'restarts',ires,'modsteps',imod

      if ( bfgste .lt. 0._float ) test1=.true.
      if ( test1 ) then
       write(iout,fmt='('' bfgs short protocol '')')
       do j=1,niter
        write(iout,*) (accinf(j,k),k=1,14)
       enddo
      endif
      close(10)
      deallocate(xst)
      deallocate(accinf)
      deallocate(a0)
      deallocate(a)
      deallocate(z)
      deallocate(y)
      deallocate(diaga0)
      deallocate(diaga)
      deallocate(difx)
      deallocate(gradx0)
      deallocate(d0)
      deallocate(x0)
      deallocate(gradx)
      deallocate(d)
      deallocate(x)

    end subroutine domin


    !-----------------------------------------------------------------
    subroutine step(ftn,gftn,scaleopt)
      !
      ! Determination of stepsize by armijo-like test for descent
      !
      use domindat
      integer i,nstepfail,scaleopt
      real (float)  t,sigmin,fmin,cosphi,f1
	  real			:: lp,l2p,newstep,oldstep,a11,a21,a12,a22
	  real			:: v1,v2,apq,bpq,f2
	  integer		:: cubic
      logical err
      external ftn,gftn

      !   n = actual dimension
      !   x = current point
      !   d = current direction
      !   gradx = gradient of f at x
      !   x,d,gradx = input ( in common )
      !
      !   x0,d0,gradx0 etc. information from previous step
      !  
      !   xnorm,dnorm = euclidean length of x and d
      !   stepte = 1 in case of success , =-1 otherwise
      !   sig = computed stepsize
      !   the routine assumes that sig=1 is asymptotically optimal
      !   (otherwise one has to change initialization of sig)
      !   sigsm = minimal stepsize allowed
      !   sigla = maximal stepsize allowed
      !   alpha = maximal reduction of stepsize per reduction-step
      !   delta = the armijo-parameter
      !   beta = max. growth of  xnorm allowed for  sig=1
      !   theta = bound for cos(angle(d,d0))
      !           we try stepsizes larger than one, if this is exceeded.
      !           in that case d0 and d point (almost) in the same direction
      !
		xnorm		= vecnor(n,x)
10		continue
		dnorm		= vecnor(n,d)
		if(scaleopt.eq.0) then
			if( dnorm >= 0.5 ) then 
				d(1:n) = d(1:n)/2.0
				goto 10
			end if
		end if
		dskal		= 1._float
		cosphi		= skal1(1,n,d,d0)
		nstepfail   = 0
		cubic       = 0



		do i=1,n
			write(*,*)    x(i),d(i)
		end do

		t=1._float
		do i=1,n
			if ( abs(d(i)) .gt. (abs(x(i))+1._float)*(beta-1._float) ) &
				t=min(t,(beta-1._float)*(abs(x(i))+1._float)/abs(d(i)))
		enddo
		if(t.lt.1._float)then
			dnorm	= t*dnorm
			dirder	= t*dirder
			cosphi	= cosphi*t
			dskal	= t
			d(1:n)	= d(1:n)*t
		endif
		cosphi		= abs(cosphi)/(dnorm*d0norm)
		sig		= 1._float

100		continue
		
		x1(1:n)		= x(1:n)-sig*d(1:n)

		err		=.false.
		icf		= icf + 1
! Evaluate f
		call byscale(x1)
		call ftn(n,x1,f1,err)
		call divscale(x1)
	
! If theres a problem evaluating f
! half the step and try again
		if ( err ) then
			if ( sig .gt. 1._float ) then
				err		= .false.
				sig		= sigmin
				f1		= fmin
				x1(1:n) = x(1:n)-sig*d(1:n)
				goto 200
			endif
				sig		= sig*.5_float
			if ( sig .ge. sigsm ) then
				goto 100
			else
				stepte  = -1._float
				return
			endif
		endif
! If sig is greater than 1
		ftest		= f1
		call domin_info(6)

		if ( sig .gt. 1._float ) then
			if ( f1 .gt. fmin ) then
				sig		= sigmin
				f1		= fmin
				x1(1:n) = x(1:n)-sig*d(1:n)
				goto 200
			else
				if ( sig .lt. sigla ) then
					fmin = f1
					sigmin = sig
					sig=min(sigla,sig+sig)
					goto 100
				else
					goto 200
				endif
			endif
		endif
! Goldstein-Armijo satisfied 
! If satisfied and nothing unusual accept step and
! return from subroutine
! If not satisfied then 
! chose a step that is max of 
! 0.01*current step or quadratic fit to points 
		if ( fx - f1 .ge. sig*delta*dirder ) then
			if ( sig .eq. 1._float .and. cosphi .ge. theta .and. sig0  &
					.ge. 1._float  ) then
					fmin	= f1
					sigmin  = sig
					if ( fx-f1 .lt. sig*dirder ) then
						sig=min(sigla, &
							max(sig+sig,dirder*sig**2/(2._float*(f1-fx+sig*dirder))))
					else
						sig=min(sigla,sig+sig)
					endif
					goto 100
			endif
			goto 200
		else

! PQ added
! Count number of steps
! if can't find a better f after 10 steps then return with step fail
! i.e. stepterm= - 1.0
! Also added cubic fit to points
				nstepfail=nstepfail+1
				if(nstepfail.gt.10) then
					stepte = -1.0
					return
				end if
				if(cubic.eq.0) then
					oldstep = sig
					newstep = sig*dirder/(2.0*(f1-fx+sig*dirder))
					newstep = oldstep*newstep
					cubic   = 1
					! safe-guarded
					sig=max(alpha*sig,min(newstep,0.5_float*sig))
 					else if (cubic.eq.1) then
					lp		= newstep
					l2p		= oldstep
					a11		= 1 / (lp*lp)
					a21		= - l2p / (lp*lp)
					a12		= - 1 / (l2p*l2p)
					a22		= lp / (l2p*l2p)
					v1		= f1 - fx - dirder*lp
					v2		= f2 - fx - dirder*l2p
					apq		= 1. / (lp - l2p) * (a11 * v1 + a12 * v2)
					bpq		= 1. / (lp - l2p) * (a21 * v1 + a22 * v2)


					oldstep = newstep
					! Can obtain valid cubic fit Yes or no ?
!					if(abs(apq) > 0.0 .and. (bpq*bpq) > (3.0*apq*dirder)) then
					if(abs(apq) > tolmac) then
						newstep = - bpq + sqrt(bpq*bpq-3.0*apq*dirder)/(3.*apq)	
						newstep = - newstep    
					else
						newstep =   newstep*dirder/(2.*(f1-fx+newstep*dirder))
						newstep =   newstep*oldstep
					end if
					! Safe guarded 
					sig=max(alpha*sig,min(newstep,0.5_float*sig))
			   end if
               f2 = f1
			   temp_aw(1:n)=sig*d(1:n)
! Pq added parameter scaling 			   
			   temp_aw(1:n)=temp_aw(1:n)*scal(1:n)
! PQ modified min step size
! is set so as smallest 'real' step is 0.001
				if((vecnor(n,temp_aw)).lt.0.001) then		
					stepte = -1.0
					return
				else
					goto 100
				endif
		endif
200		continue
! Store new best values 
		fx0		= fx
		fx		= f1
		stepte		= 1._float 
		x0norm		= xnorm
		d0norm		= dnorm
		sig0		= sig
		gradx0(1:n)	= gradx(1:n)
		x0(1:n)		= x(1:n)
		d0(1:n)		= d(1:n)
		x(1:n)		= x1(1:n)
		difx(1:n)	= x(1:n)-x0(1:n)
! Estimate gradients
! Scale and unscale
        call byscale(x)
		call gftn(n,x,gradx,grad2x,err)
		call divscale(x)
        call byscale(gradx)
! Output scaled and unscaled vectors
	    call domin_info(8)
		icgf		= icgf + 1
		xnorm		= vecnor(n,x)
		if ( err ) then
			stepte=-2._float
		return
		endif
    end subroutine step
    
    !----------------------------------------------------------------
    
    real (float)  function skal1(i,j,a,b)
		implicit none
		integer i,j
		real (float)  a(*),b(*)
		if ( i .gt. j ) then
			skal1	= 0._float
		else
			skal1	= dot_product(a(i:j),b(i:j))
		endif
    end function skal1
    !-------------------------------------------------------------------
    subroutine bfgs(cold1,ftn,gftn,rftn,rftol,iout,scaleopt)
      use domindat
      implicit none
      logical		:: cold1,fit
      integer		:: cfold,cgfold,icsf,i,j
      integer		:: iout,ifail,scaleopt
      real (float)  :: s1,s2,s3,s4,s5,difxin,wi,rftol
      real (float)  :: den1 
	  real (float)  :: dif1,dif2,h,term_crit1,term_crit2,term_crit3
      real (float)  :: user_xtol,user_ftol,user_gtol
      logical		:: resta
      external		   ftn,gftn,rftn
      !
	  resta = .false.
      icsf=0
      ! Fresh start
		if (cold1) then
			x(1:n)		= xst(1:n)
			call initia(ftn,gftn,scaleopt)
			x0(1:n)		= x(1:n)
			difx(1:n)	= 1._float+abs(x(1:n))
			xnorm		= vecnor(n,x)
			sig0		= 0._float
			d0norm		= 1._float
			niter		= 0
			thet2		= 1._float
			bfgste		= 0._float
			d0			= 0._float
      ! Restarting search using old data			
		else
			write(iout,*)'---------------------------------------------'
			write(iout,*)'Reloading search data from previous dump file'
			write(iout,*)'---------------------------------------------'
			call domin_read
		end if
100   continue
		if(idump.gt.0.or.(idump.eq.0.and.cold1))then
			if(idump.eq.0)then
				write(iout,*)
				write(iout,*) 'Creating restart file on unit 69'
			else
				write(iout,*)
				write(iout,*)'Updating restart file'
			end if
			call domin_dump
		end if
		idump = idump + 1
		niter=niter+1
		if ( niter .gt. iterma ) then
			niter	= niter-1
			bfgste	= -2._float
		return
		endif
		accinf(niter,:)		= 0._float
		accinf(niter,1)		= niter
		accinf(niter,2)		= fx
		accinf(niter,3)		= vecnor(n,difx)
		accinf(niter,4)		= vecnor(n,gradx)
		accinf(niter,13)	=-1._float
        call domin_info(1)
		s1			= sum(diaga(1:n))
		s2			= a(1,1)
		do i=2,n
			s2=min(s2,a(i,i))
		enddo
		if ( niter .gt. 1 ) then
			if ( (tracea*epsmac .gt. 1._float .and. traceb*epsmac .gt. 1._float) &
				.or. s1*epsmac .gt. s2**2 .or.  &
				accinf(niter-1,8) .gt. 1._float /eta  ) then
					call refres(niter,ftn,gftn,scaleopt)
					if ( bfgste .eq. -3._float ) return
					resta=.true.
			else
				resta=.false.
			endif
		endif
		call testx
		cfold			= icf
		cgfold			= icgf
		if ( bfgste .eq. 1._float )  return
200		continue
        call  cholso(a,n,gradx,d)
		dnorm	= vecnor(n,d)
		if ( dnorm .eq. 0._float ) then
			bfgste=0._float
			return
		endif

		dirder	= skal1(1,n,d,gradx)

		accinf(niter,7)	= dirder

!
!		Removed by pq
!		ends in premature termination
!
!		if ( delta*dirder .le. epsmac*(abs(fx)+1._float)*100._float ) then
!			bfgste=2._float
!			return
!		endif


! =====================================
! Perform linesearch
! =====================================
     call domin_info(2)
     call step(ftn,gftn,scaleopt)
! 
! Reevaluate variable scaling done after first search 
! step
! Found that if first guess neg second derivatives
! it effects the speed of the search
! However The first step : steepest descent direction
! usually results in positive hessian 
!

!	 if(scaleopt.eq.1.and.niter.eq.1) call scaling(grad2x)	


! =====================================
! Check convergence
! =====================================
	call byscale(x)
	call byscale(x0)	
	call divscale(gradx)		

	  do i=1,n	
      		temp_aw(i) = x(i) - x0(i)	
	  end do
	      term_crit1 = vecnor(n,temp_aw)
	      term_crit2 = abs(fx-fx0)
	      term_crit3 = vecnor(n,gradx)
	  user_xtol  = sqrt(rftol)*(1.0+(vecnor(n,x0))) 
	  user_ftol  = rftol*(1.0+abs(fx0))
	  user_gtol  = (rftol**0.33333333)*(1+abs(fx0))

	call divscale(x)
	call divscale(x0)	
	call byscale(gradx)		

		
!CBI
1966  format(/,' XTOL =',f18.10,10x,'(CONV. =',f18.10,')')
1967  format(' ETOL =',f18.10,10x,'(CONV. =',f18.10,')')
1968  format(' GTOL =',f18.10,10x,'(CONV. =',f18.10,')')
      write(*,1966) term_crit1, user_xtol
      write(*,1967) term_crit2, user_ftol
      write(*,1968) term_crit3, user_gtol
!CBF
	   call rftn(fit)
       if(term_crit1.lt.user_xtol.and.term_crit2.lt.user_ftol	&
	&   	.and.term_crit3.lt.user_gtol)then
        bfgste = 5.0
        return
        end if

        if(fit)then
			bfgste = 5.0
			return
        end if

! ===========================!
! Convergence not satisfied  !
! ===========================!
! Record current parameters	 !
! in array accinf			 !
! ===========================!

      accinf(niter,9)	= sig
      accinf(niter,10)	= dskal
      accinf(niter,11)	= dnorm
      i					= icf-cfold
      if(analyt.eq.1) i = i-6*n*(icgf-cgfold)
      if(analyt.eq.2) i = 1-n*(icgf-cgfold)
      accinf(niter,12)  = real(i)



!====================================== 
! Linesearch didn't find suitable step
! so restart
!======================================
    call domin_info(3)
	if ( stepte .ne. 1._float ) then
			accinf(niter,13)	= 1._float
! If you've already restarted then quit 
!			if (resta) then
!				bfgste=-1._float
!				return
			! otherwise restart search
			! Reinitialise hessian etc.....
!			else
				call refres(niter,ftn,gftn,scaleopt)
				if ( bfgste .eq. -3._float ) return
					resta=.true.
					goto 200
!			endif		
	endif
!
! Check change in best f
! if repeats 4 times quit

			if ( fx0-fx .gt. epsf*(abs(fx)+epsf)) then
				icsf	= 0
			else
				icsf	= icsf+1
			endif
			if ( icsf .eq. 4 ) then
				bfgste	= 3._float
				return
			endif
!
! If change in x is 0.0 then quit
!
			if ( vecnor(n,difx) .eq. 0 ) then
				bfgste	= 4._float
				return
			endif
!
! Update using bfgs method 
!
!*********************************************
!     compute update vectors z and y
!*********************************************
			thet2	= 1._float
			s1		= 0._float
			s2		= 0._float
			s3		= 0._float
			den1	= skal1(1,n,d,gradx0)/dskal
			h		= 1._float/sqrt(den1)
			do   i=1,n
				y(i)=gradx0(i)*h
				dif1=difx(i)
				dif2=gradx(i)-gradx0(i)
				s1=dif1**2 + s1
				s2=dif2**2 + s2
				s3=dif1*dif2 + s3
				z(i)=dif2
			enddo
			if ( s3 .lt. eta*s2 ) then
!**********************************************
!     bfgs-updating impossible from this information
!     f not uniformly convex of course
!**********************************************
				s4=((sig*dskal)**2)*den1
				if ( s3 .lt. .2_float*s4 ) thet2=.8_float*s4/(s4-s3)
				if ( thet2 .ne. 1._float ) imod=imod+1
				s3=0.2_float*s4
				h=(1._float-thet2)*sig*dskal
!******** powells modification
!******** h*gradx0 = a(k)*s(k)*(1-thet2)
				do   i=1,n
					z(i)=thet2*z(i)+h*gradx0(i)
				enddo
			endif
			accinf(niter,6)=thet2
			s3=1._float/sqrt(s3)
			do   i=1,n
				z(i)=z(i)*s3
			enddo
	                call domin_info(4)
			accinf(niter,14)=1._float
! Solve 
            call cholso(a,n,z,v)
!			call solve(n,A,P,v,z)
!**** v = b(k)*y(k)/sqrt(y's) in the usual notation
			s4=0._float
			s5=0._float
			do i=1,n
				difxin=difx(i)*s3
				wi=difxin+v(i)
!****   w=(sig(i)+(b*y)(i))/sqrt(y's)
				s5=s5+z(i)*wi
				s4=s4+v(i)*difxin
			enddo

			do i=1,n
					diaga0(i)	= diaga(i)
					a0(i,i)		= a(i,i)
					diaga(i)	= diaga(i)-y(i)**2+z(i)**2
					a(i,i)		= diaga(i)               
				do  j=1,i-1
					a0(i,j)	= a(i,j)
					a0(j,i)	= a(j,i)
					a(i,j)	= a(i,j)-y(i)*y(j)+z(i)*z(j)
				enddo
			enddo
            call chol(a,n,a,ifail)
! Modified Cholesky factorisation
!			call modchl(n,a,tau1,tau2,P,E)
!			iup=iup+1 
!
	        if ( ifail .ne. 0 ) then
			   accinf(niter,14)=-1._float
			   inoup=inoup+1
   			   diaga(1:n)=diaga0(1:n)
 			   a(1:n,1:n) =a0(1:n,1:n)
			else
			iup=iup+1 
			endif

		    if ( accinf (niter,14) .eq. 1._float ) then
				if ( tracea .gt. 0._float ) tracea=tracea+vecnor(n,z)**2-vecnor(n,y)**2
				if ( traceb .gt. 0._float ) traceb=traceb-2._float*s4+s1*s3**2*s5 
				call domin_info(7)
			endif
    call domin_info(5)
	goto 100
    end subroutine bfgs
!-----------------------------------------------------------
    subroutine domin_dump
!
!	Dump current variables
!
    use domindat
    open(unit=69,form='unformatted')
    write(69)test1,test2,test3,cold,test4
    write(69)n,analyt
    write(69)icf,icgf,iup,inoup,ires,imod
    write(69)niter,iterma
    write(69)fx,xnorm,dnorm
    write(69)sig,fx0,x0norm
    write(69)d0norm,sig0,ftest,dirder
    write(69)stepte,sigsm,sigla,alpha
    write(69)beta,delta,theta,dskal
    write(69)epsx,epsg,epsf
    write(69)epsmac,tolmac,deldif
    write(69)tracea,traceb
    write(69)eta,thet2,bfgste, gstep
    write(69)x,d,gradx,x0,d0,gradx0,difx
    write(69)diaga,diaga0,y,z,xst
    write(69)a,a0,accinf
    close(unit=69)
    end subroutine domin_dump
!-----------------------------------------------------------
    subroutine domin_read
!
!	Read variables form dump file
!
    use domindat
    write(6,*)'reading from dump file'
    open(unit=69,form='unformatted')
    read(69)test1,test2,test3,cold,test4
    read(69)n,analyt
    read(69)icf,icgf,iup,inoup,ires,imod
    read(69)niter,iterma
    read(69)fx,xnorm,dnorm
    read(69)sig,fx0,x0norm
    read(69)d0norm,sig0,ftest,dirder
    read(69)stepte,sigsm,sigla,alpha
    read(69)beta,delta,theta,dskal
    read(69)epsx,epsg,epsf
    read(69)epsmac,tolmac,deldif
    read(69)tracea,traceb
    read(69)eta,thet2,bfgste, gstep
    read(69)x,d,gradx,x0,d0,gradx0,difx
    read(69)diaga,diaga0,y,z,xst
    read(69)a,a0,accinf
    close(unit=69)
    end subroutine domin_read
    !------------------------------------------------------------
    subroutine initia(ftn,gftn,scaleopt)

!   Setup    approximate hessian 
!   optional parameter   scaling   
  
      use domindat
      integer i,scaleopt
      logical err,neg
      external ftn,gftn
! Set x to last value 
      
!     x(1:n)          = xst(1:n)
      scal(1:n)		  = 1.00
      
      call ftn(n,x,fx,err)
      err = .false.
      call gftn(n,x,gradx,grad2x,err)
      if ( err ) then
         bfgste=-3._float
         return
      endif
	   a	 = 0._float
	   a0	 = 0._float
 	   tracea = 0._float
 	   traceb = 0._float
       gn = vecnor(n,grad2x)
!	   call modchl(n,a,tau1,tau2,P,E)		      
		neg=.false.
			do i=1,n
				if(grad2x(i)<0) neg=.true.
			end do
       if(scaleopt.eq.1) then
			if(err) then 
				scal(1:n) = gn
			else
				call scaling(grad2x)	
			end if
	   call byscale(gradx)
	   call divscale(x)	
	   do i=1,n
  	           p(i) = i 
	           a(i,i) = 1.0
    	       diaga  = 1.0
	           diaga0 = 1.0
! 	   	       a(i,i)=1.0
!    	       diaga  = 1.0
!	           diaga0 = 1.0
	           tracea = tracea + a(i,i)
		   traceb = traceb + 1._float / a(i,i)
		end do
	  else 		
		do i=1,n
			p(i)   = i
			a(i,i) = sqrt(gn)
    		diaga  = gn
        	diaga0 = gn
!			a(i,i) = 1.0/sqrt(abs(a(i,i)) )
!    		diaga  = 1.0/sqrt(abs(a(i,i)) )
!        	diaga0 = 1.0/sqrt(abs(a(i,i)) )

			tracea = tracea + a(i,i)
			traceb = traceb + 1._float / a(i,i)
		end do
	  end if		
	call domin_info(8)
	call domin_info(9)
    end subroutine initia

!-------------------------------------------------------------
    subroutine refres(niter1,ftn,gftn,scaleopt)
!
!	Restart/Reinitialise hessian
!
		use domindat
		integer niter1,scaleopt,i
		external ftn,gftn
		ires=ires+1
		accinf(niter1,13)=1._float
		if(scaleopt.eq.1) then
			call byscale(x)
! Set new x as average of current best and first guess
!			do i=1,n
!				x(i) = ( x(i) + xst(i) )/2.0
!			end do
		end if
		call initia(ftn,gftn,scaleopt)

    end subroutine refres
    !----------------------------------------------------------------
    subroutine testx
		use domindat
		logical xsatis,gsatis
		real (float)  an,sx,sg,damin
		integer i
      !
		an=0._float
		sx=vecnor(n,difx)
		sg=vecnor(n,gradx)
		damin=a(1,1)
		do   i=1,n
			an=an+diaga(i)
			damin=min(damin,a(i,i))
		enddo
		accinf(niter,5)=an
		if ( tracea .gt. 0._float .and. traceb .gt. 0._float ) then
			 accinf(niter,8)=tracea*traceb/real(n*n)
		else
			 accinf(niter,8)=an/damin**2
		endif
		xsatis = sx .le. epsx*(xnorm+epsx)
		gsatis = sg .le. max(epsg,epsmac*accinf(niter,8)) 
		if ( xsatis .and. gsatis ) bfgste=1._float
    end subroutine testx
    !-----------------------------------------------------------------
    real (float)  function vecnor(n,x)
!
!	Euclidean norm
!
      implicit none
      integer n,i
      real (float)  x(*)
      real (float)  xm,h
      if ( n .le. 0 ) then
         vecnor=0._float
         return
      endif
      xm=abs(x(1))
      do i=2,n
         xm=max(xm,abs(x(i)))
      enddo
      if ( xm .eq. 0._float ) then
         vecnor=0._float
         return
      else
         h=0._float
         do i=1,n
            h=h+(x(i)/xm)**2
         enddo
         vecnor=xm*sqrt(h)
         return
      endif
    end function vecnor

!*********************************************************************
!
!       subroutine name: modchl
!
!       authors :  Elizabeth Eskow and Robert B. Schnabel
!
!       date    : December, 1988
!
!       purpose : perform a modified cholesky factorization
!                 of the form (Ptranspose)AP  + E = L(Ltranspose),
!       where L is stored in the lower triangle of the
!       original matrix A.
!       The factorization has 2 phases:
!        phase 1: Pivot on the maximum diagonal element.
!            Check that the normal cholesky update
!            would result in a positive diagonal
!            at the current iteration, and
!            if so, do the normal cholesky update,
!            otherwise switch to phase 2.
!        phase 2: Pivot on the minimum of the negatives
!            of the lower gerschgorin bound
!            estimates.
!            Compute the amount to add to the
!            pivot element and add this
!            to the pivot element.
!            Do the cholesky update.
!            Update the estimates of the
!            gerschgorin bounds.
!
!       input   : ndim    - largest dimension of matrix that
!                           will be used
!
!                 n       - dimension of matrix A
!
!                 A       - n*n symmetric matrix (only lower triangular
!                           portion of A, including the main diagonal, is used)
!
!
!                tau1    - tolerance used for determining when to switch
!                          phase 2
!
!                tau2    - tolerance used for determining the maximum
!                          condition number of the final 2X2 submatrix.
!
!
!       output  : L     - stored in the matrix A (in lower triangular
!                           portion of A, including the main diagonal)
!
!                 P     - a record of how the rows and columns
!                         of the matrix were permuted while
!                         performing the decomposition
!
!                 E     - n*1 array, the ith element is the
!                         amount added to the diagonal of A
!                         at the ith iteration
!
!
!***********************************************************************
      subroutine modchl(n,A,tau1,tau2,P,E)
      integer	:: n
      real		:: A(n,n),tau1,tau2
	  real,allocatable,dimension(:) :: g
      integer	:: P(n)
      real		:: E(n)

!
!  j        - current iteration number
!  iming    - index of the row with the min. of the
!           neg. lower Gersch. bounds
!  imaxd    - index of the row with the maximum diag.
!           element
!  i,itemp,jpl,k  - temporary integer variables
!  delta    - amount to add to Ajj at the jth iteration
!  gamma    - the maximum diagonal element of the original
!           matrix A.
!  normj    - the 1 norm of A(colj), rows j+1 --> n.
!  ming     - the minimum of the neg. lower Gersch. bounds
!  maxd     - the maximum diagonal element
!  taugam - tau1 * gamma
!  phase1      - logical, true if in phase1, otherwise false
!  delta1,temp,jdmin,tdmin,tempjj - temporary double precision vars.
!

      integer		:: j,iming,i,imaxd,itemp,jp1,k
      real			:: delta,gamma
      REAL			:: normj, ming,maxd
      REAL			:: delta1,temp,jdmin,tdmin,taugam,tempjj
      logical		:: phase1
      intrinsic abs, max, sqrt, min
	
	  allocate(g(n))
      call init(n,A, phase1, delta, P, g, E,	&
     &         ming,tau1,gamma,taugam)

!     check for n=1

      if (n.eq.1) then
         delta = (tau2 * abs(A(1,1))) - A(1,1)
         if (delta .gt. 0) E(1) = delta
         if (A(1,1) .eq. 0) E(1) = tau2
         A(1,1)=sqrt(A(1,1)+E(1))
      endif

      do 200 j = 1, n-1
!
!        PHASE 1
!
         if ( phase1 ) then
!
!           Find index of maximum diagonal element A(i,i) where i>=j
!
            maxd = A(j,j)
            imaxd = j
            do 20 i = j+1, n
               if (maxd .lt. A(i,i)) then
                  maxd = A(i,i)
                  imaxd = i
               end if
 20         continue

!
!           Pivot to the top the row and column with the max diag
!
            if (imaxd .ne. j) then
!
!              Swap row j with row of max diag
!
               do 30 i = 1, j-1
                  temp = A(j,i)
                  A(j,i) = A(imaxd,i)
                  A(imaxd,i) = temp
 30            continue
!
!              Swap colj and row maxdiag between j and maxdiag
!
               do 35 i = j+1,imaxd-1
                  temp = A(i,j)
                  A(i,j) = A(imaxd,i)
                  A(imaxd,i) = temp
 35            continue
!
!              Swap column j with column of max diag
!
               do 40 i = imaxd+1, n
                  temp = A(i,j)
                  A(i,j) = A(i,imaxd)
                  A(i,imaxd) = temp
 40            continue
!
!              Swap diag elements
!
               temp = A(j,j)
               A(j,j) = A(imaxd,imaxd)
               A(imaxd,imaxd) = temp
!
!              Swap elements of the permutation vector
!
               itemp = P(j)
               P(j) = P(imaxd)
               P(imaxd) = itemp

            end if


!           Check to see whether the normal cholesky update for this
!           iteration would result in a positive diagonal,
!           and if not then switch to phase 2.

            jp1 = j+1
            tempjj=A(j,j)

            if (tempjj.gt.0) then

               jdmin=A(jp1,jp1)
               do 60 i = jp1, n
                  temp = A(i,j) * A(i,j) / tempjj
                  tdmin = A(i,i) - temp
                  jdmin = min(jdmin, tdmin)
 60            continue

               if (jdmin .lt. taugam) phase1 = .false.

            else

               phase1 = .false.
 
            end if

            if (phase1) then
!
!              Do the normal cholesky update if still in phase 1
!
               A(j,j) = sqrt(A(j,j))
               tempjj = A(j,j)
               do 70 i = jp1, n
                  A(i,j) = A(i,j) / tempjj
 70            continue
               do 80 i=jp1,n
                  temp=A(i,j)
                  do 75 k = jp1, i
                     A(i,k) = A(i,k) - (temp * A(k,j))
 75               continue
 80            continue

               if (j .eq. n-1) A(n,n)=sqrt(A(n,n))

            else

!
!              Calculate the negatives of the lower gerschgorin bounds
!
               call gersch(n,A,j,g)

            end if

         end if


!
!        PHASE 2
!
         if (.not. phase1) then

            if (j .ne. n-1) then
!
!              Find the minimum negative gershgorin bound
!

               iming=j
               ming = g(j)
               do 90 i = j+1,n
                  if (ming .gt. g(i)) then
                     ming = g(i)
                     iming = i
                  end if
 90            continue

!
!               Pivot to the top the row and column with the
!               minimum negative gerschgorin bound
!
                if (iming .ne. j) then
!
!                  Swap row j with row of min gersch bound
!
                   do 100 i = 1, j-1
                      temp = A(j,i)
                       A(j,i) = A(iming,i)
                       A(iming,i) = temp
 100               continue
!
!                  Swap colj with row iming from j to iming
!
                   do 105 i = j+1,iming-1
                      temp = A(i,j)
                      A(i,j) = A(iming,i)
                      A(iming,i) = temp
 105              continue
!
!                 Swap column j with column of min gersch bound
!
                  do 110 i = iming+1, n
                     temp = A(i,j)
                     A(i,j) = A(i,iming)
                     A(i,iming) = temp
 110              continue
!
!                 Swap diagonal elements
!
                  temp = A(j,j)
                  A(j,j) = A(iming,iming)
                  A(iming,iming) = temp
!
!                 Swap elements of the permutation vector
!
                  itemp = P(j)
                  P(j) = P(iming)
                  P(iming) = itemp
!
!                 Swap elements of the negative gerschgorin bounds vecto
!
                  temp = g(j)
                  g(j) = g(iming)
                  g(iming) = temp

               end if
!
!              Calculate delta and add to the diagonal.
!              delta=max{0,-A(j,j) + max{normj,taugam},delta_previous}
!              where normj=sum of |A(i,j)|,for i=1,n,
!              delta_previous is the delta computed at the previous iter
!              and taugam is tau1*gamma.
!

               normj = 0.0
               do 140 i = j+1, n
                  normj = normj + abs(A(i,j))
 140           continue

               temp = max(normj,taugam)
               delta1 = temp - A(j,j)
               temp = 0.0
               delta1 = max(temp, delta1)
               delta = max(delta1,delta)
               E(j) =  delta
               A(j,j) = A(j,j) + E(j)
!
!              Update the gerschgorin bound estimates
!              (note: g(i) is the negative of the
!               Gerschgorin lower bound.)
!
               if (A(j,j) .ne. normj) then
                  temp = (normj/A(j,j)) - 1.0


                  do 150 i = j+1, n
                     g(i) = g(i) + abs(A(i,j)) * temp
 150              continue

               end if
!
!              Do the cholesky update
!
               A(j,j) = sqrt(A(j,j))
               tempjj = A(j,j)
               do 160 i = j+1, n
                  A(i,j) = A(i,j) / tempjj
 160           continue
               do 180 i = j+1, n
                  temp = A(i,j)
                  do 170 k = j+1, i
                     A(i,k) = A(i,k) - (temp * A(k,j))
 170              continue
 180           continue

            else

               call fin2x2(n, A, E,tau2, delta,gamma)

            end if

         end if

 200   continue
	  deallocate(g)
      return
      end subroutine modchl

!***********************************************************************
!       subroutine name : init
!
!       purpose : set up for start of cholesky factorization
!
!       input : n, A, tau1
!
!       output : phase1    - boolean value set to true if in phase one,
!             otherwise false.
!      delta     - amount to add to Ajj at iteration j
!      P,g,E - described above in modchl
!      ming      - the minimum negative gerschgorin bound
!      gamma     - the maximum diagonal element of A
!      taugam  - tau1 * gamma
!
!***********************************************************************
      subroutine init(n,A,phase1,delta,P,g,E,ming,	&
      &               tau1,gamma,taugam)

      integer		:: n
      real			:: A(n,n)
      logical		:: phase1
      real			:: delta,g(n),E(n)
      integer		:: P(n)
      real			:: ming,tau1,gamma,taugam



      phase1 = .true.
      delta = 0.0
      ming = 0.0
      do i=1,n
         P(i)=i
         g(i)= 0.0
         E(i) = 0.0
	  end do	

!
!     Find the maximum magnitude of the diagonal elements.
!     If any diagonal element is negative, then phase1 is false.
!
      gamma = 0.0
      do i=1,n
         gamma=max(gamma,abs(A(i,i)))
         if (A(i,i) .lt. 0.0) phase1 = .false.
	  end do		

      taugam = tau1 * gamma

!
!     If not in phase1, then calculate the initial gerschgorin bounds
!     needed for the start of phase2.
!
	  if ( .not.phase1) call gersch(n,A,1,g)

      return
      end subroutine init
!***********************************************************************
!
!       subroutine name : gersch
!
!       purpose : Calculate the negative of the gerschgorin bounds
!                 called once at the start of phase II.
!
!       input   : ndim, n, A, j
!
!       output  : g - an n vector containing the negatives of the
!           Gerschgorin bounds.
!
!***********************************************************************
      subroutine gersch(n, A, j, g)
      integer			:: n, j
      real				:: A(n,n), g(n)
      integer			:: i, k
      real              :: offrow
      intrinsic abs

      do i = j, n
         offrow = 0.0
         do k = j, i-1
            offrow = offrow + abs(A(i,k))
		 end do	
         do k = i+1, n
            offrow = offrow + abs(A(k,i))
		 end do 	
            g(i) = offrow - A(i,i)
	  end do	

      return
      end subroutine gersch
!***********************************************************************
!
!  subroutine name : fin2x2
!
!  purpose : Handles final 2X2 submatrix in Phase II.
!            Finds eigenvalues of final 2 by 2 submatrix,
!            calculates the amount to add to the diagonal,
!            adds to the final 2 diagonal elements,
!            and does the final update.
!
!  input : n, A, E, j, tau2,
!          delta - amount added to the diagonal in the
!                  previous iteration
!
!  output : A - matrix with complete L factor in the lower triangle,
!           E - n*1 vector containing the amount added to the diagonal
!               at each iteration,
!           delta - amount added to diagonal elements n-1 and n.
!
!***********************************************************************
      subroutine fin2x2(n, A, E,tau2, delta,gamma)
      integer		 :: n
      real			 :: A(n,n), E(n), tau2, delta,gamma
      real			 :: t1, t2, t3,lmbd1,lmbd2,lmbdhi,lmbdlo
      real			 :: delta1, temp
      intrinsic sqrt, max, min

!
!     Find eigenvalues of final 2 by 2 submatrix
!
      t1 = A(n-1,n-1) + A(n,n)
      t2 = A(n-1,n-1) - A(n,n)
      t3 = sqrt(t2*t2 + 4.0*A(n,n-1)*A(n,n-1))
      lmbd1 = (t1 - t3)/2.
      lmbd2 = (t1 + t3)/2.
      lmbdhi = max(lmbd1,lmbd2)
      lmbdlo = min(lmbd1,lmbd2)
!
!     Find delta such that:
!     1.  the l2 condition number of the final
!     2X2 submatrix + delta*I <= tau2
!     2. delta >= previous delta,
!     3. lmbdlo + delta >= tau2 * gamma,
!     where lmbdlo is the smallest eigenvalue of the final
!     2X2 submatrix
!

      delta1=(lmbdhi-lmbdlo)/(1.0-tau2)
      delta1= max(delta1,gamma)
      delta1= tau2 * delta1 - lmbdlo
      temp = 0.0
      delta = max(delta, temp)
      delta = max(delta, delta1)

      if (delta .gt. 0.0) then
         A(n-1,n-1) = A(n-1,n-1) + delta
         A(n,n) = A(n,n) + delta
         E(n-1) = delta
         E(n) = delta
      end if
!
!     Final update
!
      A(n-1,n-1) = sqrt(A(n-1,n-1))
      A(n,n-1) = A(n,n-1)/A(n-1,n-1)
      A(n,n) = A(n,n) - (A(n,n-1)**2)
      A(n,n) = sqrt(A(n,n))

      return
      end subroutine fin2x2
!**********************************************************************
!
!     subroutine name : solve
!
!     purpose :  solves (LLtranspose)P(x)=P(b), where L is stored in
!                the lower triangle of A, P is the record of the
!                permutations  performed in forming L,b is the rhs vecto
!                and x is the result.
!                L is the result of the modified cholesky factorization
!                which computes Ptranspose(A+E)P=LLtranspose.
!
!  input:   ndim  - largest dimension of matrix that will be used
!
!           n     - dimension of matrix
!
!           A     - n*n array (only lower triangle portion,
!                   including the diagonal, is used).
!
!           P     - n*1 integer vector that contains a record
!                   of permutations performed in forming L.
!                   i.e. each Pi was initialized to i, and if
!                   row and columns i & j were interchanged when
!                   computing L, the values of Pi and Pj were swapped.
!
!           b     - n*1 double precision vector which is the
!                   right hand side of the system to be solved.
!
!           y,z   - n*1 double precision  work vectors.
!
!  output:  x     - n*1 double precision result vector
!
!
!**********************************************************************

      subroutine solve(n,A,P,x,b)
      integer			:: n
      integer			:: P(n)
	  integer			:: i,j	  
	  real			    :: A(n,n)
      real			    :: x(n),b(n)
	  real,allocatable,dimension(:) :: y,z
      real				:: sum
	  allocate(y(n),z(n))
!
!     Solve Ly=Pb
!
      y(1) = b(P(1))/A(1,1)
      do i=2,n
         sum=0.0
         do j=1,i-1
            sum=sum + (A(i,j)*y(j))
		 end do
         y(i) = (b(P(i)) - sum) / A(i,i)
	  end do	
!
!     Solve Ltranspose z = y
!
      z(n) = y(n)/A(n,n)
      do i=n-1,1,-1
         sum=0.0
         do j=i+1,n
            sum = sum + (A(j,i) * z(j))
		 end do
         z(i) = (y(i) - sum) / A(i,i)
	  end do	
!
!     x = Ptranspose z
!
      do i=1,n
         x(P(i))=z(i)
	  end do	
	  deallocate(y,z)
      return
      end subroutine solve

    !----------------------------------------------------------
    subroutine chol(a,n,r,ifail)
      implicit none
      integer i,j,k,ifail,n
      real (float)  a(n,n),r(n,n),h,s
      !
      ifail=0
      do    i=1,n
       h=a(i,i)
       do   j=1,i-1
        h=h-r(j,i)**2
       enddo
       if ( h .le. 0._float ) then
        ifail=1
        return
       endif
       h=sqrt(h)
       r(i,i)=h
       h=1._float/h
       do    k=i+1,n
        s=0._float
        do    j=1,i-1
         s=s+r(j,i)*r(j,k)
        enddo
        s=(a(k,i)-s)*h
        r(i,k)=s
       enddo
      enddo
    end subroutine chol
    !--------------------------------------------------------------
    subroutine cholso(a,n,b,x)
!
!    Solve ax  = b 
!
!
      implicit none
      integer i,j,n
      real (float)  a(n,n),b(n),c(n),x(n)
      real (float)  s
      !
      do   i=1,n
       s=0._float
       do    j=1,i-1
        s=s+a(j,i)*c(j)
       enddo
       c(i)=(b(i)-s)/a(i,i)
      enddo
      do    i=n,1,-1
       s=0._float
       do    j=i+1,n
        s=s+a(i,j)*x(j)
       enddo
       x(i)=(c(i)-s)/a(i,i)
      enddo
    end subroutine cholso
    !--------------------------------------------------------------

  subroutine scaling(initgd2)
!
! Choose a parameter scaling
! Currently scaling is sqrt(1.0/(abs(hessian)))
!  
  use domindat
  integer :: i
  real :: initgd2(n)
  do i=1,n
    	scal(i)=sqrt(1.0/abs(initgd2(i)))
  end do
  end subroutine scaling
  
  subroutine byscale(xd)
!
! Multiply by scale
!  
  use domindat
  integer :: i
  real :: xd(*)  
  do i=1,n
		xd(i)   = xd(i)*scal(i)
  end do
  end subroutine byscale
  subroutine divscale(xd)
!
! Divide by scale 
!  
  use domindat
  integer :: i
  real :: xd(*)
  do i=1,n
	xd(i)   = xd(i)/scal(i)
  end do
  end subroutine divscale

    

      subroutine domin_info(itestloc)
!*******************************************************************
!     info provides for intermediate output for 
!     testing purposes
!     info may have an empty body of course
!*******************************************************************
	  use domindat      
      integer itestloc,i

      open(10,file='domin.res',status='old',position='append')
	  select case(itestloc)
	  case(1)
! Iteration number, function value, norm(gradx)
			write(10,1100) niter,fx,accinf(niter,4)			
			write(10,*) ' x = '
			write(10,fmt='(1x,5(d21.14,2x))') (x(j),j=1,n)
			write(10,*) ((A(I,J),i=1,n),j=1,n)
			write(10,*) 'diagonal of updated quasi newton matrix'
			write(10,1200) (diaga(j),j=1,n)

	  case (2) 
! Norm, condition number, d=direction vector
			write(10,2100)    accinf(niter,5),accinf(niter,8)
			write(10,*) 'Direction d = '
			write(10,fmt='(1x,5(d21.14,2x))') (d(j),j=1,n)
	  case (3)
! Sigma, dskal,norm of d, no. of function calls
			write(10,3100) (accinf(niter,j),j=9,12)

	  case (4) 
! Update vectors z,y, theta2
			write(10,4100) thet2
			write(10,*) ' update vector z '
			write(10,4200) (z(i),i=1,n)
			write(10,*) ' update vector y '
			write(10,4200) (y(i),i=1,n)
	  case (5)	
! Restarts and updates
			write(10,5100) accinf(niter,13),accinf(niter,14)
	 	
	  case (6)
! Step size and fx
			write(10,6100) sig,ftest

	  case(7) 
! Trace a , b
      write(10,fmt='(1x,''trace a='',d15.7,'' trace b='',d15.7)') &
     &      tracea,traceb 
	
	   case(8)
! Output scaled and unscaled values
		write(10,'(1X,A)')      ' Scaled X,Gradx '
		write(10,'(5(1X,F8.3))')   (x(i),i=1,n)
		write(10,'(5(1X,F8.3))')   (gradx(i),i=1,n)
		write(10,'(1X,A)')      ' Unscaled X,Gradx '
		write(10,'(5(1X,F8.3))')   (x(i)*scal(i),i=1,n)
		write(10,'(5(1X,F8.3))')   (gradx(i)/scal(i),i=1,n)

	  case(9)
! Output inital hessian and scaling vector
	   write(10,*) 'Diagonal of Hessian and scaling vector'	
	   write(10,*) 	(grad2x(i),i=1,n)
	   write(10,*) 	(scal(i),i=1,n)
	  end select 
  close(10)
1100   format(1h0,'******************************************'   &
     &       /1x,i3,'-ter iteration step',						 &
             /1x,'f = ',d21.14,' norm(gradx)=',d21.14)
1200   format(1x,5(d21.14,2x))
2100		format(/'norm(a)=',d10.4,'  cond(a)=',d10.4)
3100   format(/,'sigma=',d10.4,'  dskal=',d10.4,' norm(d)=',d10.4,  &
     &     ' number of function calls=',d10.4)
4100		format(/,'parameter theta2 =',d10.4)
4200		format(1x,5(d21.14,2x))
5100		format(/'restart =',d8.1,' update = ',d8.1)
6100		format(1x,'step   sig = ',d10.4,'  f(x-sig*d)=',d21.14)

  end subroutine domin_info




  end module search1







!
! Used rosenbrock for testing Domin
!
  
!  MODULE ROSEN
!	INTERFACE
!		subroutine ftn(n,x,f,err) 
!			INTEGER :: n
!			REAL    :: f
!			REAL	:: x(*)
!			LOGICAL :: err
!		END SUBROUTINE ftn
!		subroutine gradf(n,x,g,g2,err) 
!			INTEGER :: n
!			REAL	:: x(*),g(*),g2(*)
!			LOGICAL :: err
!		END SUBROUTINE gradf
!	END INTERFACE
!   END MODULE ROSEN






!SUBROUTINE FTN (N,X,F,ERR)
!INTEGER    N
!REAL       X(N), F
!LOGICAL    ERR
!ERR = .FALSE.
!F = 1.0E2*(X(2)-X(1)*X(1))**2 + (1.0E0-X(1))**2
!RETURN
!END subroutine FTN


!SUBROUTINE GRADF (N, X, G,G2,ERR)
!INTEGER    N
!REAL       X(N), G(N),G2(N)
!LOGICAL    ERR
!ERR = .FALSE.
!G(1) = -4.0E2*(X(2)-X(1)*X(1))*X(1) - 2.0E0*(1.0E0-X(1))
!G(2) = 2.0E2*(X(2)-X(1)*X(1))
!G2(1) = -4.0E2*X(2) + 1.2E3*X(1)*X(1) + 2.0E0
!G2(2) = 2.0E2
!RETURN
!END SUBROUTINE GRADF









