# correct.tcl---corrections  manager (Energy/Intensity)

# Fri Jul 15 09:46:44 BST 1994, SCL
#   added check for OS version due to different output from ls


set started 0

set frame ENGCORR
set frametitle "Energy Corrections Manager"
set icontitle EngCorr

set server  /vme

set defdir [file join $env(MIDASBASE) meis config corrections]
set directory $defdir
set file ""
set savefile ""

set table(2D.Intensity.File)	{update-txt	INTENSITY-FILE	{}}

set table(INTENSITY-FILE)	{click-txt	2D.Intensity.File {} }

proc redisplay-registers {} {
# picks up 2D.*.Angle & 2D.*.File
#   frame-busy
   global EGreport
   set egr $EGreport
   set EGreport -1
   set pair [list 2D.Intensity.File [inform sys inquire intensity_file]]
   update-pair $pair
   set EGreport $egr
#   frame-idle
}

proc make-frame {} {
   global env

   :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/spectrum"  \
      :icon-name [self icon-title]

    :panel P1 :layout V :pady 5 :padx 10
     :layout H
      :text DIRECTORY :label "Directory" :w 35 :h 20 :shifters T
      :hskip 100
      :button UP :w 4; :button DOWN :w 4

    :panel P2 :layout V :padx 10
      :list DIRECTORYLIST :rows 6 :cols 70 :scroll B :exclusive T :double-click T

    :panel P3 :layout V :pady 10 :padx 10
     :layout H; :row-gap 15;
      :text INTENSITY-FILE :label "Current file:" :w 70 :deaf T 
      :next-row T
#      :message LOAD :label "Load Selected:" :bold T; :col-gap 20
      :button INTENSITY :label "Use Selected"; :col-gap 20
      :button EDIT :label "Edit..." :w 4
      :button REDISPLAY-ALL :label "Redisplay"; :hskip 20
    :show T
}

proc Edit_Corr {text}  {
	global env __editcorrdone

  :frame "[self frame-name].textedit" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
	  :icon-image "@$env(ICONHOME)/default" \
	  :icon-name "[self icon-title]"

	set __editcorrdone 0
  
	:panel TEXTEDITPANEL :layout T :padx 10 :pady 10
	  :layout H; :row-gap 5; :col-gap 20

	:galley TEXTEDIT :rows 30 :cols 80 :font fixed
	 :next-row T

	:button TEXTEDITDONE :label "Save file and finish"     :command Edit_Corr_done
	:button TEXTEDITQUIT :label "Quit without saving file" :command Edit_Corr_quit
	:show T

	:modify TEXTEDIT :strings "$text" :see 0

   vwait __editcorrdone

   if {$__editcorrdone == 1}  {
	   set text "[[:path TEXTEDIT].text get 1.0 end]"
   } else {
	   set text ""
   }

   destroy "[self frame-name].textedit"

   return $text
}

proc Edit_Corr_done {} {global __editcorrdone; set __editcorrdone 1}
proc Edit_Corr_quit {} {global __editcorrdone; set __editcorrdone 2}

proc edit_corr {filename} {
	set line     ""
	set contents ""

	set z [ catch {open $filename r} f ]
	if { $z != 0 } then {
		midas-warning "Unable to open file $filename\n$f"
		return
	} else {
	   foreach line [ split [ read $f ] \n ] {
		 append contents $line "\n" 
	   }
	   close $f
	}

	set contents [Edit_Corr $contents ]
 
	if {$contents == ""} then {clear-footer;return}
   
	set z [ catch {open $filename w} f ]
	if {$z != 0} then {
		midas-warning "Unable to write file $filename\n$f"
		return
	} else {
		puts $f "$contents"
	}
	close $f
	set contents ""
	set-footer "$filename updated"

}

proc double-click arg {
   global debug
   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}
   set w [lindex $arg 2]

   switch -glob $w {
        DIRECTORYLIST        {click-down}
	default     {puts stdout $arg}
   }
}

proc get-file {df} {
# df is the type - directory or file
   global selection directorylist directory file
#   clear-footer
   if {[llength $selection] == 0} then {
      set-footer "no corrections $df selected!!!"
      return
   }
   if {[llength $selection] > 1} then {
      set-footer "multiple selections!!!"
      return
   }
   set target [lindex $directorylist $selection]
   set link [last $target]
   set fullfile [file join $directory $link]
   set type [file type $fullfile]
   if {$type == "link"} then {
      set link [file readlink $link]
      set fullfile [file join $directory $link]
      set type [file type $fullfile]
   }
   set file [last $target]
   if {$df == "file" && $type == "directory"} then {
      set-footer "can't use a directory!!!"
      after 10000 clear-footer
      return ""
   }
   if {$df == "directory" && $type == "file"} then {
      set-footer "can't move up or down to a file!!!"
      after 10000 clear-footer
      return ""
   }
   if {"$file" == "." || "$file" == ".."} {return $file} {return $fullfile}
}

proc do-load {t} {
   global selection directorylist directory file
   clear-footer
   set loadfile [get-file file]
   if {"$loadfile" == ""} then {return}
   inform sys set intensity_file $loadfile
   redisplay-registers
}

proc click-intensity {} {do-load INTENSITY}

proc click-edit {} {
   global selection directorylist directory file
   clear-footer
   set editfile [get-file file]
   if {"$editfile" == ""} then {return}

   set-footer "Editing $editfile"
   if {![file exists $editfile]} then {
	display-error-report self "ERROR: Failed to edit file" "File does not exist:" $editfile
	error 1
   }
   if {![file isfile $editfile]} then {
	display-error-report self "ERROR: Failed to edit file" "Bad file name:" $srdir/$srfile
	error 2
   }
#   exec textedit $editfile &
   edit_corr $editfile
}

proc click-down {} {
   global selection directorylist directory file

   clear-footer
   set link [get-file directory]
   if {"$link" == ""} then {return}
   if {"$link" == "."} then {click-redisplay; return}
   if {"$link" == ".."} then {click-up; return}
#   if {$directory == "/"} then {set directory ""}
   set directory $link
   click-redisplay
   set selection {}
}

proc click-redisplay-all {} {
   click-redisplay
   redisplay-registers
}

proc start args {
   global defdir started
    if {$started == 0} then {
	make-frame
	set z [catch {:file mkdir $defdir}]
	set started 1
    } else {open-frame}
   click-redisplay-all
}
