#    MEIS Control System
#        June 1996   TK version VP


set frame ADC
set frametitle "CAEN Mod V.419 4 Channel ADC"
set icontitle ADC

set server /aq
set ganging 1

set rtptable {2us 4us 8us 16us 32us 64us 128us 256us
              512us 1ms 2ms 4ms 8ms 16ms 32ms}
set modetable {"Auto-Trig" "Ext-TTL" "Software" "Self-Test"}

set type(DIGITAL) 1
set type(RTP)  2
set type(MODE)  3
set type(SLIDER)  4
set adclist {A B C D}
for {set i 0} {$i<4} {incr i} {
	set adc [lindex $adclist $i]

	set regname(ENABLE-$adc) Enable
	set regname(RTP-$adc)    RiseTimeProtection
	set regname(MODE-$adc)   LinearGateMode
	set regname(HIGH-$adc)   HighThreshold
	set regname(LOW-$adc)    LowThreshold
	set regname(RESET-$adc)  DataReset
	set regname(READY-$adc)  Ready

	set chaname(ENABLE-$adc) $adc
	set chaname(RTP-$adc)    $adc
	set chaname(MODE-$adc)   $adc
	set chaname(HIGH-$adc)   $adc
	set chaname(LOW-$adc)    $adc
	set chaname(RESET-$adc)  $adc
	set chaname(READY-$adc)  $adc

	set widgetname($adc.Enable)             ENABLE-$adc
	set widgetname($adc.RiseTimeProtection) RTP-$adc
	set widgetname($adc.LinearGateMode)     MODE-$adc
	set widgetname($adc.HighThreshold)      HIGH-$adc
	set widgetname($adc.LowThreshold)       LOW-$adc
	set widgetname($adc.DataReset)          RESET-$adc
	set widgetname($adc.Ready)              READY-$adc

	set widgettype(ENABLE-$adc) $type(DIGITAL)
	set widgettype(RTP-$adc)    $type(RTP)
	set widgettype(MODE-$adc)   $type(MODE)
	set widgettype(HIGH-$adc)   $type(SLIDER)
	set widgettype(LOW-$adc)    $type(SLIDER)
	set widgettype(RESET-$adc)  $type(DIGITAL)
	set widgettype(READY-$adc)  $type(DIGITAL)
}
         
proc click-help {} {inform mb goto {ADC Setup}}

proc make-reg-name {chan reg} {
   global server ganging
	if {$ganging} then {
		return $server/ADC.?.$reg
	} else {
		return $server/ADC.$chan.$reg
	}
}

proc write-reg {c r v} {

	set rr [make-reg-name $c $r]
	EG "register :pattern '$rr' :value '$v'"
}

proc click-trigger {} {
	global server
	EG "register :name '$server/ADC.Trigger' :value '1'"   
}

proc click-reset {} {
	global server

	EG "register :name '$server/ADC.Reset' :value '1'"  
}

proc click-on {w v} {
	global server chaname regname widgettype type
	set reg $server/ADC.$chaname($w).$regname($w)
	set t $widgettype($w)
	set val $v

	if {$t==$type(SLIDER)} then {

		set val [EG "register :name '$reg' ?value"]  
		set oldv [scan-int $val]
		if {$v==$oldv+1} then {set v [expr $oldv+16]}
		if {$v==$oldv+2} then {set v [expr $oldv+16]}
		if {$v==$oldv-1} then {set v [expr $oldv-16]}
		if {$v==$oldv-2} then {set v [expr $oldv-16]}
		set val "$v mV"
	}
	set reg [make-reg-name $chaname($w) $regname($w)]

	EG "register :pattern '$reg' :value '$val'"   
	click-redisplay
}

proc click-redisplay {} {
	global server

	set list [EG "register :pattern '$server/ADC*' ?names-and-values"]  
	refresh-frame $list
}

proc click arg {
   global debug ganging
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {finish; return}
   switch $w {
      REDISPLAY {click-redisplay}
      RESET     {click-reset}
      TRIGGER   {click-trigger}
      GANG      {set ganging $v}
      HELP      {click-help}
      default   {click-on $w $v}
   }
}


proc make-frame {} {
	global rtptable modetable

	:frame [self frame-name] :label "[self frame-title]" :resize F  :show-footer T \
		:icon-name "[self icon-title]"

	:panel P :padx 5 :pady 5
        :layout H; :col-gap 10; :row-gap 10

	:message M1 :label "Enable" :bold T
	:message M2 :label "Rise Time" :bold T
        :hskip 10
	:message M3 :label "Gate Mode" :bold T
        :hskip 50
	:message M4 :label "Low Threshold (mV)" :bold T
        :hskip 80
	:message M5 :label "High Threshold (mV)" :bold T
        :hskip 50
	:message M6 :label "Reset" :bold T
        :hskip -5
	:message M7 :label "Ready" :bold T
	:next-row T
	:checkbox     ENABLE-A :label "A:"
	:choice-stack RTP-A   :label "" :strings $rtptable
	:choice-stack MODE-A  :label "" :strings $modetable
	:slider       LOW-A   :label "" :min 0 :max 4096 :resolution 1
	:slider       HIGH-A  :label "" :min 0 :max 4096 :resolution 1
	:checkbox     RESET-A :label ""
	:checkbox     READY-A :label ""
	:next-row T
	:checkbox     ENABLE-B :label "B:"
	:choice-stack RTP-B :label "" :strings $rtptable
	:choice-stack MODE-B :label "" :strings $modetable
	:slider       LOW-B  :label "" :min 0 :max 4096 :resolution 1
	:slider       HIGH-B :label "" :min 0 :max 4096 :resolution 1
	:checkbox     RESET-B :label "" 
	:checkbox     READY-B :label ""
	:next-row T
	:checkbox     ENABLE-C :label "C:"
	:choice-stack RTP-C :label "" :strings $rtptable
	:choice-stack MODE-C :label "" :strings $modetable
	:slider       LOW-C  :label "" :min 0 :max 4096 :resolution 1 
	:slider       HIGH-C :label "" :min 0 :max 4096 :resolution 1
	:checkbox     RESET-C :label ""
	:checkbox     READY-C :label "" 
	:next-row T
	:checkbox     ENABLE-D :label "D:"
	:choice-stack RTP-D :label "" :strings $rtptable
	:choice-stack MODE-D :label "" :strings $modetable
	:slider       LOW-D  :label "" :min 0 :max 4096 :resolution 1
	:slider       HIGH-D :label "" :min 0 :max 4096 :resolution 1
	:checkbox     RESET-D :label "" 
	:checkbox     READY-D :label ""
	:next-row T
	:button REDISPLAY
	:button RESET
	:button TRIGGER
        :hskip 50
	:choice GANG  :label "Gang ADCs:"  :strings [list Off On]
        :hskip 100
	:button HELP

   :show T
}

proc refresh-frame list {
   global debug ganging widgetname

   foreach pair $list {
      if {[catch {set w $widgetname([strip-to-dot [first $pair]])}]==0} then {

         :modify $w :value [scan-int [second $pair]]
      }
   }
   :modify GANG :value $ganging
}
