# saving spectra---aqs.tcl

set aqsindex 0
set aqstime 0
set aqsserverlist {disc}
set aqsserver [lindex $aqsserverlist $aqsindex]
set aqscomment {}

# also used by aqw.tcl, should be initialised in sys.meis.tcl
if {![info exists intensity_file]} then {
  set intensity_file [file join $env(MIDASBASE) meis config corrections energy.default.dat]
}

proc aqs-report c {set-footer "total of $c spectra successfully saved"}

proc click-aqs-save {} {
	global SpectrumServer aqsserver aqsdir aqscomment 
	global intensity_file 
	set c 0
	set z 0

	set aqspath /$aqsserver/$aqsdir
	set endpath [string index $aqspath [expr {[string length $aqspath] - 1}]]
	if {$endpath != "." && $endpath != "/"} then {append aqspath /}

	foreach s [get-spectrum-list] {
		if {$s=="2d.data"} then {
			set-footer "saving /$SpectrumServer/$s and correcting intensity..."
			set z [catch {exec correctintensity /$SpectrumServer/$s $aqspath$s $intensity_file} m]
		} else {
			set-footer "saving /$SpectrumServer/$s..."
			set z [catch {exec copy-spectrum /$SpectrumServer/$s $aqspath$s} m]
		}
		set-footer $m
		if {$z} then {
			break
		} else {
			if {$aqscomment!={}} then {
				EG "spectrum :name '$aqspath$s' :label '$aqscomment'"
			}
			incr c
		}
	}
	if {$z==0} then {after 2000 aqs-report $c}
}

proc find-resource {ps res} {
   set l {}
   set role {}; set name {}; set host {}
   foreach p $ps {
      switch [first $p] {
         name {set name [second $p]}
         role {set role [second $p]}
         host {set host [second $p]}
         default {} 
         }
      }
   if {$role==$res}  then  {set l "$name"}
   return $l
}

proc make-resource-list {rsl res} {
   set ll {}
   foreach r $rsl {set l [find-resource $r $res]; if {$l!={}} then {
      lappend ll $l}}
   return $ll
}

proc click-aqs-start {} {
	global aqsformat aqsdir aqsserver aqsserverlist aqsindex SpectrumServer aqscomment

   make-aqsframe

#   if {[catch {exec egresources | tr "()" "{}"} m]} then {
#	set-footer "no experiment connected!!!"
#   } else {
#	set aqsserverlist [make-resource-list $m disc]
#   }

   set aqsserverlist [make-resource-list [:experiment resources] disc]

   set aqsserver [lindex $aqsserverlist $aqsindex]
   set aqscomment {}

   :modify AQS-SERVER :strings $aqsserverlist :value $aqsindex

#	set aqsdir specs/meis/run-[run-num-read]([date +%d%h%y-%H:%M])
   set-default-dir
   aqs-repaint
}

proc aqs-repaint {} {
   global aqsdir aqstime aqsserver aqsindex aqscomment

   :modify AQS-SERVER  :value $aqsindex
   :modify AQS-TIME    :value $aqstime
   :modify AQS-COMMENT :value "$aqscomment"
   :modify AQS-DIR     :value $aqsdir
}

proc click-aqs-server index {
   global aqsserver aqsserverlist aqsindex
   set aqsindex $index
   set aqsserver [lindex $aqsserverlist $index]
   aqs-repaint
}

proc click-aqs-dir dir {
   global aqsdir
   set aqsdir $dir
   aqs-repaint
}

proc click-aqs-comment comment {
   global aqscomment
   set aqscomment $comment
   aqs-repaint
}

proc set-default-dir {} {
   global aqstime aqsdir aqscomment
   if {$aqstime} then {
        set aqsdir specs/meis/run[run-num-read]([date +%d%h%y-%H:%M]).
   } else {
	set aqsdir specs/meis/run[run-num-read].
   }
}

proc click-aqs-time on {
   global aqstime
   set aqstime $on
   set-default-dir
   aqs-repaint
}

proc click-aqs arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   switch $w {
      AQS-SAVE          {click-aqs-save}
      AQS-SERVER        {click-aqs-server $v}
      AQS-DIR           {click-aqs-dir $v}
      AQS-TIME          {click-aqs-time $v}
      AQS-COMMENT       {click-aqs-comment $v}
      default          {puts stdout $arg}
   }
}

proc make-aqsframe {} {
  global aqsindex aqsserverlist aqstime

  :frame [self frame-name].aqs :label "Save Spectra" :show-footer T :resize F :transient T

  :panel AQS-PANEL :padx 5 :pady 5
  :layout H; :col-gap 10; :row-gap 10
  :choice-stack AQS-SERVER :exclusive T :valuex 10c :value $aqsindex  \
          :label Server:  :strings $aqsserverlist
  :hskip 20
  :checkbox AQS-TIME :label "Time Stamp:" :value $aqstime
  :button AQS-SAVE :label Save
  :next-row T
  :text AQS-DIR :label Path: :valuex 10c :w 35 :value "specs/meis"
  :next-row T
  :text AQS-COMMENT :label Comment: :valuex 10c :w 35 :value ""
# :focus T

#  centre-frame $aqsframe
#  :modify  $aqsframe :transient T
  :show T
}
