# saving window when data acquisition complete---aqw.tcl


set aqwindex 0
set aqwtime 0
set aqwserverlist {disc}
set aqwserver [lindex $aqwserverlist $aqwindex]
set aqwcomment {}

# also used by aqs.tcl, should be initialised in sys.meis.tcl
if {![info exists intensity_file]} then {
  set intensity_file [file join $env(MIDASBASE) meis config corrections energy.default.dat]
}

set aqwsavefull 0
set aqwsavetac 1

set aqwdatafrom 2d.data
set aqwdatato   2d.data

set aqwfullfrom 2d.full
set aqwfullto   2d.full

set aqwtacfrom TAC
set aqwtacto   TAC

proc aqw-report to {set-footer "Data Saved to $to"}

proc click-aqw-save {} {
	global SpectrumServer aqwserver aqwdir aqwcomment aqwdatafrom
	global intensity_file 
	set c 0
	set z 0

	set aqwfrom /$aqwdatafrom
	set aqwpath /$aqwserver/$aqwdir

	set-footer "saving /$SpectrumServer/$aqwfrom and correcting intensity..."
	set z [catch {exec correctintensity /$SpectrumServer/$aqwfrom $aqwpath $intensity_file} m]
	set-footer $m
	if {$z==0} then {
		if {$aqwcomment!={}} then {
			EG "spectrum :name '$aqwpath' :label '$aqwcomment'"
		}
		after 2000 aqw-report $aqwpath
	}
	clear-busy
}

proc aqw-start-frame {} {
   global aqwdatato aqwformat aqwdir aqwserver aqwserverlist aqwindex SpectrumServer aqwcomment

   set-busy
   make-aqwframe
#   if {[catch {exec egresources | tr "()" "{}"} m]} then {
#	set-footer "no experiment connected!!!"} else {
#	set aqwserverlist [make-resource-list $m disc]}

   set aqwserverlist [make-resource-list [:experiment resources] disc]
   set aqwserver [lindex $aqwserverlist $aqwindex]
   set aqwcomment {}

   :modify AQW-SERVER :strings $aqwserverlist :value $aqwindex

   aqw-default-dir $aqwdatato
   aqw-repaint
}

proc aqw-repaint {} {
   global aqwdir aqwtime aqwdatafrom aqwindex aqwcomment

   :modify AQW-SERVER  :value $aqwindex
   :modify AQW-TIME    :value $aqwtime
   :modify AQW-COMMENT :value "$aqwcomment"
   :modify AQW-FROM    :value $aqwdatafrom
   :modify AQW-DIR     :value $aqwdir
}

proc aqw-auto-save-file {aqwfromto} {
   global  aqwdir aqwserver aqwserverlist aqwindex server aqwcomment intensity_file SpectrumServer
   global activelist

   set-busy
   set aqwserverlist [make-resource-list [:experiment resources] disc]
   set aqwserver [lindex $aqwserverlist $aqwindex]
   set aqwcomment {}

   foreach sp $aqwfromto {
      set aqwfrom $sp
      set aqwto $sp

      aqw-default-dir $aqwto

      set c 0
      set z 0

      set aqwpath $aqwdir
      if {$aqwserver!=""} then {set aqwpath /$aqwserver/$aqwdir}

      if {[lsearch $activelist $aqwfrom] != -1} then {
         set-footer "saving /$SpectrumServer/$aqwfrom and correcting intensity..."
         set z [catch {exec correctintensity /$SpectrumServer/$aqwfrom $aqwpath $intensity_file} m]
      } else {
         set-footer "saving /$SpectrumServer/$aqwfrom..."
         set z [catch {exec copy-spectrum /$SpectrumServer/$aqwfrom $aqwpath} m]
      }

      set-footer $m
      if {$z==0} then {
         if {$aqwcomment!={}} then {
            EG "spectrum :name '$aqwpath' :label '$aqwcomment'"
         }
            after 2000 aqw-report $aqwpath
      }
   }
   clear-busy
}

proc aqw-auto-save {} {
   global activelist; # "imported" from sys.meis.tcl
   global aqwdatafrom aqwdatato
   global aqwsavefull aqwfullfrom aqwfullto
   global aqwsavetac aqwtacfrom aqwtacto
   aqw-auto-save-file $activelist
   if {$aqwsavefull} {aqw-auto-save-file $aqwfullfrom}
   if {$aqwsavetac} {
  global VMEserver server

set server $VMEserver

if {[read-reg ERDA.Enable]} {aqw-auto-save-file $aqwtacfrom}
   }
}

proc click-aqw-server index {
   global aqwserver aqwserverlist aqwindex
   set aqwindex index
   set aqwserver [lindex $aqwserverlist $index]
   aqw-repaint
}

proc click-aqw-dir dir {
   global aqwdir
   set aqwdir $dir
   aqw-repaint
}

proc click-aqw-comment comment {
   global aqwcomment
   set aqwcomment $comment
   aqw-repaint
}

proc click-aqw-from from {
   global aqwdatafrom
   set aqwdatafrom $from
   aqw-repaint
}

proc aqw-default-dir aqwto {
   global aqwtime aqwdir aqwcomment
   if {$aqwtime} then {
	set aqwdir specs/meis/run[run-num-read]([date +%d%h%y-%H:%M]).$aqwto
   } else {
	set aqwdir specs/meis/run[run-num-read].$aqwto
   }
}

proc click-aqw-time on {
   global aqwtime aqwdatato
   set aqwtime $on
   aqw-default-dir $aqwdatato
   aqw-repaint
}

proc click-aqw arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   switch $w {
	AQW-SAVE          {click-aqw-save}
	AQW-SHOW          {click-aqw-show}
	AQW-SERVER        {click-aqw-server $v}
	AQW-DIR           {click-aqw-dir $v}
	AQW-TIME          {click-aqw-time $v}
	AQW-COMMENT       {click-aqw-comment $v}
	AQW-FROM          {click-aqw-from $v}
	default           {puts stdout $arg}
   }
}

proc make-aqwframe {} {
  global aqwindex aqwserverlist aqwtime aqwdatafrom

   :frame [self frame-name].aqw :label "Data Acquisition Complete" :transient T
   :panel AQW-PANEL :padx 5 :pady 5
   :layout H; :col-gap 10; :row-gap 10
   :choice-stack AQW-SERVER :exclusive T :valuex 12c :value $aqwindex  \
       :label Server:  :strings $aqwserverlist
   :checkbox AQW-TIME :label "Time Stamp:" :valuex 12c :value $aqwtime
   :button AQW-SAVE :label Save
   :next-row T
   :text AQW-FROM :label Save: :valuex 12c :w 35 :value "$aqwdatafrom"
   :next-row T
   :text AQW-DIR  :label To:   :valuex 12c :w 35 :value "specs/meis"
   :next-row T
   :text AQW-COMMENT :label Comment: :valuex 12c :w 35 :value ""
# :focus T

 :show T
}
