#    Meis Detector PSU (ramping version) control
#      June 1996   TK version  VP
#      August 2001   SCL based on dps.tcl (as above)

set frame DPSR
set frametitle "Detector Power Supply Ramp"
set icontitle "MCP Ramp"

set server /VME

#
#                      proc          register                       args
#
set table(VOLTS-REQ)   {click-slider Detector.Volts.Request         {1 1 V }}
set table(VOLTS-TXT)   {click-null   Detector.Volts.Request         {1 1 V }}
#
#                                   proc            widgets      args
#
set table(Detector.Volts)           {update-psu     VOLTS        {}}
set table(Detector.Volts.Request)   {update-slider  VOLTS-REQ     1}

# multipliers to convert seconds/minutes/hours/days to seconds
set multipliers [list 1 60 [expr 60 * 60] [expr 24 * 60 * 60]]
set multiplier [second $multipliers]
set threshold 1200
set ramptime 10
set rampstep 1
set going 0
# Always moves at 10 seconds/step
set steptime 10
set final 0
set latest 0
set current 0

proc continue-ramp {} {
   global rampafterid going
   global latest current final rampstep steptime

   if {$rampafterid!=""} then {catch {kill-after $rampafterid}}
   if {$going} then {
      set rampafterid [after [expr $steptime * 1000] continue-ramp]
      set-footer "MCP Voltage Ramping in Progress"
      click-dpsr-redisplay
# insert-debug "continue latest: $latest"
      set new [expr $latest + $rampstep]
      if {$new > $final} then {set new $final}
# insert-debug "continue new: $new"
      click-on VOLTS-REQ [expr round($new)]
      if {$new == $final} then {set going 0}
      set latest $new
# insert-debug "continue current now: $current"
# insert-debug "continue latest now: $latest"
# insert-debug "continue done"
   } else {
      click-redisplay
      set-footer "MCP Voltage Ramping Complete"
   }
}

proc click-final v {
   global final
   set final $v
}

proc click-threshold v {
   global threshold
   set threshold $v
}

proc click-time t {
   global ramptime
   set ramptime $t
}

proc click-multiplier u {
   global multiplier multipliers
   set multiplier [lindex $multipliers $u]
}

proc click-stopgo v {
   global rampafterid going
   global latest current final threshold multiplier ramptime rampstep steptime 

   if {$v==$going} then {return}

   if {$v} then {
      click-dpsr-redisplay
# insert-debug "stopgo current: $current V"
# insert-debug "stopgo threshold: $threshold V"
# insert-debug "stopgo final: $final V"
      if {$final < $threshold || $final < $current} then {
         click-on VOLTS-REQ $final
         return
      }

      if {$current < $threshold} then {
         click-on VOLTS-REQ $threshold
# insert-debug "current now: $current V"
         set pause $steptime
      } else {
         set pause 0
      }


      set latest [expr $current.0]
      set period [expr $ramptime * $multiplier]

# insert-debug "total period: $period seconds"
      set steps [expr $period / $steptime]

      if {$steps < 1} then {set steps 1}
# insert-debug "No. steps: $steps"
      set interval [expr $final - $latest]
# insert-debug "interval: $interval V"
      set rampstep [expr $interval / $steps]
      if {$rampstep < 1} then {set rampstep 1}
# insert-debug "ramp per step: $rampstep"

      set going 1
      set rampafterid [after [expr $pause * 1000] continue-ramp]
   } else {
      set going 0
      continue-ramp
   }
}

proc click-help {} {inform mb goto {XVME-560 Setup}}

proc update-psu {w v a} {
   :modify $w-TXT :value "$v"
   :modify $w-GGE :value [scan-int $v]
}

proc click-dpsr-redisplay {} {
   global going current
   :modify STOPGO :value $going
   set list [read-regs D*.Volts*]
   foreach pair $list {update-pair $pair}
   set current [scan-int [second [lindex $list [lsearch -regexp $list Request]]]]
}

proc click-redisplay {} {
   click-dpsr-redisplay
   send dps clear-footer
   send dps click-redisplay
}

proc click arg {
   global debug 
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      VOLTS-END	{click-final $v}
      THRESHOLD	{click-threshold $v}
      TIME	{click-time $v}
      UNITS	{click-multiplier $v}
      STOPGO	{click-stopgo $v}
      REDISPLAY	{click-dpsr-redisplay}
      HELP	{click-help}
      default   {click-on $w $v}
   }
}

proc make-frame {} {
    global threshold multiplier multipliers ramptime
    set ind [lsearch $multipliers $multiplier]

    set gw 110

   :frame [self frame-name] :label "[self frame-title]" :resize F :show-footer T  \
       :icon-name "[self icon-title]"

   :panel P :padx 5 :pady 5
   :layout H; :row-gap 10; :col-gap 10
   :number  TIME  :label "Ramping Time :" :w 3 :valuex 15c :bold T :value $ramptime
   :choice  UNITS  :label "Units :" :strings [list "Seconds" "Minutes" "Hours" "Days"] :exclusive T :bold T :value $ind
   :row-gap 15; :next-row T
   :slider  THRESHOLD  :label "Threshold (V) :" :min 0 :max 3500 :valuex 15c :entry-width 7 :bold T :resolution 1 :value $threshold
   :row-gap 10; :next-row T
   :slider  VOLTS-END  :label "End Voltage (V) :" :min 0 :max 3500 :valuex 15c :entry-width 7 :bold T :resolution 1
   :choice  STOPGO  :strings [list "Stop" "Go"] :exclusive T :bold T
   :row-gap 20; :next-row T
   :slider  VOLTS-REQ  :label "Set Voltage (V) :" :min 0 :max 3500 :valuex 15c :entry-width 7 :bold T :deaf T :resolution 1
   :row-gap 10; :next-row T
   :text   VOLTS-TXT :w 7 :deaf T :label "Monitor :" :valuex 15c :bold T
   :gauge  VOLTS-GGE :label "" :min 0 :max 3500 :max-tick-label 3.5kV :ticks 8 :gauge-width $gw
   :next-row T
   :button REDISPLAY
   :button HELP

   :show T
}
