# erda.tcl
#
# This code was adcconf then part of operator
# it has now been extracted from operator.tcl
# This code is used to assign the ADC channel for the ERDA TAC
# and to set the ADC pedestal for that channel
# It is also used set the ERDA TAC window Base and Range,
# the scale of the TAC spectrum and the mass of the target atom
# (for automatically changing the window base during an energy scan)
#

set frame ERDA
set frametitle "ERDA Configuration"
set icontitle ERDA

set server /VME

set dt_fix 100

set table(TAC_VAL)     {click-tac       T        {}}

set table(PEDESTAL_T)  {click-slider v785n#1.Pedestal16    {1 1}}

set table(ENABLE_T)    {click-enable v785n#1.Disable16   {}}

set table(ERDAMODE)    {click-erda ERDA.Enable {}}

set table(WINDOW1_ENABLE)    {click-int ERDA.TAC.Window1.Enable {}}
set table(WINDOW2_ENABLE)    {click-int ERDA.TAC.Window2.Enable {}}
set table(WINDOW3_ENABLE)    {click-int ERDA.TAC.Window3.Enable {}}

set table(TAC_SCALE)        {click-scale  ERDA.TAC.Scale        {1 1}}
set table(TACWINDOW1_BASE)  {click-slider ERDA.TAC.Window1.Base  {1 1 ns}}
set table(TACWINDOW1_RANGE) {click-slider ERDA.TAC.Window1.Range {1 1 ns}}
set table(TACWINDOW2_BASE)  {click-slider ERDA.TAC.Window2.Base  {1 1 ns}}
set table(TACWINDOW2_RANGE) {click-slider ERDA.TAC.Window2.Range {1 1 ns}}
set table(TACWINDOW3_BASE)  {click-slider ERDA.TAC.Window3.Base  {1 1 ns}}
set table(TACWINDOW3_RANGE) {click-slider ERDA.TAC.Window3.Range {1 1 ns}}

# The repeat of TAC.Channel is a bugfix hack until I do it better
# they refer to the same register
set table(v785n#1.TAC.Channel)         {update-tac       T {}}
set table(ERDA.TAC.Channel)            {update-tac       T {}}

set table(v785n#1.Disable16)     {update-enable    ENABLE_T {}}

set table(v785n#1.Pedestal16)      {update-slider PEDESTAL_T        1}

set table(ERDA.TAC.Window1.Enable)  {update-int WINDOW1_ENABLE {}}
set table(ERDA.TAC.Window2.Enable)  {update-int WINDOW2_ENABLE {}}
set table(ERDA.TAC.Window3.Enable)  {update-int WINDOW3_ENABLE {}}

set table(ERDA.TAC.Scale)         {update-slider TAC_SCALE   1}
set table(ERDA.TAC.Window1.Base)  {update-slider TACWINDOW1_BASE    1}
set table(ERDA.TAC.Window1.Range) {update-slider TACWINDOW1_RANGE   1}
set table(ERDA.TAC.Window2.Base)  {update-slider TACWINDOW2_BASE    1}
set table(ERDA.TAC.Window2.Range) {update-slider TACWINDOW2_RANGE   1}
set table(ERDA.TAC.Window3.Base)  {update-slider TACWINDOW3_BASE    1}
set table(ERDA.TAC.Window3.Range) {update-slider TACWINDOW3_RANGE   1}

proc click-enable {r v a} {
global EGallow
#   inform v785n click-adc-widget ENABLE$r $v
  click-int $r [expr $v^1] $a
  set started [inform v785n inquire started]
  if {$started == 0} then {
    click-redisplay
  } else {
    inform v785n click-redisplay; # NB 785n's redisplay calls back to operator
  }
}

proc update-enable {w v a} {
  update-int $w [expr $v^1] $a
}

proc click-erda {r v a} {
  global EGreport
  set egreport $EGreport
  set EGreport -1
  set ov [read-reg $r]
  if {$ov == $v} then {return}
  click-int $r $v $a
  click-on ENABLE_T $v
  if {$v} then {
     inform sys create_erda_spectra
  } else {
     inform sys delete_erda_spectra
  }
  set EGreport $egreport
  inform sys redisplay-state
#  if {[is-started operator]} {inform operator click-redisplay}
  click-redisplay
}

proc click-tac {r v a} {
global EGallow
  set v [expr $v * 2]
  click-int ERDA.TAC.Channel $v $a
  click-redisplay
}

proc update-tac {w v a} {
  update-int  ADC_FOR_$w $v $a
  update-int  TAC_VAL [expr $v / 2] $a
}

proc setup-scale {} {
   click-on TAC_SCALE [read-reg ERDA.TAC.Scale]
}

proc click-scale {r v a} {
  global EGreport initialise
  if {[scan-num $v] == 0} {set initialise 1; return}
  set egreport $EGreport
  set EGreport -1
  click-slider $r $v $a
  for {set w 1} {$w <= 3} {incr w} {
    set b [read-reg ERDA.TAC.Window$w.Base]
    set r [read-reg ERDA.TAC.Window$w.Range]
    :modify TACWINDOW[set w]_BASE :max $v
    :modify TACWINDOW[set w]_RANGE :max $v
    write-attributes ERDA.TAC.Window$w.Base -r0x0:${v}ns,0xfff:0ns
    write-attributes ERDA.TAC.Window$w.Range -r0x0:0ns,0xfff:${v}ns
    write-reg ERDA.TAC.Window$w.Base $b
    write-reg ERDA.TAC.Window$w.Range $r
  }
  set EGreport $egreport
  if {[read-reg ERDA.Enable]} {
    inform sys label-erda-tac-spectrum
  }
}

proc click-saverestore v {
   if {$v == "save"} {inform vmesaverestore click-SSRsave}
   if {$v == "restore"} {inform vmesaverestore click-SSRrestore-current;setup-scale;click-redisplay}
}

set initialise 1

proc click-redisplay {} {
    global table debug initialise

if {$debug} then {insert-debug "click-redisplay"}
    
    set tacchan [first [read-regs ERDA.TAC.Channel]]
       update-pair $tacchan
       set corner T
       set chan [second $tacchan]

       set old_reg [second $table(PEDESTAL_$corner)]
       catch {unset table($old_reg)}
       set table(v785n#1.Pedestal$chan)   [list update-slider PEDESTAL_$corner  1]
       set table(PEDESTAL_$corner)        [list click-slider v785n#1.Pedestal$chan     {1 1}]

       set old_reg [second $table(ENABLE_$corner)]
       catch {unset table($old_reg)}
       set table(v785n#1.Disable$chan)   [list update-enable ENABLE_$corner  {}]
       set table(ENABLE_$corner)        [list click-enable v785n#1.Disable$chan {}]

#   ERDA.*Window* picks up both window enables & tac parameters
    set tacwins [read-regs ERDA.*Window*]
    foreach tacwin $tacwins {
       update-pair $tacwin
    }

    set tac [first [read-regs ERDA.TAC.Scale*]]
    update-pair $tac
if {$initialise} {set initialise 0;setup-scale}

    set enables [read-regs v785n#1.Disable*]
    foreach enable $enables {
       update-pair $enable
    }

    update-int ERDAMODE [read-reg ERDA.Enable] ""
    update-int SAVETAC [inform sys inquire aqwsavetac] ""

if {$debug} then {insert-debug "click-redisplay done"}
}

proc click-save-tac v {
   inform sys set aqwsavetac $v
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {finish; return}
   switch $w {
      SAVETAC  {click-save-tac $v}
      SAVERESTORE {click-saverestore [string tolower $v]}
      REDISPLAY {click-redisplay}
      HELP      {click-help}
      default   {click-on $w $v}
   }
}

# calculate delta t for mass m and adjacent energy tiles (i.e. 1 energy step)
# (this isn't called, use the generic version, for any number of steps)
proc calc-dt {m} {

set E [split [read-reg Analyser.Energy] k]

set u [second $E]
if {$u != "eV"} then {
   midas-report "Energy units not keV.\nNew gate base time will not be set."
   return
}

set E1 [first $E]
# calculate delta t in ns
set dt [expr -4.093 * sqrt($m / $E1)]
}

# calculate delta t for mass m and number of energy steps s
proc calc-dtn {m s} {
  set E [split [read-reg Analyser.Energy] k]

  set u [second $E]
  if {$u != "eV"} then {
    midas-report "Energy units not keV.\nNew gate base time will not be set."
    return
  }

  set E1 [first $E]
  set E2 [expr pow(1.0/1.015, $s) * $E1]
# calculate delta t in ns
  set dt [expr 5.396 * 100.0 * sqrt($m) * (1/sqrt($E2) - 1/sqrt($E1))]
}

# We assume the units are correct (ns)
proc set-new-tac-base {w m} {
  global dt_fix

  set t [read-reg ERDA.TAC.Window$w.Base]
  set t [scan-num $t]
  set dt [calc-dtn $m 1]

  set dt [expr $dt * $dt_fix / 100]

  set newt "[expr $t + $dt] ns"
  set-footer "Setting TAC window base ($w) to ${newt}"
  write-reg ERDA.TAC.Window$w.Base $newt
  click-redisplay
}


# Mass in AMU of species of interest
set mass1 1
set mass2 1
set mass3 1

proc click-mass1 {r v a} {set-mass 1 $v}
proc click-mass2 {r v a} {set-mass 2 $v}
proc click-mass3 {r v a} {set-mass 3 $v}

proc click-dt_fix {r v a} {global dt_fix; set dt_fix $v}

proc set-mass {n m} {
upvar #0 mass$n mass
set mass $m
}

proc set-new-tac-bases {} {
   global mass
#   set m1 [read-reg ERDA.Recoil1.Mass]
   for {set i 1} {$i <= 3} {incr i} {
      upvar #0 mass$i mass
      set e [read-reg ERDA.TAC.Window$i.Enable]
      if {$e == 1} {set-new-tac-base $i $mass}
   }
}

set table(MASS1) {click-mass1 XXX {}}
set table(MASS2) {click-mass2 XXX {}}
set table(MASS3) {click-mass3 XXX {}}

set table(TAC_FIX) {click-dt_fix XXX {}}

set xpos 75

proc make-frame {} {
    global rtptable modetable xpos mass1 mass2 mass3 dt_fix
    
    :frame [self frame-name] :label "[self frame-title]" :resize T  :show-footer T \
        :icon-name "[self icon-title]"

    :panel P0 :padx 5 :pady 5
        :layout H; :col-gap 10; :row-gap 12

    :message      MTACWINS  :label "TAC Windows" :bold T
    :next-row T
    :slider       TAC_SCALE  :label "Scale (ns):" :min 0 :max 2500 :resolution 10 :valuex $xpos
    :next-row T
    :button       VIEWTAGS :label "View Current Tags" :command "set-tacspectrum-tags"
    :button       GRABTAGS :label "Grab All Tags" :command "set-tacwindows-baserange"
    :next-row T
    :number       TAC_FIX :label "Energy scan TAC base correction (%)" :w 4 :min 0 :max 200 :value $dt_fix

    :col-gap 5; :row-gap 15
    :next-row T
    :col-gap 5; :row-gap 10
    :message      MTACWIN2  :label "Window 2" :bold T
    :button       VIEW2 :label "View Tags" :command "set-tacspectrum-tags 2"
    :button       GRAB2 :label "Grab Tags" :command "set-tacwindow-baserange 2"
    :row-gap 8
    :next-row T
    :slider       TACWINDOW2_BASE   :label "Base (ns)" :min 0 :max 500 :resolution 1 :valuex $xpos
    :row-gap 10
    :next-row T
    :slider       TACWINDOW2_RANGE  :label "Range (ns)" :min 0 :max 500 :resolution 1 :valuex $xpos
    :next-row T
    :number	  MASS2 :label "Target Mass(amu):" :w 3 :value $mass1
    :checkbox     WINDOW2_ENABLE   :label "Enable:"


    :panel P1 :padx 5 :pady 5 :layout H
        :layout H; :col-gap 5; :row-gap 10

#    :next-row T
    :message      MTACWIN1  :label "Window 1" :bold T
    :button       VIEW1 :label "View Tags" :command "set-tacspectrum-tags 1"
    :button       GRAB1 :label "Grab Tags" :command "set-tacwindow-baserange 1"
    :row-gap 8
    :next-row T
    :slider       TACWINDOW1_BASE   :label "Base (ns)" :min 0 :max 500 :resolution 1 :valuex $xpos
    :row-gap 10
    :next-row T
    :slider       TACWINDOW1_RANGE  :label "Range (ns)" :min 0 :max 500 :resolution 1 :valuex $xpos
    :next-row T
    :number	  MASS1 :label "Target Mass(amu):" :w 3 :value $mass2
    :checkbox     WINDOW1_ENABLE   :label "Enable:"

    :col-gap 5; :row-gap 20
    :next-row T
    :message      MTACWIN3  :label "Window 3" :bold T
    :button       VIEW3 :label "View Tags" :command "set-tacspectrum-tags 3"
    :button       GRAB3 :label "Grab Tags" :command "set-tacwindow-baserange 3"
    :row-gap 8
    :next-row T
    :slider       TACWINDOW3_BASE   :label "Base (ns)" :min 0 :max 500 :resolution 1 :valuex $xpos
    :row-gap 10
    :next-row T
    :slider       TACWINDOW3_RANGE  :label "Range (ns)" :min 0 :max 500 :resolution 1 :valuex $xpos
    :next-row T
    :number	  MASS3 :label "Target Mass(amu):" :w 3 :value $mass3
    :checkbox     WINDOW3_ENABLE   :label "Enable:"

    :panel P3 :padx 5 :pady 5 :layout B :after FOOTER
        :layout H; :col-gap 5; :row-gap 10

    :message MFP :label "V785N Front Panel" :bold T
    :message MADC :label "ADC" :bold T
    :message MPED :label "Pedestal (mV)" :bold T
:hskip 80
    :message MENA :label "Enable" :bold T
    :next-row T

    :number       TAC_VAL :w 3 :label "TAC:" :min 0 :max 15 :valuex $xpos
    :text         ADC_FOR_T :w 3 :label "" :deaf T
    :slider       PEDESTAL_T   :label "" :min 0 :max 4096 :resolution 1
    :checkbox     ENABLE_T   :label ""

    :row-gap 15; :col-gap 15
    :next-row T
    :choice ERDAMODE :label "Mode:" :strings [list Normal ERDA]
    :checkbox SAVETAC  :label "Auto-save TAC:"
    :hskip 80
    :menubutton SAVERESTORE :label "Save" :menu [list "Save" "Restore"]
    :button REDISPLAY

   :show T
}

proc min {v1 v2} {expr $v1<$v2 ? $v1 : $v2} 
proc max {v1 v2} {expr $v1>$v2 ? $v1 : $v2} 

set testing 0

proc set-tacwindows-baserange args {
   global testing

   clear-footer
   set name [inform sd get-spectrum-name]
   set title [EG "spectrum :name '$name' ?title"]
   set t1 [first $title]
   set t2 [second $title]
   if {"$t1" != "MEIS" && "$t2" != "TAC"} then {
      set-footer "Not the MEIS ERDA TAC spectrum!!!"
      if {!$testing} {return}
   }

# The control window can "set" tags that are off-scale (so invisible)
# so get more than we need to (try to)  ensure we've got the ones we want
# i.e. the last 6
#   set alltags [lsort -integer [inform sd get-tags 10]]
   set alltags [inform sd get-tags 25]
   do-tacwindow-baserange [lrange $alltags 0 1] 1
   do-tacwindow-baserange [lrange $alltags 2 3] 2
   do-tacwindow-baserange [lrange $alltags 4 5] 3
}

proc set-tacwindow-baserange args {
   global testing

   clear-footer

   if {[llength $args] == 0} {
      set-footer "Window number not specified"
      return
   }

   set w [first $args]
   if {$w<1 || $w>3} {
      set-footer "Window number out of range"
      return
   }

   set name [inform sd get-spectrum-name]
   if {$testing} {
      set title "MEIS TAC"
   } else {
      set title [EG "spectrum :name '$name' ?title"]
   }
   set t1 [first $title]
   set t2 [second $title]
   if {"$t1" != "MEIS" && "$t2" != "TAC"} then {
      set-footer "Not the MEIS ERDA TAC spectrum!!!"
      if {!$testing} {return}
   }

   do-tacwindow-baserange [inform sd get-tags 2] $w
}

proc do-tacwindow-baserange {tags w} {
   if {[llength $tags] < 2} then {set-footer "you haven't set enough tags for Window $w"; return}
   set v1 [first $tags]
   set v2 [second $tags]
   set m1 [min $v1 $v2]
   set m2 [max $v1 $v2]
   set rng [expr $m2 - $m1 +1]
   write-reg ERDA.TAC.Window$w.Base [format "0x%x" $m1]
   write-reg ERDA.TAC.Window$w.Range [format "0x%x" $rng]

   click-redisplay
}

proc set-tacspectrum-tags args {
   clear-footer
   set name [inform sd get-spectrum-name]
   set title [EG "spectrum :name '$name' ?title"]
   set t1 [first $title]
   set t2 [second $title]
   if {"$t1" != "MEIS" && "$t2" != "TAC"} then {
      set-footer "You need to select the MEIS ERDA TAC spectrum"
      return
   }

   inform sd clear-tags

   if {[llength $args] == 0} {set args "1 2 3"}
   foreach w $args {
      set base [scan-num [read-reg ERDA.TAC.Window$w.Base]]
      set range [scan-num [read-reg ERDA.TAC.Window$w.Range]]
      set tag1 "=[set base]ns"
      set tag2 "=[expr $base - $range +1]ns"

      inform sd click-channel $tag1
      inform sd click-channel $tag2
   }
}
