# maxv.tcl---OMS MAXv6000 motor control module control frame

set frame "MAXv"
set frametitle "OMS MAXv6000 Setup & Monitor"
set icontitle "MAXv"

set modulename MAXv
set moduletype  "OMS MAXv"

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set Cmd ""
set AxisList [list "X" "Y" "Z" "T" "U" "V"]
set Axis [first $AxisList]

loadsource common.vme

get-module-configuration

set electronicsvisible 0
set electronicsframe "[self frame-name].electronics"


proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :transient T

    :galley G :deaf T :strings $text :font fixed :rows 50
    :show T
}

proc click-command {v} {
	global Axis Cmd
	set Cmd $v
}

proc click-axis v {
   global Axis AxisList

   set Axis [lindex $AxisList $v]
}

proc click-wo {} {
	global Axis Cmd
	global modulename module
	write-vme-register $modulename#$module.$Axis.Command.WriteOnly $Cmd
}

proc click-rw {} {
	global Axis Cmd
	global modulename module
	write-vme-register $modulename#$module.$Axis.Command.WriteRead $Cmd
	set rc [read-vme-register $modulename#$module.$Axis.Command.WriteRead]
    insert "$Cmd => $rc"
}

proc click-widget {w value} {
    global w2r
    global modulename module
    global allmodules

    set register $w2r($w)

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $value
    } else {
        write-vme-registers $modulename#\[?,??\].$register $value
    }

#    click-redisplay
}

proc reset-module {} {
    global modulename module allmodules
}
    
proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc click-electronics {} {
#  display electronics sub frame
   global electronicsvisible electronicsframe
}

proc click-select-module {v} {
   global debug module maxmodule
   if {$debug} then {puts stderr "click-select-module with argument \{$v\}"}
   set module $v
   if {$module<1} then {set module 1}
   if {$module>$maxmodule} then {set module $maxmodule}
   click-redisplay
}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc repaint {} {}

proc redisplay-module {} {
   global env w2r modulename module maxmodule

   set maxmodule [inform vmecontrol inquire [set modulename]count]

   set menu {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend menu $i}
   :modify MODULE-SEL :strings $menu
   :modify MODULE-NUM :max $maxmodule
   :modify MODULE-NUM :value $module

   foreach w {DISABLE RTP VSN1 VSN2} {
       set register $w2r($w)
       set value [read-vme-register $modulename#$module.$register]
       :modify $w :value $value
   }  
}

proc redisplay-adcs {} {
    global w2r
    global modulename module maxadc

    foreach W {THRESHOLD} {
       for {set i 0} {$i < $maxadc} {incr i} {
          set w $W$i
          set register $w2r($w)
          set value [read-vme-register $modulename#$module.$register]
          :modify $w :value $value
       }
    }
}

proc redisplay-electronics {} {
   global electronicsvisible

   if {$electronicsvisible==0} then {return}
}

proc click-redisplay {} {
   global EGreport

   clear-footer
   set KEEP $EGreport
   set EGreport -1
   set-busy

#   redisplay-module
#   redisplay-adcs
   redisplay-electronics

   clear-busy
   set EGreport $KEEP
}

proc xclick arg {
   global debug actionlist
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-ADCS    {click-all-adcs $v}

      RTP -
      VSN1 -
      VSN2 -
      TESTAMP -
      DISABLE       {click-widget $w $v}

      THRESHOLD*  {click-adc-widget $w $v}

      ENGINEERING       {click-$v}
      REDISPLAY  	{click-redisplay}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env actionlist Cmd AxisList
  global module maxmodule modulelist 

set rows 24
set cols 80

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
     :menubutton MODULE-SEL :label "select:" :strings $modulelist
     :next-row T

   :panel CMD :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 25
     :text COMMAND   :label "Command:" :w 72 :value $Cmd
     :next-row T
     :choice AXIS :label "Axis:" :strings $AxisList :value 0
     :button WO    :label "Write Only"
     :button RW    :label "Read after Write"

   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY
      :menubutton ENGINEERING :label "Engineering"  :menu [list software] 

  :panel P2 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font variable
    configuretags

   open-frame

  :galley_size LOG $rows $cols

}

proc click-configure {W} {:galley_resize LOG}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n; update idletasks}

