# scan.tcl
# to be loaded by gon.tcl
# moves the height axis of the goniometer across/back a range
# during a (set of) DA run(s) in order to spread sample damage
#

# The method of passing parameters is rather "odd" in that the existing
# code (in gon.tcl) is used to set parameters (Dwell, Pitch etc) in the
# VME register server and these parameters are then read back from the
# VME and used by this new tcl code. The tcl code assumes
# the correct attributes have been set, in particular qualifiers,
# so that (e.g.) we assume time to be read in seconds not ms.

set scan_debug 0
set scan_height_active 0

set scanStepAfterid ""
set scanAcqAfterid ""

set scanStepRegister ""
set scanStepArgs ""
set scanStepRequest 0
set scanStepRange 0
set scanStepDwell 0
set scanStepPitch 0
set scanStepStart 0
set scanStepNext 0
set scanStepEnd 0
set scanStepStopping 0

set scanStepInitDirect 1
set scanStep1Pass 0

set scanStepDirection $scanStepInitDirect

proc scan-height-next cur {
   global scanStepRegister scanStepDirection scanStepPitch
   global scanStepStart scanStepEnd scanStep1Pass
   global scan_debug scanStepInitDirect
   global VMEserver server debug

   set server $VMEserver
if {$scan_debug} {insert-debug "scan-height-next: scanStepEnd: $scanStepEnd"}
if {$scan_debug} {insert-debug "scan-height-next: scanStepStart: $scanStepStart"}

   set changed 0
   set Full [read-reg $scanStepRegister.Full]
   if {$Full} then {
      if {$scanStep1Pass} then {
         stop-height-scan
         halt-energy-scan
if {$scan_debug} {insert-debug "scan-height-next: +ve limit switch, 1 Pass => \"OnePassDone\""}
         return "OnePassDone"
      } else {
if {$scan_debug} {insert-debug "scan-height-next: +ve limit switch, change direction"}
         set scanStepDirection -1; set changed 1
      }
   }
   set Home [read-reg $scanStepRegister.Home]
   if {$Home} then {
      if {$scanStep1Pass} then {
         stop-height-scan
         halt-energy-scan
if {$scan_debug} {insert-debug "scan-height-next: -ve limit switch, 1 Pass => \"OnePassDone\""}
         return "OnePassDone"
      } else {
if {$scan_debug} {insert-debug "scan-height-next: -ve limit switch, change direction"}
         set scanStepDirection  1; set changed 1
      }
   }

   set pitch [scan-num $scanStepPitch]
   set nxt [expr $cur + ($scanStepDirection * $pitch)]
if {$scan_debug} {insert-debug "scan-height-next: nxt is $nxt"}

# If  we started with direction +ve End is > Start
# If  we started with direction -ve End is < Start

if {$scanStepEnd > $scanStepStart} then {
   if {$nxt > $scanStepEnd} then {
if {$scan_debug} {insert-debug "scan-height-next: $nxt > $scanStepEnd, End > Start pass done"}
      if {$scanStep1Pass} then {
         stop-height-scan
         halt-energy-scan
if {$scan_debug} {insert-debug "scan-height-next: +ve limit, 1 Pass => \"OnePassDone\""}
         return "OnePassDone"
      }
      if {$changed} then {
         set nxt $cur
      } else {
         set scanStepDirection -1
         set changed 1
         set nxt [expr $cur +  ($scanStepDirection * $pitch)]
      }
   } 
   if {$nxt < $scanStepStart} {
if {$scan_debug} {insert-debug "scan-height-next: $nxt < $scanStepStart, End > Start pass done"}
      if {$changed} then {
         set nxt $cur
      } else {
         set scanStepDirection 1
         set nxt [expr $cur +  ($scanStepDirection * $pitch)]
      }
   }
} else {
   if {$nxt < $scanStepEnd} then {
if {$scan_debug} {insert-debug "scan-height-next: $nxt < $scanStepEnd, End < Start pass done"}
      if {$scanStep1Pass} then {
         stop-height-scan
         halt-energy-scan
if {$scan_debug} {insert-debug "scan-height-next: -ve limit, 1 Pass =>  \"OnePassDone\""}
         return "OnePassDone"
      }
      if {$changed} then {
         set nxt $cur
      } else {
         set scanStepDirection 1
         set changed 1
         set nxt [expr $cur +  ($scanStepDirection * $pitch)]
      }
   } 
   if {$nxt > $scanStepStart} {
if {$scan_debug} {insert-debug "scan-height-next: $nxt > $scanStepStart, End < Start pass done"}
      if {$changed} then {
         set nxt $cur
      } else {
         set scanStepDirection -1
         set nxt [expr $cur +  ($scanStepDirection * $pitch)]
      }
   }
}

if {$scan_debug} {insert-debug "scan-height-next $cur => $nxt"}
   return $nxt
}

proc scan-height-wait {} {
   global scanStepNext scanStepDwell scanAcqAfterid
   global scanStepRegister scanStepStopping
   global scan_debug

   if {$scan_debug} {insert-debug "scan-height-wait: [second [split $scanStepRegister .]], $scanStepNext, Dwell: $scanStepDwell"}

   click-redisplay

   if {$scanAcqAfterid != ""} then {
      catch {kill-after $scanAcqAfterid}
      set scanAcqAfterid ""
   }
   clear-footer
   after 5000 click-redisplay
   if {!$scanStepStopping} then {
      set scanAcqAfterid [after [expr $scanStepDwell - 5000] scan-step-again]
   } else { after 2000 click-redisplay}
}

proc scan-step-refresh {} {
   global scanStepAfterid driving refreshmessage
   global scanStepRegister scanStepArgs scanStepRequest scanStepRange
   global scanStepDwell scanStepPitch scanStepNext scanStepStart scanStepEnd
   global scan_debug

if {$scan_debug} {insert-debug "scan-step-refresh: [second [split $scanStepRegister .]]"}

   set driving 0
   click-redisplay
   if {$driving} then {
if {$scan_debug} {insert-debug "scan-step-refresh: stepper still moving"}
      if {$scanStepAfterid != ""} then {
         catch {kill-after $scanStepAfterid}
         set scanStepAfterid ""
      }
      set scanStepAfterid [after 1000 scan-step-refresh]
   } else {
	set-footer $refreshmessage
	scan-height-wait
   }
}

proc scan-step-again {} {
   global scanStepAfterid refreshmessage scanAcqAfterid
   global scanStepRegister scanStepArgs scanStepRequest scanStepRange
   global scanStepDwell scanStepPitch scanStepNext scanStepStart scanStepEnd
   global scanStepDirection scanStepStopping scan_debug

if {$scan_debug} {insert-debug "scan-step-again: [second [split $scanStepRegister .]]"}

   if {$scanAcqAfterid != ""} then {
      catch {kill-after $scanAcqAfterid}
      set scanAcqAfterid ""
   }

   set nxt [scan-height-next $scanStepNext]
   if {"$nxt" == "OnePassDone"} then {return} else {set scanStepNext $nxt}

if {$scan_debug} {insert-debug "scanStepNext now set to $nxt"}

   if {!$scanStepStopping} {do-scan-step $scanStepNext} {after 500 click-redisplay}
}

proc do-scan-step v {
   global gondebug refreshmessage
   global scanStepRegister scanStepArgs scanStepRequest scanStepRange
   global scanStepDwell scanStepPitch scanStepNext scanStepStart scanStepEnd
   global scan_debug
   global VMEserver server debug

   set server $VMEserver

if {$scan_debug} {insert-debug "do-scan-step: [second [split $scanStepRegister .]]"}
if {$scan_debug} {insert-debug "do-scan-step: moving to $v"}

   if {![string match *.* $v]} then {set v $v.0}

   set m      [first $scanStepArgs]
   set step   [second $scanStepArgs]
   set unit   [third $scanStepArgs]

   set newv [expr {$v * (1000.0 / $m)}]
   set value [read-reg $scanStepRegister.Request]
   set oldv   [expr {[scan-num $value] * 1000.0}]
   if {$newv > $oldv && $newv < [expr $oldv+$step]} then {
      set newv [expr $oldv+$step] }
   if {$newv < $oldv && $newv > [expr $oldv-$step]} then {
      set newv [expr $oldv-$step] }

   set newv "[expr {$newv / 1000.0}] $unit"
if {$scan_debug} {insert-debug "do-scan-step: setting $scanStepRegister.Request to $newv"}
if {$scan_debug} then {
   insert-debug "write-reg-nocheck $scanStepRegister.Request $newv"
}
   write-reg-nocheck $scanStepRegister.Request $newv
   set refreshmessage "Height Stepping Complete"
   scan-step-refresh
   set-footer "$scanStepRegister set to $newv..."
}

proc click-zscan-dir v {
# expression changes 0/1 to -1/+1
   global scanStepInitDirect
   set scanStepInitDirect [expr ($v *2) - 1]
}

proc click-zscan-mode v {
# expression changes 0/1 to -1/+1
   global scanStep1Pass
   set scanStep1Pass $v
}

proc click-height-scan {r v a} {
   global debug refreshmessage table
   global scanStepRegister scanStepArgs scanStepRequest scanStepRange
   global scanStepDwell scanStepPitch scanStepNext scanStepStart scanStepEnd
   global scanStepDirection scanStepInitDirect scanStepStopping
   global scan_debug scan_height_active
   global VMEserver server debug

   set server $VMEserver

   set scanStepStopping 0
   set scan_height_active 1

   if {!$v} then {
      set scanStepStopping 1
      set scan_height_active 0
      return
   }

   set scanStepRegister $r
   set scanStepArgs $a

   set scanStepDirection $scanStepInitDirect

   set scanStepRequest [read-reg $scanStepRegister.Request]
   set scanStepRequest [scan-num $scanStepRequest]
   set scanStepPitch [read-reg $scanStepRegister.Scan.Pitch]
#   set scanStepDwell [read-reg $scanStepRegister.Scan.Dwell]
#   set scanStepDwell [expr int([scan-num $scanStepDwell] * 1000)]
   set scanStepDwell [read-reg $scanStepRegister.Scan.Dwell.Raw]
   set scanStepRange [read-reg $scanStepRegister.Scan.Range]
#   set scanStepStart [expr $scanStepRequest - [scan-num $scanStepRange]]
   set scanStepStart $scanStepRequest
   set scanStepEnd [expr $scanStepRequest + ($scanStepInitDirect * [scan-num $scanStepRange]) ]

# The next line has been re-instated SCL 11/01/06
# We now scan from where we happen to be, again
#   set scanStepNext [scan-height-next $scanStepRequest]
   set scanStepNext $scanStepRequest

# Change & comment added SCL 22/09/05 (?)
# We used to scan from the initial position
# we now move to the end and continue scanning from there
# The next line has been re-commented SCL 11/01/06
#   set scanStepNext $scanStepEnd

if {$scan_debug} {insert-debug "scanStepNext starting at $scanStepNext"}
   do-scan-step $scanStepNext
}

proc stop-height-scan {} {
   click-height-scan Goniometer.Height 0 ignore
   click-redisplay
   after 5000 click-redisplay
}

proc click-redisplay {} {
   global scan_height_active
   global VMEserver server debug

   set server $VMEserver
   update-choice SCAN-HEIGHT $scan_height_active ignore

   set list [read-regs \[An,D,G\]*]
   foreach pair $list {update-pair $pair}
}

# These 2 procedures are for use in ERDA mode
# Therefore do not call them in 1 Pass mode - we don't want to stop DAQ
# at the movement limit. (Do we?)

proc do-height-step v {
   global gondebug refreshmessage
   global scanStepRegister scanStepArgs scanStepRequest scanStepRange
   global scanStepDwell scanStepPitch scanStepNext scanStepStart scanStepEnd
   global scan_debug
   global VMEserver server debug

   set server $VMEserver

if {$scan_debug} {insert-debug "do-height-step: [second [split $scanStepRegister .]]"}
if {$scan_debug} {insert-debug "do-height-step: moving to $v"}

   if {![string match *.* $v]} then {set v $v.0}

   set m      [first $scanStepArgs]
   set step   [second $scanStepArgs]
   set unit   [third $scanStepArgs]

   set newv [expr {$v * (1000.0 / $m)}]
   set value [read-reg $scanStepRegister.Request]
   set oldv   [expr {[scan-num $value] * 1000.0}]
   if {$newv > $oldv && $newv < [expr $oldv+$step]} then {
      set newv [expr $oldv+$step] }
   if {$newv < $oldv && $newv > [expr $oldv-$step]} then {
      set newv [expr $oldv-$step] }

   set newv "[expr {$newv / 1000.0}] $unit"
if {$scan_debug} {insert-debug "do-height-step: setting $scanStepRegister.Request to $newv"}
if {$scan_debug} then {
   insert-debug "write-reg-nocheck $scanStepRegister.Request $newv"
}
   write-reg-nocheck $scanStepRegister.Request $newv
   set-footer "$scanStepRegister set to $newv..."
}

proc move-height {} {
   global scanStepRegister scanStepArgs scanStepDirection scanStepPitch
   global scanStepStart scanStepEnd scanStep1Pass scanStepInitDirect
   global scan_debug
   global VMEserver server debug

   set scanStep1Pass 0
   :modify ZSCANMODE :value 0

   set scanStepInitDirect 1
   :modify ZSCANDIR :value [expr ($scanStepInitDirect + 1) / 2]

   set server $VMEserver

   set scanStepRegister Goniometer.Height
   set scanStepArgs {1 13 mm}

   set scanStepRequest [read-reg $scanStepRegister.Request]
   set scanStepRequest [scan-num $scanStepRequest]
   set scanStepPitch [read-reg $scanStepRegister.Scan.Pitch]
   set scanStepRange [read-reg $scanStepRegister.Scan.Range]
   set scanStepStart [expr $scanStepRequest - [scan-num $scanStepRange]]
   set scanStepEnd   [expr $scanStepRequest +  [scan-num $scanStepRange]]

   set scanStepNext [scan-height-next $scanStepRequest]
if {$scan_debug} {insert-debug "move-height: scanStepNext $scanStepNext"}
   do-height-step $scanStepNext

   set pitch [scan-num $scanStepPitch]
   set pause [expr int ($pitch * 10 * 1000)]
   after $pause
   click-redisplay
}
