#    MEIS Control System
#      June 1996   TK Version   VP

set frame STEP
set frametitle "Daresbury - SRS122 - Stepper Motor Controller"
set icontitle  "Stepper"

set server /VME

set hicount(STEPS) 0
set locount(STEPS) 0
set hicount(ENC) 0
set locount(ENC) 0
set hicount(DECEL) 0
set locount(DECEL) 0

#
#                     proc      register           args
#
set table(CHANNEL)     {click-int    Channel         {}}
set table(INT-MASK)    {click-int    Int.Mask        {}}
set table(SPEED)       {click-slider Speed    {1 40 Hz}}
set table(STEPS)       {click-cnt    Step            {}}
set table(ENC)         {click-cnt    Encoder         {}}
set table(DECEL)       {click-cnt    Decel           {}}
set table(STEP-RES)    {click-set    Step.Reset      {}}
set table(ENC-RES)     {click-set    Encoder.Reset   {}}
set table(DIR)         {click-int    Direction       {}}
set table(COUNT)       {click-int    Feedback        {}}
set table(GO)          {click-int    Go              {}}
set table(STOP)        {click-int    Stop            {}}
set table(BOOST)       {click-int    Boost           {}}
set table(RESET)       {click-set    Reset           {}}
#
#                          proc          widgets     args
#
set table(Channel)         {update-int    CHANNEL     {}}
set table(Int.Request)     {update-int    INT-REQUEST {}}
set table(Int.Mask)        {update-int    INT-MASK    {}}
set table(Status)          {update-int    STATUS      {}}
set table(Slow)            {update-int    SLOW        {}}
set table(Encoder.Dir)     {update-int    ENC-FOR     {}}
set table(Speed)           {update-slider SPEED        1}
set table(Step.Hi)         {update-hi     STEPS       {}}
set table(Step.Lo)         {update-lo     STEPS       {}}
set table(Encoder.Hi)      {update-hi     ENC         {}}
set table(Encoder.Lo)      {update-lo     ENC         {}}
set table(Decel.Hi)        {update-hi     DECEL       {}}
set table(Decel.Lo)        {update-lo     DECEL       {}}
set table(Feedback)        {update-int    COUNT       {}}
set table(Direction)       {update-int    DIR         {}}
set table(Go)              {update-int    GO          {}}
set table(Stop)            {update-int    STOP        {}}
set table(Boost)           {update-int    BOOST       {}}

proc click-help {} {inform mb goto {XVME-240 Setup}}

proc make-reg-name reg {return "srs122#1.$reg"}

proc update-hi {w v a} {
   global hicount locount
   set hicount($w) [expr {[scan-int $v] << 16}]
   if {$hicount($w) & 0x800000} then {set hicount($w) [expr {$hicount($w) | 0xff000000}]}
   :modify $w :value [expr {$hicount($w) | $locount($w)}]
}

proc update-lo {w v a} {
   global hicount locount
   set locount($w) [expr {[scan-int $v] & 0xffff}]
   :modify $w :value [expr {$hicount($w) | $locount($w)}]
}

proc update-slider {w v m} {
   set value [first $v] 
   :modify $w :value $value
}

proc update-pair {p} {
	global debug table

	if {$debug} then {puts stderr $p}

	set reg [strip-to-dot [first $p]]
	set val [second $p]
	catch {
		set func   [first  $table($reg)]
		set widget [second $table($reg)]
		set arg    [third  $table($reg)]
		$func $widget $val $arg
	}
}

proc click-reset {} {
   write-reg Reset 1
   click-redisplay
}

proc click-cnt {r v a} {
   global debug
   if {$debug} then {puts stderr "click-cnt $r $v $a"}

   write-reg $r.Hi [expr {($v & 0xff0000) >> 16}]
   write-reg $r.Lo [expr {$v & 0xffff}]
}

proc click-redisplay {} {
   set list [read-regs *]
   foreach pair $list {update-pair $pair}
}

proc click arg {
   global debug 
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      REDISPLAY {click-redisplay}
      HELP      {click-help}
      default   {click-on $w $v}
   }
}


proc make-frame {} {

set col1 13c

set bits {Complete Home Full Fault}
set chanlist {0 1 2 3 4 5 6 7 8}

   :frame [self frame-name] :label "[self frame-title]"  :resize F :show-footer T \
	:icon-name [self icon-title]

   :panel P :padx 5 :pady 5
   :layout H; :col-gap 10; :row-gap 10

	:choice-stack CHANNEL :label "Channel:" :strings $chanlist :valuex $col1
	:next-row T
	:choice  INT-REQUEST :strings $bits :label "Interrupt:" :exclusive F :deaf T :valuex $col1
	:next-row T
	:choice  INT-MASK :strings $bits :label "Int Mask:" :exclusive F  :valuex $col1
	:next-row T
	:choice  STATUS :strings $bits :label "Status:" :exclusive F :deaf T :valuex $col1
	:choice  SLOW   :strings "Slow" :label "" :exclusive F :deaf T
	:next-row T
	:choice DIR :label "Direction:" :strings [list "Home" "Full"] :exclusive T :valuex $col1
        :hskip 80
	:choice  ENC-FOR :strings [list "Home" "Full"] :label "Encoder:" :exclusive T :deaf T
	:next-row T
	:choice GO :label "Control:" :strings "Go" :exclusive F :valuex $col1
	:choice STOP :label "" :strings "Stop" :exclusive F
	:choice BOOST :label "" :strings "Boost" :exclusive F
	:choice COUNT :label "Count:" :strings [list "Steps" "Encoder"] :exclusive T
	:next-row T
	:slider STEPS :min -8388608 :max 8388607 :resolution 1 :label "Steps:" :valuex $col1 :entry-width 10
	:next-row T
	:slider ENC :min -8388608 :max 8388607 :resolution 1 :label "Encoder:" :valuex $col1 :entry-width 10
	:next-row T
	:slider DECEL :min 0 :max 131071 :resolution 1 :label "Decel:" :valuex $col1
	:next-row T
	:slider SPEED :label "Speed (Hz):" :min 40 :max 10240 :resolution 1 :valuex $col1
	:next-row T
	:button REDISPLAY
	:button RESET
	:button STEP-RES :label "Reset Steps"
	:button ENC-RES :label "Reset Encoder"
	:button HELP

   :show T
}
