#  stepper.defn.tcl

set ROTA_STEPPER 0
set SPIN_STEPPER 1
set TILT_STEPPER 2
set ANAL_STEPPER 3
set Z_STEPPER    4

set netvar_name MeisRegs

set NUM_STEPPERS 5
set STEPPER_SIZE 11
set LOCAL_STEPPER_BASE 9

proc LOCAL_STEPPER_MACRO {r s} {
global STEPPER_SIZE LOCAL_STEPPER_BASE
return [expr ($LOCAL_STEPPER_BASE+$r) + $s*$STEPPER_SIZE]
}

proc LOCAL_STEPPER_CURRENT {s} {return [LOCAL_STEPPER_MACRO  0 $s]}
proc LOCAL_STEPPER_REQUEST {s} {return [LOCAL_STEPPER_MACRO  1 $s]}
proc LOCAL_STEPPER_RANGE   {s} {return [LOCAL_STEPPER_MACRO  2 $s]}
proc LOCAL_STEPPER_PITCH   {s} {return [LOCAL_STEPPER_MACRO  3 $s]}
proc LOCAL_STEPPER_DWELL   {s} {return [LOCAL_STEPPER_MACRO  4 $s]}
proc LOCAL_STEPPER_SCAN    {s} {return [LOCAL_STEPPER_MACRO  5 $s]}
proc LOCAL_STEPPER_ZERO    {s} {return [LOCAL_STEPPER_MACRO  6 $s]}
proc LOCAL_STEPPER_ENABLE  {s} {return [LOCAL_STEPPER_MACRO  7 $s]}
proc LOCAL_STEPPER_DRIVE   {s} {return [LOCAL_STEPPER_MACRO  8 $s]}
proc LOCAL_STEPPER_STATUS  {s} {return [LOCAL_STEPPER_MACRO  9 $s]}
proc LOCAL_STEPPER_SPEED   {s} {return [LOCAL_STEPPER_MACRO 10 $s]}

proc Define-Rotation {netvar stepper} {
   set attr "-n$netvar -a[LOCAL_STEPPER_CURRENT $stepper] -w32 -pro -q -udeg -d3 -r0x7ef582:360deg,0x8084d0:-100deg"
   define-vme-register Goniometer netAnalogue Rota $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_REQUEST $stepper] -w32 -prw -q -udeg -d3 -r0x7ef582:360deg,0x8084d0:-100deg -i180deg"
   define-vme-register Goniometer netAnalogue Rota.Request $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_RANGE $stepper] -w32 -prw -q -udeg -d0 -r0:0deg,0x13880:360deg -i3deg"
   define-vme-register Goniometer netAnalogue Rota.Scan.Range $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_PITCH $stepper] -w32 -prw -q -udeg -d6 -r0:0deg,0x13880:360deg -i0.2deg"
   define-vme-register Goniometer netAnalogue Rota.Scan.Pitch $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DWELL $stepper] -w32 -prw -q -us   -d0 -r0:0s,3600:60s -i1s"
   define-vme-register Goniometer netAnalogue Rota.Scan.Dwell $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SCAN $stepper] -w32 -prw -fd -i0"
   define-vme-register Goniometer netDigital Rota.Scan $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ZERO $stepper] -w16 -prw -fd -i0"
   define-vme-register Goniometer netDigital Rota.Zero $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ENABLE $stepper] -w16 -prw -fd -i0"
   define-vme-register Goniometer netDigital Rota.Enable $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DRIVE $stepper] -w16 -pro -fd"
   define-vme-register Goniometer netDigital Rota.Drive $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fx"
   define-vme-register Goniometer netDigital Rota.Status $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b0 -l1"
   define-vme-register Goniometer netDigital Rota.Complete $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b1 -l1"
   define-vme-register Goniometer netDigital Rota.Home $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b2 -l1"
   define-vme-register Goniometer netDigital Rota.Full $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b3 -l1"
   define-vme-register Goniometer netDigital Rota.Fault $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b4 -l1"
   define-vme-register Goniometer netDigital Rota.Go $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b5 -l1"
   define-vme-register Goniometer netDigital Rota.Boost $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b6 -l1"
   define-vme-register Goniometer netDigital Rota.Encoder $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b7 -l1"
   define-vme-register Goniometer netDigital Rota.Direction $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b8 -l1"
   define-vme-register Goniometer netDigital Rota.EncDir $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b9 -l1"
   define-vme-register Goniometer netDigital Rota.Slow $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b10 -l1"
   define-vme-register Goniometer netDigital Rota.Stop $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SPEED $stepper] -w16 -prw -fx"
   define-vme-register Goniometer netDigital Rota.Speed $attr
}

proc Define-Spin {netvar stepper} {
   set attr "-n$netvar -a[LOCAL_STEPPER_CURRENT $stepper] -w32 -pro -q -udeg -d3 -r0x7f9c00:0deg,0x80af00:220deg"
   define-vme-register Goniometer netAnalogue Spin $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_REQUEST $stepper] -w32 -prw -q -udeg -d3 -r0x7f9c00:0deg,0x80af00:220deg -i90deg"
   define-vme-register Goniometer netAnalogue Spin.Request $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_RANGE $stepper] -w32 -prw -q -udeg -d0 -r0:0deg,0x11300:220deg -i3deg"
   define-vme-register Goniometer netAnalogue Spin.Scan.Range $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_PITCH $stepper] -w32 -prw -q -udeg -d6 -r0:0deg,0x11300:220deg -i0.2deg"
   define-vme-register Goniometer netAnalogue Spin.Scan.Pitch $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DWELL $stepper] -w32 -prw -q -us   -d0 -r0:0s,3600:60s -i1s"
   define-vme-register Goniometer netAnalogue Spin.Scan.Dwell $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SCAN $stepper] -w32 -prw -fd -i0"
   define-vme-register Goniometer netDigital Spin.Scan $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ZERO $stepper] -w16 -prw -fd -i0"
   define-vme-register Goniometer netDigital Spin.Zero $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ENABLE $stepper] -w16 -prw -fd -i0"
   define-vme-register Goniometer netDigital Spin.Enable $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DRIVE $stepper] -w16 -pro -fd"
   define-vme-register Goniometer netDigital Spin.Drive $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fx"
   define-vme-register Goniometer netDigital Spin.Status $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b0 -l1"
   define-vme-register Goniometer netDigital Spin.Complete $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b1 -l1"
   define-vme-register Goniometer netDigital Spin.Home $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b2 -l1"
   define-vme-register Goniometer netDigital Spin.Full $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b3 -l1"
   define-vme-register Goniometer netDigital Spin.Fault $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b4 -l1"
   define-vme-register Goniometer netDigital Spin.Go $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b5 -l1"
   define-vme-register Goniometer netDigital Spin.Boost $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b6 -l1"
   define-vme-register Goniometer netDigital Spin.Encoder $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b7 -l1"
   define-vme-register Goniometer netDigital Spin.Direction $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b8 -l1"
   define-vme-register Goniometer netDigital Spin.EncDir $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b9 -l1"
   define-vme-register Goniometer netDigital Spin.Slow $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b10 -l1"
   define-vme-register Goniometer netDigital Spin.Stop $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SPEED $stepper] -w16 -prw -fx"
   define-vme-register Goniometer netDigital Spin.Speed $attr
}

proc Define-Tilt {netvar stepper} {
   set attr "-n$netvar -a[LOCAL_STEPPER_CURRENT $stepper] -w32 -pro -q -udeg -d3 -r0x7ff472:-5.25deg,0x800b8e:5.25deg"
   define-vme-register Goniometer netAnalogue Tilt $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_REQUEST $stepper] -w32 -prw -q -udeg -d3 -r0x7ff472:-5.25deg,0x800b8e:5.25deg -i0deg"
   define-vme-register Goniometer netAnalogue Tilt.Request $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_RANGE $stepper] -w32 -prw -q -udeg -d1 -r0:0deg,0xb8e:5.25deg -i30deg"
   define-vme-register Goniometer netAnalogue Tilt.Scan.Range $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_PITCH $stepper] -w32 -prw -q -udeg -d6 -r0:0deg,0xb8e:5.25deg -i0.099deg"
   define-vme-register Goniometer netAnalogue Tilt.Scan.Pitch $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DWELL $stepper] -w32 -prw -q -us   -d0 -r0:0s,3600:60s -i1s"
   define-vme-register Goniometer netAnalogue Tilt.Scan.Dwell $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SCAN $stepper] -w32 -prw -fd -i0"
   define-vme-register Goniometer netDigital Tilt.Scan $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ZERO $stepper] -w16 -prw -fd -i0"
   define-vme-register Goniometer netDigital Tilt.Zero $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ENABLE $stepper] -w16 -prw -fd -i0"
   define-vme-register Goniometer netDigital Tilt.Enable $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DRIVE $stepper] -w16 -pro -fd"
   define-vme-register Goniometer netDigital Tilt.Drive $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fx"
   define-vme-register Goniometer netDigital Tilt.Status $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b0 -l1"
   define-vme-register Goniometer netDigital Tilt.Complete $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b1 -l1"
   define-vme-register Goniometer netDigital Tilt.Home $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b2 -l1"
   define-vme-register Goniometer netDigital Tilt.Full $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b3 -l1"
   define-vme-register Goniometer netDigital Tilt.Fault $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b4 -l1"
   define-vme-register Goniometer netDigital Tilt.Go $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b5 -l1"
   define-vme-register Goniometer netDigital Tilt.Boost $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b6 -l1"
   define-vme-register Goniometer netDigital Tilt.Encoder $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b7 -l1"
   define-vme-register Goniometer netDigital Tilt.Direction $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b8 -l1"
   define-vme-register Goniometer netDigital Tilt.EncDir $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b9 -l1"
   define-vme-register Goniometer netDigital Tilt.Slow $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b10 -l1"
   define-vme-register Goniometer netDigital Tilt.Stop $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SPEED $stepper] -w16 -prw -fx"
   define-vme-register Goniometer netDigital Tilt.Speed $attr
}

proc Define-Height {netvar stepper} {
   set attr "-n$netvar -a[LOCAL_STEPPER_CURRENT $stepper] -w32 -pro -q -umm -d3 -r0x7eddc0:-14mm,0x8264c0:14mm"
   define-vme-register Goniometer netAnalogue Height $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_REQUEST $stepper] -w32 -prw -q -umm -d3 -r0x7eddc0:-14mm,0x8264c0:14mm -i0mm"
   define-vme-register Goniometer netAnalogue Height.Request $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_RANGE $stepper] -w32 -prw -q -umm -d1 -r0:0mm,0x14280:10mm -i1mm"
   define-vme-register Goniometer netAnalogue Height.Scan.Range $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_PITCH $stepper] -w32 -prw -q -umm -d6 -r0:0mm,0x14280:10mm -i0.05mm"
   define-vme-register Goniometer netAnalogue Height.Scan.Pitch $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DWELL $stepper] -w32 -prw -q -us  -d0 -r0:0s,3600:60s -i1s"
   define-vme-register Goniometer netAnalogue Height.Scan.Dwell $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SCAN $stepper] -w32 -prw -fd -i0"
   define-vme-register Goniometer netDigital Height.Scan $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ZERO $stepper] -w16 -prw -fd -i0"
   define-vme-register Goniometer netDigital Height.Zero $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ENABLE $stepper] -w16 -prw -fd -i0"
   define-vme-register Goniometer netDigital Height.Enable $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DRIVE $stepper] -w16 -pro -fd"
   define-vme-register Goniometer netDigital Height.Drive $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fx"
   define-vme-register Goniometer netDigital Height.Status $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b0 -l1"
   define-vme-register Goniometer netDigital Height.Complete $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b1 -l1"
   define-vme-register Goniometer netDigital Height.Home $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b2 -l1"
   define-vme-register Goniometer netDigital Height.Full $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b3 -l1"
   define-vme-register Goniometer netDigital Height.Fault $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b4 -l1"
   define-vme-register Goniometer netDigital Height.Go $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b5 -l1"
   define-vme-register Goniometer netDigital Height.Boost $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b6 -l1"
   define-vme-register Goniometer netDigital Height.Encoder $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b7 -l1"
   define-vme-register Goniometer netDigital Height.Direction $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b8 -l1"
   define-vme-register Goniometer netDigital Height.EncDir $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b9 -l1"
   define-vme-register Goniometer netDigital Height.Slow $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b10 -l1"
   define-vme-register Goniometer netDigital Height.Stop $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SPEED $stepper] -w16 -prw -fx"
   define-vme-register Goniometer netDigital Height.Speed $attr
}

proc Define-Analyser {netvar stepper} {
   set attr "-n$netvar -a[LOCAL_STEPPER_CURRENT $stepper] -w32 -pro -q -udeg -d3 -r0x8029bf:0deg,0x7fb7ac:360deg"
   define-vme-register Analyser netAnalogue Angle $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_REQUEST $stepper] -w32 -prw -q -udeg -d3 -r0x8029bf:0deg,0x7fb7ac:360deg -i120deg"
   define-vme-register Analyser netAnalogue Angle.Request $attr

#/* these registers for completeness */

   set attr "-n$netvar -a[LOCAL_STEPPER_RANGE $stepper] -w32 -prw -q -udeg -d0 -r0:0deg,811:10deg -i10deg"
   define-vme-register Analyser netAnalogue Angle.Scan.Range $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_PITCH $stepper] -w32 -prw -q -udeg -d6 -r0:0deg,264:3.25deg -i0.05deg"
   define-vme-register Analyser netAnalogue Angle.Scan.Pitch $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DWELL $stepper] -w32 -prw -q -us   -d0 -r0:0s,3600:60s -i10s"
   define-vme-register Analyser netAnalogue Angle.Scan.Dwell $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SCAN $stepper] -w32 -prw -fd -i0"
   define-vme-register Analyser netDigital Angle.Scan $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ZERO $stepper] -w16 -prw -fd -i0"
   define-vme-register Analyser netDigital Angle.Zero $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_ENABLE $stepper] -w16 -prw -fd -i0"
   define-vme-register Analyser netDigital Angle.Enable $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_DRIVE $stepper] -w16 -pro -fd"
   define-vme-register Analyser netDigital Angle.Drive $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fx"
   define-vme-register Analyser netDigital Angle.Status $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b0 -l1"
   define-vme-register Analyser netDigital Angle.Complete $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b1 -l1"
   define-vme-register Analyser netDigital Angle.Home $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b2 -l1"
   define-vme-register Analyser netDigital Angle.Full $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b3 -l1"
   define-vme-register Analyser netDigital Angle.Fault $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b4 -l1"
   define-vme-register Analyser netDigital Angle.Go $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b5 -l1"
   define-vme-register Analyser netDigital Angle.Boost $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b6 -l1"
   define-vme-register Analyser netDigital Angle.Encoder $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b7 -l1"
   define-vme-register Analyser netDigital Angle.Direction $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b8 -l1"
   define-vme-register Analyser netDigital Angle.EncDir $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b9 -l1"
   define-vme-register Analyser netDigital Angle.Slow $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_STATUS $stepper] -w16 -pro -fd -b10 -l1"
   define-vme-register Analyser netDigital Angle.Stop $attr
   set attr "-n$netvar -a[LOCAL_STEPPER_SPEED $stepper] -w16 -prw -fx"
   define-vme-register Analyser netDigital Angle.Speed $attr
}

proc {Add_Stepper_Module} {name server addressdata} {
   global CurrentVMEServer ROTA_STEPPER SPIN_STEPPER TILT_STEPPER ANAL_STEPPER Z_STEPPER

   set CurrentVMEServer  $server

   Define-Rotation $name $ROTA_STEPPER
   Define-Spin $name $SPIN_STEPPER
   Define-Tilt $name $TILT_STEPPER
   Define-Height $name $Z_STEPPER
   Define-Analyser $name $ANAL_STEPPER
}

#Add_Stepper_Module $netvar_name VME addressdata

