# These routines are used to "calibrate" the Goniometer steppers.
# The range values for each stepper are built in to the VME software.
# At any position the current real position can be entered by an operator.
#
# The range values are then changed by the difference between the
# position seen by the system and the position measured by the operator
# This means subsequent reads will return the correct real position
# The new range setting is saved in the experiment options
# After a crate reboot the ranges are reset to the original defaults.
# The saved ranges will be applied to overwrite the defaults when
# a new meis-session is started or Setup is executed
#
#
# This is not very good...
# These default values have been copied from the C source for the VME register server.
# If it ever changes this must be changed too
#
set stepDefaultRange(Goniometer.Rota) "0x7ef582:360deg,0x8084d0:-100deg"
set stepDefaultRange(Goniometer.Spin) "0x7f9c00:0deg,0x80af00:220deg"
set stepDefaultRange(Goniometer.Tilt) "0x7ff472:-5.25deg,0x800b8e:5.25deg"
#set stepDefaultRange(Goniometer.Height) "0x7eddc0:-14mm,0x8264c0:14mm"
set stepDefaultRange(Goniometer.Height) "0x7a54c0:-14mm,0x8bf7c0:14mm"
set stepDefaultRange(Analyser.Angle) "0x8029bf:0deg,0x7fb7ac:360deg"

#
#
proc defaultRange {stepReg} {
   global stepDefaultRange
   set nr $stepDefaultRange($stepReg)
   if {$nr != ""} then {
      write-attributes $stepReg.Request "-r=$nr"
      write-attributes $stepReg "-r=$nr"
      inform sys unset_Options $stepReg.Range
      inform sys update_Options
   }
}
#
proc findErrStepper {stepReg new} {
   set en [read-reg $stepReg.Enable]
   write-reg $stepReg.Enable 0
   set origA [read-reg $stepReg.Request]

# Change the register attribute to read raw values
   write-attributes $stepReg.Request "-s=1"
# read where we (think we) are
   set origD [read-reg $stepReg.Request]

# Change the register value to where we (really) are
   write-reg $stepReg.Request $new
# read where we (really) are
   set newD [read-reg $stepReg.Request]

# reset things to how they were
   write-reg $stepReg.Request $origA
   write-attributes $stepReg.Request "-s=0"

   write-reg $stepReg.Enable $en

# return the (raw value) difference between where we are and should be
   return [expr $newD - $origD]
}


proc calcRangeStepper {stepReg errD} {
# If there's no difference do nothing
   if {!$errD} {return ""}

# read attributes, extract the range, then raw min & max
# subtract the error from the raw min & max values
# this a gives same engineering values, same range but new min, max 
# therefore a new converted value for any raw value

   set att [read-attributes $stepReg.Request]

   set r [string first -r= $att]
   set i [string first -i= $att]
   set s [string first -s= $att]

   set init [string range $att [expr $i+3] [expr $s-2]]

   set range [string range $att [expr $r+3] [expr $i-2]]

   set i [string first 0x $range]
   set x [expr [string first : $range $i] - 1]
   set minD [string range $range $i $x]
   set NminDx [format "0x%08x" [expr $minD - $errD]]
   set Nrange [string replace $range $i $x $NminDx]

   set i [string first 0x $range $x]
   set x [expr [string first : $range $i] - 1]
   set maxD [string range $range $i $x]
   set NmaxDx [format "0x%08x" [expr $maxD - $errD]]
   set Nrange [string replace $Nrange $i $x $NmaxDx]

   return $Nrange
}

proc calibStepper {stepReg new} {
   if {$new == "Default"} {
      global stepDefaultRange
      set nr $stepDefaultRange($stepReg)
   } else {
      set d [findErrStepper $stepReg $new]
      set nr [calcRangeStepper $stepReg $d]
   }
   if {$nr != ""} then {
      write-attributes $stepReg.Request "-r=$nr"
      write-attributes $stepReg "-r=$nr"
      if {$new == "Default"} {
         inform sys unset_Options $stepReg.Range
      } else {
         inform sys set_Options $stepReg.Range "$nr"
      }
      inform sys update_Options
   }
}

proc setup-steppers-range {} {
   global stepRegs
   foreach stepReg $stepRegs {
      set Nrange [inform sys inquire Options $stepReg.Range]
# format should be similar to: 0x007ff401:-5.250 deg,0x00800b1d:5.250 deg
# do some simple checks: look for at least 2 "0x", at least 2 ":", 1 ","
# also last (in fact should be 2nd) "0x" should immediately follow the ","
      set x0 [string first 0x $Nrange]
      if {$x0 == -1} {continue}
      set x1 [string last 0x $Nrange]
      if {$x1 == -1} {continue}
      if {$x0 == $x1} {continue}
      set c0 [string first : $Nrange]
      if {$c0 == -1} {continue}
      set c1 [string last : $Nrange]
      if {$c1 == -1} {continue}
      if {$c0 == $c1} {continue}
      if {$c0 <= $x0} {continue}
      if {$c1 <= $x1} {continue}
      set m0 [string first , $Nrange]
      if {$m0 == -1} {continue}
      set m1 [string last , $Nrange]
      if {$m1 == -1} {continue}
      if {$m0 != $m1} {continue}
      if {[expr $x1 - $m1] != 1} {continue}

      write-attributes $stepReg.Request "-r=$Nrange"
      write-attributes $stepReg "-r=$Nrange"
   }
}

