#  ibw-to-midas, import Igor Binary Wave file as midas spectrum
#  based on copy, save, export and import spectra

#  Dec 1 1998  (made using speccopy as a start point)

#  IBW import spectrum from Igor Binary Wave (Windows/Linux only)
#  based on common source which supports speccopy (SPECCOPY); import (IDB) and export (EXP)

set started 0

set frame [uppercase [Get_My_Name]]

     set frametitle "Importing Spectra from Igor Binary Wave"
     set icontitle "Igor"


#    source resource variables
      set src-resources {}
      set src-resource ""
      set src-directory ""
      set src-filelist ""
      set src-dirlist ""
      set src-selection ""
      set defaultsrcresource ""
      set src-sorttypes {"alphabetic" "numeric" "unsorted"}
      set src-sorttype "numeric"
      set src-patternmenu {"" "*.ibw"}
      set src-pattern "*.ibw"

#    destination resource variables
      set destn-resources {direct}
      set destn-resource ""
      set destn-directory ""
      set destn-filelist ""
      set destn-dirlist ""
      set destn-selection ""
      set defaultdestnresource direct
      set destn-sorttypes {"alphabetic" "numeric" "unsorted"}
      set destn-sorttype "numeric"
      set destn-patternmenu {"" "*.tile" "*.tile.*" "*.tile.msf" "*.msf"}
      set destn-pattern ""

set preferences(direct.directory) $env(HOME)

set abortflag 0

set selection {}

   set option 0

#    option for IBW
#    0  import only
#    1  import and display

   set idboptionmenu [list "Import Only" "Import and Display"]

   proc click-option args {global option; set option [first $args]}

proc click-abort {args} {global abortflag; set abortflag 1}

#    destination spectrum access
proc make-destn-resource-list {} {
  upvar #0 destn-resources resources

   set z [catch {eg "(experiment ?resources)"} m]
   if {$z==0} then {set rs $m} else {set rs ""}

   set resources {direct}

   set count 0
   foreach r $rs {
      set type [second [fourth $r]]
      set role [second [fifth $r]]
      if {$type == "spectrum" && $role == "disc"} then {
         incr count
         if {[lsearch $resources $role] == -1} then {lappend resources $role}
      }
   }

   if {$count > 1} then {
      foreach r $rs {
#    next scan through resources and add instances of all disc spectrum servers to the list
         set type [second [fourth $r]]
         set role [second [fifth $r]]
         set name [second [first $r]]
         if {$type == "spectrum" && $role == "disc"} then {lappend resources $name}
      }
   }
}

#    IBW only has a destination spectrum resource
     proc make-src-resource-list {} {global src-resources; set src-resources ""}

proc compare-numeric {a b} {
   set pat "(\[^0-9\]+)(\[0-9\]+)(.*)(\.*)"
   set za [regexp $pat $a a0 a1 a2 a3 a4]
   set zb [regexp $pat $b b0 b1 b2 b3 b4]
   if {$za==0 || $zb==0} then {return [string compare $a $b]}
   if {"$a"!="$a0" || "$b"!="$b0"} then {return [string compare $a $b]}
   set z [string compare $a1 $b1]
   if {$z!=0} then {return $z}
   set z [expr int($a2.0-$b2.0)]
   if {$z!=0} then {return $z}
   return [string compare $a $b]
}

proc sort-filelist {which} {
   upvar #0 $which-filelist filelist
   upvar #0 $which-sorttype sorttype
   set ll {}
   foreach f $filelist {lappend ll [second $f]\\[first $f]}
   if {"$sorttype"=="numeric"} then \
      {set ll [lsort -command compare-numeric $ll]} \
   elseif {"$sorttype"=="alphabetic"} then \
      {set ll [lsort -ascii $ll]}
   set filelist {}
   foreach f $ll {
      set s [split $f "\\"]
      lappend filelist [list [second $s] [first $s]]
   }
}

proc get-resource {which} {
    upvar #0 $which-resource resource
    if {$resource == "" || $resource == "direct"} then {return ""} \
       else {return "/$resource"}
}

proc get-directory {which} {
   upvar #0 $which-resource resource
   upvar #0 $which-directory directory

    set r [get-resource $which]
    if {$r == "" } then {return "$directory"}
    if {$directory == ""} then {return $r}
    return [file join $r $directory]
}

proc spec-type t {
   if {$t==1} then {return s}
   if {$t==2} then {return d}
   if {$t==3} then {return -}
   return ?
}

proc make-dir-line {t s} {
   if {$s=="." || $s==".."} then {return ""}
   if {[string index $s 0]=="."} then {return ""}
   if {$t==1 || $t==2 || $t==3} then {return [format "%s %16s" [spec-type $t] $s]}
   return ""
}

proc make-dirlist {which} {
   upvar #0 $which-filelist filelist
   upvar #0 $which-dirlist dirlist

   set dirlist {}
   foreach p $filelist {
      set t [first $p]
      set s [second $p]
      set l [make-dir-line $t $s]
      if {$l != ""} then {lappend dirlist $l}
   }
}

proc match-pattern {which p} {
   upvar #0 $which-filelist filelist
   set ll {}
   foreach f $filelist {if {[string match $p [second $f]]} then {lappend ll $f}}
   return $ll
}

proc read-folder {path} {
      set z [catch {eg "(directory :path '$path' ?names)"} m]
      if {$z!=0} then {
         if {$m == 0x30004} then {
            set-footer "Error $m attempting to read directory $path - perhaps it has been deleted"
            after 1000; after 1 click-up
         } else {
            display-eg-error-report self $m "Attempting to read directory" $path
         }
         return 0
      }
      set ll [lsort [strip-dots $m]]
      set m {}

            foreach f $ll {
               if {[file isdirectory [file join $path $f]]} then {
                  lappend m [list 2 $f]
               } else {
                  lappend m [list 3 $f]
               }
            }
      return $m
}

proc read-directory {which} {
   upvar #0 $which-resource resource
   upvar #0 $which-directory directory
   upvar #0 $which-filelist filelist
   upvar #0 $which-pattern pattern
   global frame

   set path [get-directory $which]

   if {$which == "src"} then {

#      set m [:directory files :path $path]
#      set l {}
#      foreach line $m {
#         set name [first $line]
#         if {$name == "." || $name == ".."} then continue
#         set c [string range [second $line] 0 0]
#         if {$c == "-"} then {lappend l [list 1 $name]}
#         if {$c == "d"} then {lappend l [list 2 $name]}
#      }
#      set filelist $l
      set filelist [read-folder $path]

   } else {

      set z [catch {eg "(spectrum :path '$path' ?names)"} m]
         if {$z!=0} then {
            display-eg-error-report self $m "Attempting to read directory" $path
            clear-footer
         }
      set filelist $m

   }

      if {"$pattern" != ""} then {
         set z [catch {match-pattern $which $pattern} m]
         if {$z!=0} then {
            display-error-report self $m "Attempting to pattern match" $path
            clear-footer
         } else {set filelist $m}
      }
   sort-filelist $which
   make-dirlist $which
}

proc paint-directory {which} {
   upvar #0 $which-directory directory
   upvar #0 $which-dirlist dirlist

   if {$which == "src"} then {set W S} else {set W D}
   :modify [set W]DIRECTORY :value $directory
   :modify [set W]LIST :append F :strings $dirlist :see 0
}

proc click-resource {which v} {
    global env
   upvar #0 $which-resource resource
   if {$which == "src"} then {set W S} else {set W D}
   set resource $v
   :modify [set W]RESOURCE :value $resource
   if {$resource == "direct"} then {set directory $env(HOME)} \
      else {set directory ""}
   click-directory $which $directory
}

proc click-sorttype {which v} {
   upvar #0 $which-sorttype sorttype
   if {$which == "src"} then {set W S} else {set W D}
   set sorttype $v
   :modify [set W]RESOURCESORTTYPE :label $sorttype
   read-directory src
   paint-directory src
}

proc click-pattern {which v} {
#    selection pattern applied to source directory only
#    selection pattern applies to both directories
   upvar #0 $which-selection selection
   upvar #0 $which-pattern pattern

   if {$which == "src"} then {set W S} else {set W D}
   set pattern $v
   :modify [set W]PATTERN :value $pattern
   set selection {}

   read-directory $which
   paint-directory $which
}

proc click-directory {which d} {
   global preferences
   upvar #0 $which-directory directory
   upvar #0 $which-selection selection
   upvar #0 $which-resource resource

   if {$d != ""} then {set directory [eval file join [file split $d]]} else {set directory ""}
   set selection {}

   set preferences($resource.directory) $directory

   set-busy
   read-directory $which
   paint-directory $which
   clear-busy
}

proc click-up {which} {
   global preferences
   upvar #0 $which-directory directory
   upvar #0 $which-selection selection
   upvar #0 $which-resource resource

   set i [string last / $directory]
   if {$i < 0} then {set directory ""}
   if {$i == 0} then {set directory "/"}
   if {$i > 0} then {set directory [string range $directory 0 [expr $i-1]]}
   set selection {}

   set preferences($resource.directory) $directory

   read-directory $which
   paint-directory $which
}

proc click-down {which} {
   global preferences
   upvar #0 $which-directory directory
   upvar #0 $which-dirlist dirlist
   upvar #0 $which-selection selection
   upvar #0 $which-resource resource

   if {[llength $selection] == 0} then {
      set-footer "no directory/spectrum selected!!!"
      return
   }
   if {[llength $selection] > 1} then {
      set-footer "multiple selections!!!"
      return
   }

   set target [lindex $dirlist [first $selection]]
   if {[string first d $target] != 0} then {
      set-footer "no directory selected!!!"
      return
   }
   if {[second $target] == "."} then {click-redisplay; return}
   if {[second $target] == ".."} then {click-up $which; return}
   if {$directory == ""} {append directory [second $target]} {
        set directory [string trimright $directory /]
        append directory / [second $target]
   }
   set selection {}

   set preferences($resource.directory) $directory

   read-directory $which
   paint-directory $which
}

proc click-select {which opt} {
   upvar #0  $which-selection selection

   if {$which == "src"} then {set W S} else {set W D}
   if {"$opt" == "none"} then {
      :modify [set W]CHOSEN :value ""
      :modify [set W]LIST :select -1
   }
   set selection [[:path [set W]LIST].list curselection]
}

proc click-list {which s} {
   upvar #0 $which-dirlist dirlist
   upvar #0 $which-selection selection

#   insert-debug "click-list $which $s"

   if {$which == "src"} then {set W S} else {set W D}
    set dd 0
    set ss 0
    foreach e $s {
        if {[lsearch $selection $e] != -1} then {
           :modify [set W]LIST :deselect $e
        }
    }
    :modify [set W]CHOSEN :value ""
    set selection [[:path [set W]LIST].list curselection]
    foreach e $selection {
        if {[first [lindex $dirlist $e]] == "d"} then {set dd 1}
        if {[first [lindex $dirlist $e]] == "s"} then {set ss 1}
        :modify [set W]CHOSEN :value [second [lindex $dirlist $e]]
    }

#   insert-debug "dd=$dd; ss=$ss; selection=$selection"
   return
}

proc click-enter w {
   if {$w == "P1"} then {upvar #0 src-selection selection; set W S} \
      else {upvar #0 destn-selection selection; set W D}
   foreach i $selection {:modify [set W]LIST :select $i}
}

proc do-reshow {which} {
   read-directory $which
   paint-directory $which
}

proc click-redisplay {} {
   do-reshow src
   do-reshow destn
}

   proc click-execute {args} {
      upvar #0 src-dirlist sdirectorylist
      upvar #0 src-selection sselection
      upvar #0 destn-dirlist ddirectorylist
      upvar #0 destn-selection dselection
      global option

      set from [get-directory src]
         
      set to [get-directory destn]

      if {$sselection == ""} then {set-footer "no source selected"; return}
      set total 0
      set list ""
         set target [lindex $sdirectorylist $sselection]
         set file [last $target]
         if {[first $target] == "d"} then {set-footer "ignoring $target since it's a directory"; return}
         set fromfile [file join $from $file]

         if { $dselection != {} } then {
            set dtarget [lindex $ddirectorylist $dselection]
            set egfile [last $dtarget]
            if {[first $dtarget] == "d"} then {set-footer "ignoring $dtarget since it's a directory"; return}
            set tofile   [file join $to $egfile]
            set msg "importing $fromfile with $tofile ..."
            set-footer $msg
            set z [catch {exec ibw-to-midas $fromfile $tofile} m]
#            do-reshow destn
         } else {
            set egfile ""
            set tofile $egfile
            set msg "importing $fromfile ..."
            set-footer $msg
            set z [catch {exec ibw-to-midas $fromfile} m]
#            do-reshow src
         }

         if {$z == 0} then {
            set-footer "$m"
            incr total
            append list "$m "
         } else {
            midas-warning "$m"
            return
         }
      click-redisplay
      click-select src none
      click-select destn none
      set-footer "$total [pluralise-spectrum $total] successfully imported"
      if {$option > 0 && $total > 0} then {inform sd show $list}
   }


proc init-src {} {
   global env preferences frame 
   upvar #0 src-resource resource
   upvar #0 src-directory directory

      set directory $preferences(direct.directory)

#    process for system independance

   if {$directory != ""} then {set directory [eval file join [file split $directory]]}

   read-directory src
   paint-directory src
}

proc init-destn {} {
   global env preferences frame 
   upvar #0 destn-directory directory
   upvar #0 destn-resource resource

      upvar #0 destn-resources resources
      global defaultdestnresource

      make-destn-resource-list

      if {[lsearch $resources $defaultdestnresource] == -1} then {
        set resource [first $resources]
      } else {
        set resource $defaultdestnresource
      }
      if {$resource == "direct"} then {set directory  $preferences(direct.directory)}

      :modify DRESOURCEMENU :strings $resources
      :modify DRESOURCE :value $resource

#    process for system independance

   if {$directory != ""} then {set directory [eval file join [file split $directory]]}

   read-directory destn
   paint-directory destn
}

proc double-click arg {
   global debug frame

   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}
# insert-debug "double-click entered with argument \{$arg\}"
   set f [lindex $arg 0]
   set w [lindex $arg 2]
   set v [lindex $arg 3]

   if {$w == "SLIST"} then {set which src} else {set which destn}
      upvar #0 $which-selection selection

   switch -glob $w {
        *LIST  {
#            if {[llength $selection_last] > [llength $selection]} then {set selection $selection_last}
            click-down $which
        }
        default      {puts stdout $arg}
   }
}
proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   clear-footer
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   if {$c==":Enter"} then {click-enter $w; return}
   switch $w  {
      SRESOURCEMENU   -
      SRESOURCE       {click-resource src $v}
      SRESOURCESORTTYPE   {click-sorttype src $v}
      DRESOURCEMENU   -
      DRESOURCE       {click-resource destn $v}
      SPATTERNMENU    -
      SPATTERN        {click-pattern src $v}
      DPATTERNMENU    -
      DPATTERN        {click-pattern destn $v}
      SDIRECTORY      {click-directory src $v}
      DDIRECTORY      {click-directory destn $v}
      DDIR            {click-dname $v}
      SLIST           {click-list src [lindex $arg 4]}
      DLIST           {click-list destn [lindex $arg 4]}

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc start args {
   global started defaultsrcresource

   if {$started == 0} then {make-frame; load_preferences}
   set started 1
   open-frame

   if {[first $args] != ""} then {set defaultsrcresource [first $args]} else {set defaultsrcresource "direct"}

   init-src
   init-destn

   click-redisplay
}


proc make-frame {} {
   upvar #0 src-resources sresources
   upvar #0 src-sorttypes ssorttypes
   upvar #0 destn-resources dresources
   global frame idboptionmenu
   upvar #0 src-patternmenu spatternmenu
   upvar #0 src-pattern spattern
   upvar #0 destn-patternmenu dpatternmenu
   upvar #0 destn-pattern dpattern

set rows 12
set cols 36

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-name "[self icon-title]"

  :panel P1 :layout L :padx 5 :pady 5
     :layout H; :row-gap 5; :col-gap 5

   :enable P1 Enter

     :message SRESOURCE :label "Source (Igor file)" 
     :message SPACE :label "          " :w 15
     :menubutton SRESOURCESORTTYPE :label "numeric" :menu $ssorttypes
     :next-row T
     :text SDIRECTORY :label Directory :w [expr $cols - 4] :shifters T
     :next-row T
     :list SLIST :rows $rows :cols $cols :exclusive F :double-click T
     :next-row T
     :button SDESELECT :label "Deselect All" :command "click-select src none"
     :hskip 85
     :button SUP   :label Up   :command "click-up src"
     :button SDOWN :label Down :command "click-down src"
     :next-row T
     :text SCHOSEN :label "Selected file: " :w [expr $cols - 4] :value "" :deaf T
     :next-row T
     :menubutton SPATTERNMENU :label Selection :menu $spatternmenu
     :text SPATTERN :label "" :w [expr $cols - 7] :value $spattern

  :panel P2 :layout L :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 5

   :enable P2 Enter

     :menubutton DRESOURCEMENU :label "Spectrum Resource" :menu $dresources
     :text DRESOURCE :label "" :w 10 :value [first $dresources]

     :next-row T
     :text DDIRECTORY :label Directory :w [expr $cols - 4] :shifters T
:row-gap 5
     :next-row T
     :list DLIST :rows $rows :cols $cols :exclusive T  :double-click T
     :next-row T
     :button DDESELECT :label "Deselect All" :command "click-select destn none"
     :hskip 85
     :button DUP   :label Up   :command "click-up destn"
     :button DDOWN :label Down :command "click-down destn"
     :next-row T
     :text DCHOSEN :label "Selected file: " :w [expr $cols - 4] :value "" :deaf T
     :next-row T
     :menubutton DPATTERNMENU :label Selection :menu $dpatternmenu
     :text DPATTERN :label "" :w [expr $cols - 7] :value $dpattern

   :panel BUTTONS :layout B :padx 5 :after FOOTER
     :layout H; :col-gap 20; :row-gap 5

     :choice-stack OPTION    :label Option     :strings $idboptionmenu
     :hskip 30
#     :next-row T
     :button       EXECUTE   :label "Execute"  :command click-execute
     :hskip 50; :col-gap 50

     :button REDISPLAY   :label Redisplay  :command click-redisplay
     :button HELP        :label Help       :command click-help

   :show T
   update idletasks
}

proc click-help {} {
 midas-report [eval exec "ibw-to-midas -h"]
}
