
set frame INT
set frametitle "Current Integrator"
set icontitle  "Integrator"

set server /VME

set dosestep  600
set dosemax  [expr {$dosestep*10000}]	
set doseunit  C

set intrunning 0
set intpc 0
set intafterid ""

set auto_cup 1

#
#                                   proc            widgets 
#
set table(Integrator.PPS)             {update-txt   PPS        {}}
set table(Integrator.Current.Range)   {update-range RANGE CURRENT}
set table(Integrator.Dose.Step)       {update-step  DOSE     UNIT}
set table(Integrator.Current)         {update-current   CURRENT    CURRVAL}
set table(Integrator.Dose)            {update-dose  DOSE       {}}
set table(Integrator.Percent)         {update-percent   PERCENT    PERCVAL}
set table(Integrator.Running)         {update-int   RUN        {}}

loadsource mpxcontrol.defn

proc click-help {} {inform mb goto {Current Integrator}}

proc check-acquisition mess {
	set going [read-reg MEIS.Go]
	if {$going} then {
		display-error-report self "ERROR" "MEIS Data Aquisition Going" $mess
		error 1
	}
}

proc update-step {w v u} {
	global debug dosestep dosemax doseunit
	if {$debug} then {puts stdout "update-step $w $v $u"}
	set dosestep [first $v]	
	set dosemax  [expr {$dosestep*10000}]	
	set doseunit [second $v]	
	:modify $w :max $dosemax
	:modify $u :label "$doseunit"
}

proc update-current {w v a} {
	global debug 
	if {$debug} then {puts stdout "update-current $w $v $a"}
	update-int $w $v {}
	set r Integrator.Current.Range
	set max [scan-int [read-reg $r]]
	set val [expr [scan-int $v] * $max / 4095 ]

	update-int $a $val {}
}

proc update-percent {w v a} {
	global debug 
	if {$debug} then {puts stdout "update-percent $w $v $a"}
	update-int $w $v {}
	update-int $a [scan-int $v] {}
}

proc update-dose {w v a} {
	global debug dosestep dosemax doseunit
	if {$debug} then {puts stdout "update-dose $w $v $a"}
	scan $v %f%s val unit
	if {$unit != $doseunit} then {
		set val [expr $val*1000]
	}
	set val [format %0.0f $val]
	:modify $w :value $val
}

proc click-dose v {
	global debug dosestep dosemax doseunit
	if {$debug} then {puts stdout "click-dose $v"}

	set reg Integrator.Dose
	set value [read-reg $reg]

	scan $value %f%s val unit
	if {$unit != $doseunit} then {
		set val [expr $val*1000]
	}
	set val [first [split [expr {$val+0.5}] .]]


	if {$v > $val && $v < [expr $val+$dosestep]} then {
		set v [expr $val+$dosestep]
	}
	if {$v < $val && $v > [expr $val-$dosestep]} then {
		set v [expr $val-$dosestep]
	}

	set v "$v $doseunit"
	write-reg $reg $v
	click-redisplay
	set-footer "$reg set to $v..."
}

proc update-range {w v wc} {
	global debug
	if {$debug} then {puts stdout "update-range  $w $v $wc"}
	set ticks [expr {[string index $v 0]+1}]
	:modify $w :value "$v"
#        :modify $wc :max-tick-label "$v" :ticks $ticks
        :modify $wc :max-tick-label "$v"
}

proc click-reset {} {
	write-reg Integrator.Reset 1
	write-reg Integrator.Reset 0
	click-redisplay
}

proc click-convert {} {
	write-reg Integrator.Convert 1
	redisplay-data
}

proc move-cup out {
	global EGreport auto_cup
	set tmpReport $EGreport
	set EGreport -1
	write-mpx-reg Buzzer.switch 1
	after 500
	if {$auto_cup} {write-mpx-bits FaradayCup.control 1 $out}
	write-mpx-reg Buzzer.switch 0
	set EGreport $tmpReport
}

proc do-run {v m c} {
	set reg Stop
	set cupout 0
	if {$v} then {set reg Start; set cupout 1}
	if {$c} then {check-acquisition "Attempting To $reg Integrator"}
	if {$m} then {move-cup $cupout}
	write-reg Integrator.$reg 1
	write-reg Integrator.$reg 0
	write-reg Integrator.$reg 1
}

proc click-run v {
	do-run $v 1 1
	click-redisplay
}

proc redisplay {} {
	global debug intrunning intpc

	set list [read-regs I*]
	foreach pair $list {update-pair $pair}
	foreach pair $list {
		if {[first $pair] == "Integrator.Dose"} then {update-pair $pair}
		if {[first $pair] == "Integrator.Running"} then {set intrunning [second $pair]}
		if {[first $pair] == "Integrator.Percent"} then {set intpc [first [second $pair]]}
	}
}

proc click-aq-redisplay v {do-run $v 1 0; click-redisplay 1}

proc click-redisplay {args} {
	global intafterid intrunning intpc debug

	set intrunning 0
	set intpc 0
	redisplay
	clear-footer

	if {$intrunning} then {
		if {$intafterid!=""} then {catch {kill-after $intafterid}}
		if {$intpc > 101} then {
			move-cup 0
			set-footer "Current Integrator Overrun"
			display-error-report self "Overrun" "Abort Data Acquisition Run"
		} else {
			if {[llength $args]} then {
			   set intafterid [after 2000 click-redisplay $args]
			} else {
			   set intafterid [after 2000 click-redisplay]
			}
			set-footer "Current Integrator Running"
		}
	} else {
	   if {[llength $args]} then {move-cup 0}
	   write-reg MEIS.Go 0
	}
}

proc redisplay-data {} {
	global debug 
	set r Integrator.Data
	set v [read-reg $r]
	update-pair "$r $v"
}

proc click arg {
   global debug 
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      REDISPLAY {click-redisplay}
      RUN       {click-run $v}
      DOSE      {click-dose $v}
      HELP      {click-help}
      default   {click-on $w $v}
   }
}

proc make-frame {} {
  global dosemax

set col1 7c

	:frame [self frame-name] :label "[self frame-title]" :resize F :show-footer T  \
		:icon-name "[self icon-title]"

	:panel P :padx 5 :pady 5
        :layout H; :col-gap 10; :row-gap 16

	:text   RANGE :w 5 :deaf T :label "Range :" :valuex $col1 :bold T
	:gauge  CURRENT :min 0 :max 4095 :label "Current :" :ticks 3  \
            :gauge-width 120 :bold T :valuex 12c
	:col-gap 0
	:text   CURRVAL :w 3 :deaf T :label "" :bold T
	:col-gap 10; :next-row T
	:text   PPS :w 5 :deaf T :label "PPS :" :valuex $col1 :bold T
	:gauge  PERCENT :min 0 :max 100  :max-tick-label "100%" :ticks 11 \
            :gauge-width 120 :label "Integrated :" :bold T :valuex 12c
	:col-gap 0
	:text   PERCVAL :w 3 :deaf T :label "" :bold T
	:col-gap 10; :next-row T
	:slider DOSE    :min 0 :max $dosemax :resolution 1 :entry-width 10 :label "Dose :" :valuex $col1 :bold T
        :hskip -14
	:message UNIT   :label "C" :bold T
	:row-gap 10; :next-row T
	:choice RUN  :label "Run :"  :strings [list "Stop" "Start"] :valuex $col1 :bold T
        :hskip 50
	:button REDISPLAY
	:button HELP

   :show T

position-frame 310 700

}

#                  check the running bit
proc acqRunning {} {
   global intrunning
   set intrunning [read-reg Integrator.Running]
}

proc fcmode v {global auto_cup; set auto_cup $v}
