# Perform k squared correction

set frame K2
set frametitle "Perform k Squared Correction"
set icontitle K2

set autoshow 1

set directory ""
set spectrum ""
set result ""
set comment ""
set subtract 0
set mode 0
set beammass 4
set targetmass 60
set energy 100
set offset 0.0
set resultname ""
set resultresult ""
set action ""


proc last-n {p n} {
   set i [expr [llength $p]-$n]
   lrange $p $i end
}

proc file-front {n} {return [join [front [split $n "/"]] "/"]}

proc file-last {n} {return [last [split $n "/"]]}

proc list-take {as is} {
   set ll {}
   foreach e $is {lappend ll [lindex $as $e]}
   return $ll
}

proc click-help {} {inform mb goto {(db)projecting}}

proc get-comment {} {
   global comment directory spectrum
   set z [catch {EG "spectrum :name '$directory/$spectrum' ?label"} m]
   if {$z==0} then {
	set comment $m
	:modify K2-COMMENT :value "$comment"
   }
}

proc click-spectrum {v} {
   global debug spectrum
   if {$debug} then {puts stdout "$v"}
   set spectrum $v
   get-comment
   clear-footer
}

proc click-execute {} {
   global  directory spectrum result 
   if {$spectrum=={}} then {set-footer "No spectrum specified"; return}
   if {$result=={}} then {set-footer "No result spectrum specified"; return}
   if {$directory=={}} then {set-footer "No directory specified"; return}
   set z [catch {eg "(spectrum :name '$directory/$spectrum' ?spec)"}]
   if {$z!=0} then {set-footer "$directory/$spectrum not a spectrum"; return}
   set z [catch {eg "(spectrum :name '$directory/$result' ?spec)"}]
   if {$z==0} then {confirm-ksquared $directory/$result; return}
   ksquared-continue 2
}

proc ksquared-continue {v} {
   global directory result comment spectrum targetmass beammass energy offset autoshow mode

   if {!$v} then {set-footer "ksquared cancelled"; return}

   set frompath $directory/$spectrum
   set topath $directory/$result

   if {$v==1} then {
	set-footer "Deleting $topath..."
	catch {EG "spectrum :name '$topath' !delete"}
   }

   set a "m"; if {$mode} {set a "e"}
   set z [catch {eval exec ksquared $a $frompath $topath $energy $beammass $targetmass $offset} m]
   if {$z == 0 && $m == ""} then {
      set-footer "Result spectrum $topath successfully created"
   } else {
      if {$z == 0} then {midas-report $m} \
      else {midas-warning $m; return}
   }

   inform sb "redisplay-if-showing $directory"

   EG "spectrum :name '$topath' :label '$comment'"
   catch {add-info $topath}

   if {$autoshow} then {inform sd show [transparent $topath]}
}

proc add-info s {
   global targetmass beammass energy mode
   set infolist [EG "spectrum :name '$s' :infonum 3 ?info"]
   lappend infolist "E0: \"$energy keV\"" "m1: $beammass" "m2: $targetmass"
   if {$mode} {lappend infolist "Mode: ERDA"}
   EG "spectrum :name '$s' :infonum 3 :info '$infolist'"
}


proc click arg {
   global debug
   global result directory comment autoshow mode beammass targetmass energy offset
   if {$debug} then {puts stdout "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      K2-SPECTRUM    {click-spectrum $v}
      K2-RESULT      {set result $v}
      K2-DIRECTORY   {set directory $v}
      K2-COMMENT     {set comment $v}
      K2-AUTOSHOW    {set autoshow $v}
      K2-MODE        {set mode $v}
      K2-BEAM-MASS   {set beammass $v}
      K2-TARGET-MASS {set targetmass $v}
      K2-ENERGY      {set energy $v}
      K2-OFFSET      {set offset $v}
      K2-EXECUTE     {click-execute}
      K2-HELP        {click-help}
      default        {puts stdout $arg}
   }
   clear-footer
}

proc start args {
   global started directory spectrum result
   if {[llength $args]==1} then {set args [first $args]}
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   if {[llength $args]>0} then {
	set directory [file-front [first $args]]
	set spectrum  [file-last  [first $args]]
	set result $spectrum.k2
	get-comment 
   }
   repaint
   clear-footer
}


proc make-frame {} {
  global autoshow

set col1 17c

   :frame [self frame-name] :label "[self frame-title]" :resize F  :show-footer T \
      :icon-name "[self icon-title]"

   :panel P :padx 5 :pady 5
      :layout H; :col-gap 10; :row-gap 10

   :text K2-DIRECTORY :label Directory: :w 32 :valuex $col1
   :next-row T
   :text K2-SPECTRUM  :label Spectrum:  :w 32 :valuex $col1
   :next-row T
   :text K2-RESULT    :label Result:    :w 32 :valuex $col1
   :next-row T
   :text K2-COMMENT   :label Comment:   :w 32 :valuex $col1
   :next-row T
   :number K2-BEAM-MASS   :label "Beam Mass:"   :w 4 :min 1 :max 250 :valuex $col1
   :number K2-TARGET-MASS :label "Target Mass:" :w 4 :min 1 :max 250
   :next-row T
   :slider K2-ENERGY  :label "Energy (keV):" :valuex $col1 :min 0 :max 500 :resolution 1
   :next-row T
#   :text K2-OFFSET  :label "Angle Offset (deg):" :valuex $col1
   :slider K2-OFFSET  :label "Angle Offset (deg):" :valuex $col1 :min -20 :max 20 :resolution 0.001
   :next-row T
   :checkbox K2-AUTOSHOW :label "Show Result:" :value $autoshow :valuex $col1
   :choice K2-MODE :label "Mode:" :strings [list MEIS ERDA]
   :button K2-EXECUTE :label Execute
   :button K2-HELP    :label Help...

  :show T
}   

proc repaint {} {
   global spectrum result directory autoshow beammass targetmass energy offset
   global comment mode
   :modify K2-DIRECTORY :value $directory
   :modify K2-SPECTRUM  :value "$spectrum"
   :modify K2-RESULT    :value "$result"
   :modify K2-COMMENT   :value "$comment"
   :modify K2-AUTOSHOW  :value $autoshow
   :modify K2-BEAM-MASS :value $beammass
   :modify K2-TARGET-MASS :value $targetmass
   :modify K2-ENERGY    :value $energy
   :modify K2-OFFSET    :value $offset
   :modify K2-MODE      :value $mode
}

proc repaint-spectrum {} {
   global directory spectrum result
   :modify K2-DIRECTORY :value $directory
   :modify K2-SPECTRUM  :value "$spectrum"
   :modify K2-RESULT    :value "$result"
}


proc confirm-ksquared {s} {

   set i [tk_dialog [self frame-name].quitdialog "Ksquared" \
      "spectrum $s already exists\n Please confirm that you want to overwrite it" \
      "warning" 1 "cancel" "overwrite"]
   if {$i} then {ksquared-continue 1}

}
