# operator.tcl
#
# This code was adcconf
# it has now been extended to add general "extras for experts"
# This code is used to assign ADC channel numbers to the 4 detector corners
# and to set the ADC pedestals for those channels
# It is also used to call up the ERDA control window
#

set frame OPERATOR
set frametitle "Operator"
set icontitle Operat

set server /VME

set table(CORNER_A)    {click-corner    A        {}}
set table(CORNER_B)    {click-corner    B        {}}
set table(CORNER_C)    {click-corner    C        {}}
set table(CORNER_D)    {click-corner    D        {}}

set table(PEDESTAL_A)  {click-slider v785n#1.Pedestal12    {1 1}}
set table(PEDESTAL_B)  {click-slider v785n#1.Pedestal8     {1 1}}
set table(PEDESTAL_C)  {click-slider v785n#1.Pedestal4     {1 1}}
set table(PEDESTAL_D)  {click-slider v785n#1.Pedestal0     {1 1}}

set table(ENABLE_A)    {click-enable v785n#1.Disable12   {}}
set table(ENABLE_B)    {click-enable v785n#1.Disable8    {}}
set table(ENABLE_C)    {click-enable v785n#1.Disable4    {}}
set table(ENABLE_D)    {click-enable v785n#1.Disable0    {}}

set table(ERDAMODE)    {click-erda ERDA.Enable {dummy}}

set table(v785n#1.CornerA.Channel)     {update-corner    A {}}
set table(v785n#1.CornerB.Channel)     {update-corner    B {}}
set table(v785n#1.CornerC.Channel)     {update-corner    C {}}
set table(v785n#1.CornerD.Channel)     {update-corner    D {}}

set table(v785n#1.Disable12)     {update-enable    ENABLE_A {}}
set table(v785n#1.Disable8)      {update-enable    ENABLE_B {}}
set table(v785n#1.Disable4)      {update-enable    ENABLE_C {}}
set table(v785n#1.Disable0)      {update-enable    ENABLE_D {}}

set table(v785n#1.Pedestal12)      {update-slider PEDESTAL_A        1}
set table(v785n#1.Pedestal8)       {update-slider PEDESTAL_B        1}
set table(v785n#1.Pedestal4)       {update-slider PEDESTAL_C        1}
set table(v785n#1.Pedestal0)       {update-slider PEDESTAL_D        1}

set table(TESTMODE) {click-testmode MEIS.TestMode {}}
#set table(MEIS.TestMode) {update-int TESTMODE {}}

proc click-enable {r v a} {
global EGallow
#   inform v785n click-adc-widget ENABLE$r $v
  click-int $r [expr $v^1] $a
  set started [inform v785n inquire started]
  if {$started == 0} then {
    click-redisplay
  } else {
    inform v785n click-redisplay; # NB 785n's redisplay calls back to operator
  }
}

proc update-enable {w v a} {
  update-int $w [expr $v^1] $a
}

proc click-erda {r v a} {
  inform erda click-erda $r $v $a
}

proc click-corner {r v a} {
global EGallow
  set v [expr $v * 2]
  click-int v785n#1.Corner$r.Channel $v $a
  click-redisplay
}

proc update-corner {w v a} {
  update-int  ADC_FOR_$w $v $a
  update-int  CORNER_$w [expr $v / 2] $a
}

proc click-redisplay {} {
    global table debug

if {$debug} then {insert-debug "click-redisplay"}

    set adcchans [read-regs v785n#1.Corner?.Channel]
    foreach adcchan $adcchans {
       update-pair $adcchan
       set corner [second [split [first $adcchan] .]]
       set corner [string index $corner end]
       set chan [second $adcchan]

       set old_reg [second $table(PEDESTAL_$corner)]
       catch {unset table($old_reg)}
       set table(v785n#1.Pedestal$chan) [list update-slider PEDESTAL_$corner  1]
       set table(PEDESTAL_$corner)      [list click-slider v785n#1.Pedestal$chan {1 1}]

       set old_reg [second $table(ENABLE_$corner)]
       catch {unset table($old_reg)}
       set table(v785n#1.Disable$chan)   [list update-enable ENABLE_$corner  {}]
       set table(ENABLE_$corner)        [list click-enable v785n#1.Disable$chan {}]
    }
    
    set adcpeds [read-regs v785n#1.Pedestal*]
    foreach adcped $adcpeds {
       update-pair $adcped
    }

    set enables [read-regs v785n#1.Disable*]
    foreach enable $enables {
       update-pair $enable
    }

    update-int TESTMODE [read-reg MEIS.TestMode] ""
    update-int SAVEFULL [inform sys inquire aqwsavefull] ""
#    update-int ERDAMODE [read-reg ERDA.Enable] ""

    update-int FCMODE [inform int inquire auto_cup] ""
    update-int SCANROT [inform gon inquire scan_rot_active] ""
    set v [inform gon inquire rotStepIncr]
    update-int SCANDIR [expr ($v + 1) / 2] ""

if {$debug} then {insert-debug "click-redisplay done"}
}

proc click-testmode {r v a} {
  click-int $r $v $a
  if {$v} then {
     inform sys create_test_spectra
  } else {
     inform sys delete_test_spectra
  }
  inform sys redisplay-state
#  click-redisplay
}


proc click-corrections v {
   if {$v=="Angle"} {inform correctA start}
   if {$v=="Energy"} {inform correctE start}
}

proc click-saverestore v {
   if {$v == "save"} {inform vmesaverestore click-SSRsave}
   if {$v == "restore"} {inform vmesaverestore click-SSRrestore-current;click-redisplay}
}

proc click-fcmode v {
   inform int fcmode $v
}

proc update-scanrot v {
update-int SCANROT $v ""
}

proc click-scan-rot v {
   inform gon control-scan-rot $v
   after 1000 update-scanrot [inform gon inquire scan_rot_active]
}

proc click-scan-dir v {
# expression changes 0/1 to -1/+1
   inform gon set rotStepIncr [expr ($v *2) - 1]
}

proc click-stop-steppers {} {
   global current Request
   set steppers {Goniometer.Rota Goniometer.Spin Goniometer.Tilt Goniometer.Height Analyser.Angle}
   set res 0
   foreach s $steppers {
      set Current($s) [read-reg $s]
      set Request($s) [read-reg $s.Request]
      set diff [expr abs([scan-num $Request($s)] - [scan-num $Current($s)])]
      set diff [expr ($diff * 100) + 0.5]
      set diff [first [split $diff .]]
      if {$diff < 2} {continue}

      set res 1
      write-reg $s.Request $current($s)
      write-reg $s.Zero 0
      write-reg $s.Scan 0
      write-attributes $s.Drive "-p=rw"
      write-reg $s.Drive 0
      write-attributes $s.Drive "-p=ro"
   }
   inform gon refresh
   return $res
}

proc click-save-full v {
   inform sys set aqwsavefull $v
}

proc click-halt-steppers {} {
   inform gon write-reg-nocheck Steppers.Kill 1
   midas-warning "Emergency Steppers Halt\nAll steppers now need to be referenced"
   return
}

proc click-epics s {
   inform int epicsNewServer $s
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {finish; return}
   switch $w {
      SAVEFULL  {click-save-full $v}
      REDISPLAY {click-redisplay}
      ADC {inform sys [string tolower $v]_v785n_spectra}
      CORRECT   {click-corrections $v}
      SAVERESTORE {click-saverestore [string tolower $v]}
      FCMODE    {click-fcmode $v}
      SCANROT   {click-scan-rot $v}
      SCANDIR   {click-scan-dir $v}
      HELP      {click-help}
      default   {click-on $w $v}
   }
}

set xpos 60

set New_Epics 0

proc make-frame {} {
    global rtptable modetable xpos New_Epics
    
    :frame [self frame-name] :label "[self frame-title]" :resize F  :show-footer T \
        :icon-name "[self icon-title]"

    :panel P :padx 5 :pady 5
        :layout H; :col-gap 10; :row-gap 10

    :message MFP :label "Front Panel No." :bold T
:hskip 25
    :message MADC :label "ADC" :bold T
    :message MPED :label "Pedestal (mV)" :bold T
:hskip 60
    :message MENA :label "Enable" :bold T
    :next-row T

    :number       CORNER_A :w 3 :label "Corner A:" :min 0 :max 15 :valuex $xpos
    :text         ADC_FOR_A :w 3 :label "" :deaf T
    :slider       PEDESTAL_A   :label "" :min 0 :max 4096 :resolution 1
    :checkbox     ENABLE_A   :label ""
    :next-row T

    :number       CORNER_B :w 3 :label "Corner B:" :min 0 :max 15 :valuex $xpos
    :text         ADC_FOR_B :w 3 :label "" :deaf T
    :slider       PEDESTAL_B   :label "" :min 0 :max 4096 :resolution 1
    :checkbox     ENABLE_B   :label ""
    :next-row T

    :number       CORNER_C :w 3 :label "Corner C:" :min 0 :max 15 :valuex $xpos
    :text         ADC_FOR_C :w 3 :label "" :deaf T
    :slider       PEDESTAL_C   :label "" :min 0 :max 4096 :resolution 1
    :checkbox     ENABLE_C  :label ""
    :next-row T

    :number       CORNER_D :w 3 :label "Corner D:" :min 0 :max 15 :valuex $xpos
    :text         ADC_FOR_D :w 3 :label "" :deaf T
    :slider       PEDESTAL_D   :label "" :min 0 :max 4096 :resolution 1
    :checkbox     ENABLE_D   :label ""
    :next-row T

    :col-gap 5
    :message      MODE  :label "Modes" :bold T
    :checkbox TESTMODE :label "Test histograms:"
    :checkbox SAVEFULL  :label "Auto-save full:"
#    :hskip 50
#    :checkbox ERDAMODE  :label "ERDA:"

#    :hskip 50

if {$New_Epics} {
    :row-gap 10; :next-row T; :col-gap 20
    :choice SCANROT  :label "2D Rotation Scan:"  :strings [list "Stop" "Start"]
    :col-gap 5
    :choice SCANDIR  :label ""  :strings [list "Down" "Up"]
    :col-gap 40
    :button HALTSTEP  :label "Halt Steppers" :command "click-halt-steppers"
    :next-row T;     :col-gap 20
    :text EPICS :label "EPICS Server" :w 20
    :choice FCMODE  :label "Faraday Cup Control:"  :strings [list "Manual" "Auto"]
    :next-row T
} else {
    :col-gap 20
    :button HALTSTEP  :label "Halt Steppers" :command "click-halt-steppers"

    :row-gap 10; :next-row T; :col-gap 20
    :choice FCMODE  :label "Faraday Cup Control:"  :strings [list "Manual" "Auto"]
    :choice SCANROT  :label "2D Rotation Scan:"  :strings [list "Stop" "Start"]
    :col-gap 5
    :choice SCANDIR  :label ""  :strings [list "Down" "Up"]
    :next-row T
}
    :col-gap 10
    :button REDISPLAY
    :menubutton CORRECT   :label "Corrections..."  :menu [list "Angle" "Energy"]
    :menubutton ADC :label "ADC Spectra" :menu [list "Create" "Delete"]
    :menubutton SAVERESTORE :label "Save" :menu [list "Save" "Restore"]
    :button ERDA :label "ERDA..." :command "inform erda start"
#    :button SAVE :command "inform vmesaverestore click-SSRsave"
#    :button RESTORE :command "inform vmesaverestore click-SSRrestore-current;click-redisplay"
#    :button HELP :inactive T

   :show T
}
