#    Meis Analyser PSU (ramping version) control
#     June 1996    TK version    VP
#      December 2002   SCL based on psu.tcl (as above)

set frame PSUR
set frametitle "Analyser Power Supply Ramp"
set icontitle "PSU Ramp"

set server /VME

#
#                     proc      register           args
#
set table(ENERGY)   {click-slider Analyser.Energy         {1000 6 keV}}
set table(POWER)    {click-int    Analyser.Power          {} }
#
#                                   proc            widgets                args
#
set table(Analyser.Volts+)       {update-psu      VPLUS     {}}
set table(Analyser.Volts-)       {update-psu      VMINUS    {}}
set table(Analyser.Current+)     {update-psu      IPLUS     {}}
set table(Analyser.Current-)     {update-psu      IMINUS    {}}
set table(Analyser.Energy)       {update-slider   ENERGY  1000}
set table(Analyser.Power)        {update-int      POWER     {}}

# multipliers to convert seconds/minutes/hours/days to seconds
set multipliers [list 1 60 [expr 60 * 60] [expr 24 * 60 * 60]]
set multiplier [second $multipliers]
set threshold 50000
set ramptime 10
set rampstep 1
set going 0
# Always moves at 10 seconds/step
set steptime 10
set final 0
set latest 0
set current 0

set psur_debug 0

proc continue-ramp {} {
   global rampafterid going psur_debug
   global latest current final rampstep steptime

   if {$rampafterid!=""} then {catch {kill-after $rampafterid}}
   if {$going} then {
      set rampafterid [after [expr $steptime * 1000] continue-ramp]
      set-footer "Analyser Energy Ramping in Progress"
      click-psur-redisplay
if {$psur_debug} {insert-debug "continue latest: $latest"}
      set new [expr $latest + $rampstep]
      if {$new > $final} then {set new $final}
if {$psur_debug} {insert-debug "continue new: $new"}   
if {$psur_debug} {insert-debug "continue using new: [expr round($new)]"}
      click-on ENERGY [expr round($new)]
      if {$new == $final} then {set going 0}
      set latest $new
if {$psur_debug} {insert-debug "continue current now: $current"}
if {$psur_debug} {insert-debug "continue latest now: $latest"}
if {$psur_debug} {insert-debug "continue done"}
   } else {
      click-redisplay
if {$psur_debug} {insert-debug "Analyser Energy Ramping Complete"}
      set-footer "Analyser Energy Ramping Complete"
   }
}


proc click-final v {
   global final
   set final $v
}

proc click-threshold v {
   global threshold
   set threshold $v
}

proc click-time t {
   global ramptime
   set ramptime $t
}

proc click-multiplier u {
   global multiplier multipliers
   set multiplier [lindex $multipliers $u]
}

proc click-stopgo v {
   global rampafterid going psur_debug
   global latest current final threshold multiplier ramptime rampstep steptime 

   if {$v==$going} then {return}

   if {$v} then {
      click-psur-redisplay
if {$psur_debug} {insert-debug "stopgo current: $current eV"}
if {$psur_debug} {insert-debug "stopgo threshold: $threshold eV"}
if {$psur_debug} {insert-debug "stopgo final: $final eV"}
      if {$final < $threshold || $final < $current} then {
         click-on ENERGY $final
         return
      }

      if {$current < $threshold} then {
         click-on ENERGY $threshold
if {$psur_debug} {insert-debug "current now: $current eV"}
         set pause $steptime
      } else {
         set pause 0
      }


      set latest $current
      set period [expr $ramptime * $multiplier]

if {$psur_debug} {insert-debug "total period: $period seconds"}
      set steps [expr $period / $steptime]

      if {$steps < 1} then {set steps 1}
if {$psur_debug} {insert-debug "No. steps: $steps"}
      set interval [expr $final - $latest]
if {$psur_debug} {insert-debug "interval: $interval eV"}
      set rampstep [expr $interval / $steps]
      if {$rampstep < 1} then {set rampstep 1}
if {$psur_debug} {insert-debug "ramp per step: $rampstep"}

      set going 1
      set rampafterid [after [expr $pause * 1000] continue-ramp]
   } else {
      set going 0
      continue-ramp
   }
}

proc click-help {} {inform mb goto {XVME-560 Setup}}

proc update-psu {w v a} {
   :modify $w-TXT :value "$v"
   :modify $w-GGE :value [scan-int $v]
}

proc click-psur-redisplay {} {
   global going current
   :modify STOPGO :value $going
   set list [read-regs Analyser.\[B-Z\]*]

   foreach pair $list {update-pair $pair}

#   set current [scan-int [second [lindex $list [lsearch -regexp $list Energy]]]]

   set current [second [lindex $list [lsearch -regexp $list Energy]]]
   scan $current "%f%s" val unit
   switch $unit {
      eV {set current $val}
      heV {set current [expr $val * 100]}
      keV {set current [expr $val * 1000]}
      MeV {set current [expr $val * 1000000]}
   }
}

proc click-redisplay {} {
   click-psur-redisplay
   send psu clear-footer
   send psu click-redisplay
}

proc click arg {
   global debug 
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w  {
      ENERGY-END {click-final $v}
      THRESHOLD	 {click-threshold $v}
      TIME	 {click-time $v}
      UNITS	 {click-multiplier $v}
      STOPGO	 {click-stopgo $v}
      REDISPLAY	 {click-psur-redisplay}
      HELP	 {click-help}
      default	 {click-on $w $v}
   }
}

proc make-frame {} {
    global threshold multiplier multipliers ramptime
    set ind [lsearch $multipliers $multiplier]

    set gw 130
   
   :frame [self frame-name] :label "[self frame-title]" :resize F :show-footer T  \
         :icon-name "[self icon-title]"

   :panel P :padx 5 :pady 5 
   :layout H; :row-gap 10; :col-gap 10
   :number  TIME  :label "Ramping Time :" :w 3 :valuex 16c :bold T :value $ramptime
   :choice  UNITS  :label "Units :" :strings [list "Seconds" "Minutes" "Hours" "Days"] :exclusive T :bold T :value $ind
   :row-gap 15; :next-row T
   :slider  THRESHOLD  :label "Threshold (eV) :" :min 0 :max 416667 :valuex 16c :bold T :resolution 1 :entry-width 7 :value $threshold
   :row-gap 10; :next-row T
   :slider  ENERGY-END  :label "End Energy (eV) :" :min 0 :max 416667 :valuex 16c :bold T :resolution 1 :entry-width 7
   :choice  STOPGO  :strings [list "Stop" "Go"] :exclusive T :bold T
   :row-gap 20; :next-row T
   :slider  ENERGY  :label "Set Energy (eV) :" :entry-width 7 :min 0 :max 416667 :valuex 16c :bold T :deaf T :resolution 1
   :row-gap 10; :next-row T
   :text   VPLUS-TXT :w 7 :deaf T :label "Voltage (+) :" :valuex 16c :bold T
   :gauge  VPLUS-GGE :label "" :min 0 :max 25 :max-tick-label 25kV :ticks 6 :gauge-width $gw
   :next-row T
   :choice  POWER :label "Power:" :strings [list Off On] :exclusive T :valuex 16c
   :button REDISPLAY
   :button HELP

   :show T
}
