set patternmenu {"" *.data *.tile *.k2 *.angle *.energy *.rotation *.spin *.tilt
                 *.2d* *.data* *.tile* *.k2* *.angle* *.energy* *.scan* *.test*}

set analysismenu {"tile" "k-squared" "multiply" "squash" "project" "to-ascii" "information"}

lappend normal-ACTIONs ""
lappend normal-ACTIONs "convert ibw"

set fromrun 0

loadsource info

proc click-information {} {
     global frame directory
     upvar #0 $frame-selection selection
     upvar #0 $frame-dirlist dirlist

     clear-footer
	if {$selection!={}} then {
		foreach e $selection {
			set pair [lindex $dirlist $e]
			if {[first $pair]=="s" || [first $pair]=="-"} then {
				set spec [second $pair]
			}
		}
		click-info-name [get-directory-slash]$spec
	} else {
		set-footer "no spectrum selected!!!"
	}
}

proc click-analysis v {
   switch -glob $v {
     {tile}      {call-analysis tile start}
     {k-squared} {call-analysis ksquared start}
     {multiply}  {call-analysis multiply start}
     {squash}    {call-analysis squash start}
     {project}   {call-analysis cut start}
     {to-ascii}  {call-analysis toascii start} 
     {information} {click-information}
     {exporting} {call-analysis exp start}
     default     {call-analysis sd click-$v}
   }
}

proc mksb {} {
   global env frame frametype rowsize rowoverhead
   global resources optionsmenu analysismenu showingmenu patternmenu msfmenu msf selectmode
   upvar #0 [set frametype]-ACTIONs actionmenu

# CHANGED from original
   global fromrun
# end of CHANGE

  set rows 15
  set cols 70

   :set footer_position center
   :frame [self frame-name]  :label "[self frame-title]" :show-footer T :toplevel T  \
        :icon-image "@$env(ICONHOME)/spectrum"  \
        :icon-name [self icon-title]

   set p [string range $frame 1 end]

   wm protocol [self frame-name]  WM_DELETE_WINDOW  "click-quit [self frame-name]"

   :panel T1 :layout V
   :layout H
    make-resource-menu
   :menubutton [set p]RESOURCEMENU :label Resource :menu $resources
   :text [set p]RESOURCE :label "" :w 10
   :hskip 10
   :text [set p]DIRECTORY :label Directory :w 30 :shifters T
   :hskip 5
   :button [set p]UP      :label "@$env(ICONHOME)/uparrow"    :command click-up
   :button [set p]DOWN    :label "@$env(ICONHOME)/downarrow"  :command "click-down 0"
   :hskip 10
# CHANGED from original
#    - moved CLONE & made PATTERN smaller to make window narrower
#    - added FROM
   :button [set p]CLONE     :label Clone :command click-another
   :checkbox [set p]TICK  :label ""

   :panel T2 :layout V
   :layout H
   :number [set p]FROM :label "From run:" :w 6 :min 0 :max 100000 :value $fromrun
   :menubutton [set p]PATTERNMENU :label Selection :menu $patternmenu
   :text [set p]PATTERN :label "" :w 20
   :col-gap 5
   :menubutton [set p]MSF :label [lindex $msfmenu $msf] :strings $msfmenu :w 15
   :button [set p]REDISPLAY :label Redisplay :command click-redisplay
   :button HELP
# end of CHANGE

   :panel T3 :layout V
   :list [set p]LIST  :rows $rows :cols $cols :select $selectmode  :double-click T
   bind [:path [set p]LIST].list <ButtonRelease-2> "click-show 0"
   bind [:path [set p]LIST].list <ButtonRelease-3> "click-show 1"

   :panel B :layout B :after FOOTER
   :layout H
   :menubutton [set p]SHOWING :label Show :menu $showingmenu
   :col-gap 5
   :button [set p]PREVIOUS  :command "click-show-next -1"  :label "@$env(ICONHOME)/uparrow"
   :button [set p]NEXT      :command "click-show-next 1"   :label "@$env(ICONHOME)/downarrow"
   :button [set p]STEPUP    :command "click-show-step -1"  :label "@$env(ICONHOME)/times2"
   :button [set p]STEPDOWN  :command "click-show-step 1"   :label "@$env(ICONHOME)/divide2"
   :button [set p]SELECT    :label "Select all"   :command "click-select all"
   :button [set p]DESELECT  :label "Deselect all" :command "click-select none"
   :menubutton [set p]OPTIONS  :label Options  :menu $optionsmenu
   :menubutton [set p]ANALYSIS :label Analysis :menu $analysismenu
   :menubutton [set p]ACTION   :label Actions  :strings $actionmenu

    centre-frame [self frame-name]
    open-frame [self frame-name]
    :galley_size [set p]LIST  $rows $cols
}

proc click-from f {
	global directory fromrun selection

	frame-busy
	set fromrun $f
	set selection {}
	set-footer "Scanning directory $directory..."
	read-directory
	paint-directory
	frame-idle
}

proc match-files {f from} {
   set ll {}
   foreach f $f {
      set run 0
      scan [second $f] run%d. run
      if {$run>=$from} then { 
         lappend ll $f
      }
   }
   return $ll
}

proc read-directory {} {
   global frame
   global resource directory dircount filelist env preferences
   upvar #0 $frame-pattern pattern
   upvar #0 $frame-msf msf
   set path [get-directory]

# CHANGED from original
   global fromrun
# end of CHANGE

   set preferences(directory.$resource) $directory

   if {$resource == "" && $msf != 0} then {
      set z [catch {eg "(directory :path '$path' ?names)"} m]
      if {$z!=0} then {
         if {$m == 0x30004} then {
            set-footer "Error $m attempting to read directory $path - perhaps it has been deleted"
            after 1000; after 1 click-up
         } else {
            display-eg-error-report self $m "Attempting to read directory" $path
         }
         return 0
      }
      set ll [lsort [strip-dots $m]]
      set m {}

      if {$msf==1} then {
         if {$pattern == ""} then {
            foreach f $ll {
               if {[file isdirectory [file join $path $f]]} then {
                  lappend m "2 $f"
               } else {
                  set s [last [split $f .]]
                  if {$s == "msf" || $s == "MSF"} then {
                     lappend m "1 $f"
                  }
               }
            }
         } else {
            foreach f $ll {
               if {[file isdirectory [file join $path $f]]} then {
                  lappend m "2 $f"
               } else {
                  set s [last [split $f .]]
                  if {$s == "msf" || $s == "MSF"} then {
                     if {[string match [set pattern]* $f]} then {lappend m "1 $f"}
                  }
               }
            }
         }
      } else {
         if {$pattern == ""} then {
            foreach f $ll {lappend m "3 $f"}
         } else {
            foreach f $ll {
               if {[string match [set pattern]* $f]} then {lappend m "3 $f"}
            }
         }
      }
      set filelist $m
   } else {
       if {"$pattern" != "" && $resource != "softhist"} then {
           if {"$path" != ""} then {
               if {$resource != ""} then {
                   set path $path/$pattern
               } else {
                   set path [file join $path $pattern]
               }
           } else {
               set path $pattern
           }
       }
       set z [catch {eg "(spectrum :path '$path' ?names)"} m]
       if {$z!=0} then {
         if {$m == 0x30004 || $m == 0x30008} then {
            set-footer "Error $m attempting to read directory $path - perhaps it has been deleted"
            if {$resource != "hist" && $resource != "singles"} then {
               after 1000; after 1 click-up
            } else {
               display-eg-error-report self $m "Attempting to read directory" $path
            }
         } else {
            display-eg-error-report self $m "Attempting to read directory" $path
         }
         clear-footer
         return 0
       }
       set filelist $m
       if {"$pattern" != "" && $resource == "softhist"} {set filelist [match-pattern $pattern]}
       sort-filelist $path
   }

# CHANGED from original
   if {$fromrun!=0} then {
      set filelist [match-files $filelist $fromrun]
   }
# end of CHANGE

   make-dirlist $frame
   return 1
}

proc click arg {
   global debug frame
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
# insert-debug "click entered with argument \{$arg\}"
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]

   set f  .[string tolower [string range $w 0 2]]
   set p [string range $f 1 end]

   if {$f != $frame} then {click-tick $f}
   if {$c==":quit"} then {click-quit $f; return}

   clear-footer

# CHANGED from original - added FROM line
   switch -glob $w {
        *TICK           {click-tick $f}
        *RESOURCEMENU   -
        *RESOURCE       {click-resource $v}
        *DIRECTORY      {click-directory $v}
        *FROM           {click-from $v}
        *PATTERNMENU    -
        *PATTERN        {click-pattern $v}
        *MSF            {click-msf $v}
        *LIST           {click-list [lindex $arg 4]}
        *OPTIONS        {
           switch $v {
              names       {click-details 0}
              details     {click-details 1}
              numeric     -
              unsorted    -
              alphabetic  {click-sorttype $v}
           }
        }
        *ANALYSIS       {click-analysis $v}
        *SHOWING        {
           switch $v {
              {show and deselect}   {click-show 1}
              {just show}           {click-show 0}
           }
        }
        *ACTION      {
           switch $v {
              {zero and deselect}   {click-zero 0}
              {just zero}           {click-zero 1}
              {"just zero"}         {click-zero 1}
              {delete}              {click-delete}
              
              {add/change title}         {click-title}
              {save titles}              {click-savetitles}
              {apply titles}             {click-applytitles}

#              {Zero and deselect}   {click-Zero 0}
#              {Just Zero}           {click-Zero 1}

              {append msf suffix}   {click-append-msf-suffix}
              {remove msf suffix}   {click-remove-msf-suffix}

              {convert ibw}   {inform ibw start}

              {default}             {if {$v != ""} {click-$v}}
           }
        }
        default      {puts stdout $arg}
   }
# end of CHANGE
}

proc click-meis-delete {} {
   global frame
   global resource directory
   upvar #0 $frame-selection selection
   upvar #0 $frame-dirlist dirlist


   clear-footer
   if {[catch {set selection}] != 0} then {
      set-footer "no spectrum selected!!!"
      return
   }
   if {[llength $selection] == 0} then {
      set-footer "no spectrum selected!!!"
      return
   }

   if {[llength $selection] == 1} {
      set spex "this spectrum"
   } else {
      set spex "these spectra"
   }
   if {[midas-confirmation "Are you sure you wish to delete $spex ?"] == 0} then {return}

   click-delete
}
