# sys.meis.tcl---MEIS data acquisition control frame
# based on aq.tcl---old MEIS data acquisition control frame

# 5-7-94 PHO
#   Include run number
# 6-6-94 PHO
#   Created from Eurogam histogrammer frame
#
# 16-5-03 SCL
#   Modified for new (Lynxos/PPC/V785) DA system
#   is now loaded from sys.tcl which is based on standard MIDAS DA code
#


#set frame      DA
set frametitle "MEIS Data Acquisition"
set icontitle  MEISAcq

set meis_debug 0

set VMEserver /VME

set SpectrumServer hist
set directory $SpectrumServer

set selection {}

set aqafterid ""
set aqsaved 0

# used in aqs.tcl & aqw.tcl
set intensity_file [file join $env(MIDASBASE) meis config corrections energy.default.dat]

# temporarily moved to sys.tcl - 'cos it's set up there at the moment
#set dataangle 27.5

set ERDA 0
set activelist {}

proc dirnames l {
   foreach f $l {lappend ll [first $f]}
   return $ll
}

proc specdirnames l {
   set ll {}
   foreach f $l {lappend ll [second $f]}
   return $ll
}

proc prefix-list {l p} {
   set ll {}
   foreach f $l {lappend ll $p$f}
   return $ll
}

proc zero-spectrum n {
   global SpectrumServer
   EG "spectrum :path '/$SpectrumServer' :name '$n' !zero"
}

proc redisplay-spectra {} {
   global SpectrumServer dirlist spectrumlist selection
   set dlist [lsort [specdirnames [spectra-names]]]
   if {$dirlist!=$dlist} then {
      :modify DA-LIST  :empty T :strings [set dirlist $dlist]
      set spectrumlist {}
      set selection {}
   }
}

proc refresh args {
	global aqafterid going aqsaved meis_debug

	clear-footer  
	set going 0
	redisplay-state
	if {[llength $args] || $going} then {
		if {$aqafterid!=""} then {catch {kill-after $aqafterid}}
		set aqafterid [after 5000 refresh]
		set-footer "Data Acquisition in Progress" 
	} else {
		set-footer "Data Acquisition Complete"
		inform gon scan-energy-refresh 0
		inform gon scan-rot-refresh 0
		inform int move-cup 0
		set da [click-da 0]
#		if {$da==-1} then {return}
		write-data-collection-time
		aqw-auto-save
		set aqsaved 1
		if {[is-started sd]} then {send sd refresh-once;send sd stop-refresh;send sd stop-refresh;send sd stop-refresh}
	}
}

proc list-take {as is} {
   set ll {}
   foreach e $is {lappend ll [lindex $as $e]}
   return $ll
}

proc get-spectrum-list {} {
# get the currently selected adcs
	global spectrumlist
	return $spectrumlist
}

proc click-zero {} {
   global SpectrumServer
   set c 0
   foreach spectrum [get-spectrum-list] {
      set-footer "zeroing /$SpectrumServer/$spectrum..."
      zero-spectrum $spectrum
      incr c
      }
   set-footer "total of $c spectra successfully zeroed"
   }

proc click-show {} {
   global SpectrumServer
   set list [get-spectrum-list]
   inform sd "show [prefix-list $list /$SpectrumServer/]"
}

proc click-information {} {
   global SpectrumServer
   set list [get-spectrum-list]
   click-info-name /$SpectrumServer/[last $list]
}

proc click-help {} {inform mb goto {Data Acquisition}}

proc click-server d {
   global SpectrumServer directory
   set SpectrumServer $d
   set directory $SpectrumServer
   click-redisplay
}

proc read-beam-current {} {
   set digc  [read-reg Integrator.Current]
   set range [read-reg Integrator.Current.Range]

   set unit "nA"
   scan $range "%f%s" limit unit
   set current [expr {$limit*$digc/0xfff}]

   set dp 1
   if {$limit < 500} then {set dp 2}
   if {$limit < 50}  then {set dp 3}
   if {$limit < 5}   then {set dp 4}
   set current [format %0.*f $dp $current]

   if {$current < 1.0} then {
      set current [expr {$current * 1000}]
      switch $unit {
         mA {set unit uA}
         uA {set unit nA}
         nA {set unit pA}
      }
   }
   return $current$unit
}

proc label-erda-tac-spectrum {} {
	global SpectrumServer VMEserver server EGallow

set server $VMEserver

	set run [run-num-read]
	set run "$run-ERDA"
	set time [date +%d%h%y-%H:%M]
        set beam [read-beam-current]
	set dose [read-reg Integrator.Dose]
        set energy [read-reg Analyser.Energy]
	set angle [read-reg Analyser.Angle]
        set tilt [read-reg Goniometer.Tilt]
        set spin [read-reg Goniometer.Spin]
	set rota [read-reg Goniometer.Rota]
	set height [read-reg Goniometer.Height]

	set infolist "{Dose: \"$dose\"} {Current: \"$beam\"} {Energy: \"$energy\"} {Angle: \"$angle\"} {Tilt: \"$tilt\"} {Spin: \"$spin\"} {Rotation: \"$rota\"} {Height: \"$height\"} {Time: \" secs\"}"

	set name "/$SpectrumServer/TAC"
	set title "MEIS TAC - Run $run - $time"
#	set egallow $EGallow
#	set EGallow 0x30004
#	set sp [EG "spectrum :name '$name' ?spec"]
#	set EGallow $egallow
#	if {$sp == 0x30004} {return}
	set z [catch {eg "(spectrum :name '$name' ?spec)"} sp]
	if {$z} {return}
	set numch [third $sp].0
	set maxch [expr $numch - 1.0]
	set scale [read-reg ERDA.TAC.Scale]
	set c $scale.0
	set m [expr {0.0 - ($scale / $maxch)}]
	set cal1 "linear ns 2 $c $m"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal1'"
	EG "spectrum :name '$name' !zero"
	EG "spectrum :name '$name' :infonum 3 :info '$infolist'"
}

proc label-erda-spectra {} {
	global SpectrumServer VMEserver server dataangle
	global activelist

set server $VMEserver

	label-erda-tac-spectrum
	set run [run-num-read]
	set erdarun "$run-ERDA"

	set time [date +%d%h%y-%H:%M]
        set beam [read-beam-current]
	set dose [read-reg Integrator.Dose]
        set energy [read-reg Analyser.Energy]
	set angle [read-reg Analyser.Angle]
        set tilt [read-reg Goniometer.Tilt]
        set spin [read-reg Goniometer.Spin]
	set rota [read-reg Goniometer.Rota]
	set height [read-reg Goniometer.Height]

	set dataenergy [expr [first [split $energy k]] * 0.015]
	if {$dataenergy < 0.1} then {set dataenergy 0.1}
	set fullangle [expr $dataangle * 1.40] 
	set fullenergy [expr $dataenergy * 11.0]

	set infolist "{Dose: \"$dose\"} {Current: \"$beam\"} {Energy: \"$energy\"} {Angle: \"$angle\"} {Tilt: \"$tilt\"} {Spin: \"$spin\"} {Rotation: \"$rota\"} {Height: \"$height\"} {Time: \" secs\"}"

	set name "/$SpectrumServer/2d.erda1"
	set title "MEIS 2D ERDA Window 1 - Run $run - $time"
	set sp [EG "spectrum :name '$name' ?spec"]
	set numch [first [third $sp]].0
	set c [expr {[first $angle] - $dataangle / 2.0}]
	set ma1 [expr {$dataangle / $numch}]
	set cal1 "linear deg 2 $c $ma1"
	set numch [second [third $sp]].0
	set c [expr {[first [split $energy k]] - $dataenergy / 2.0}]
	set me1 [expr {$dataenergy / $numch}]
	set cal2 "linear keV 2 $c $me1"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal1'"
	EG "spectrum :name '$name' :dim 2 :cal '$cal2'"

	set name "/$SpectrumServer/2d.erda2"
	set title "MEIS 2D ERDA Window 2 - Run $run - $time"
#	set sp [EG "spectrum :name '$name' ?spec"]
#	set numch [first [third $sp]].0
#	set c [expr {[first $angle] - $dataangle / 2.0}]
#	set ma2 [expr {$dataangle / $numch}]
#	set cal1 "linear deg 2 $c $ma2"
#	set numch [second [third $sp]].0
#	set c [expr {[first [split $energy k]] - $dataenergy / 2.0}]
#	set me2 [expr {$dataenergy / $maxch}]
#	set cal2 "linear keV 2 $c $me2"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal1'"
	EG "spectrum :name '$name' :dim 2 :cal '$cal2'"

	set name "/$SpectrumServer/2d.erda3"
	set title "MEIS 2D ERDA Window 1 - Run $run - $time"
#	set sp [EG "spectrum :name '$name' ?spec"]
#	set numch [first [third $sp]].0
#	set c [expr {[first $angle] - $dataangle / 2.0}]
#	set ma3 [expr {$dataangle / $numch}]
#	set cal1 "linear deg 2 $c $ma3"
#	set numch [second [third $sp]].0
#	set c [expr {[first [split $energy k]] - $dataenergy / 2.0}]
#	set me3 [expr {$dataenergy / $numch}]
#	set cal2 "linear keV 2 $c $me3"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal1'"
	EG "spectrum :name '$name' :dim 2 :cal '$cal2'"

	set name "/$SpectrumServer/angle"
	set title "MEIS Angle - Run $erdarun - $time"
#	set sp [eg "(spectrum :name '$name' ?spec)"]
	set sp [EG "spectrum :name '$name' ?spec"]
#	set numch [third $sp].0
	set c [expr {[first $angle] - $fullangle / 2.0}]
#	set m [expr {$fullangle / ($numch + 1.0)}]
	set cal1 "linear deg 2 $c $ma1"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal1'"

	set name "/$SpectrumServer/energy"
	set title "MEIS Energy - Run $erdarun - $time"
#	set sp [eg "(spectrum :name '$name' ?spec)"]
	set sp [EG "spectrum :name '$name' ?spec"]
#	set numch [third $sp].0
	set c [expr {[first [split $energy k]] - $fullenergy / 2.0}]
#	set m [expr {$fullenergy / ($numch + 1.0)}]
	set cal2 "linear keV 2 $c $me1"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal2'"

	set name "/$SpectrumServer/2d.full"
	set title "MEIS 2D Data - Run $erdarun - $time"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal1'"
	EG "spectrum :name '$name' :dim 2 :cal '$cal2'"

	set splist {2d.erda1 2d.erda2 2d.erda3 2d.full angle energy}
	set activelist {}
        for {set i 1} {$i <= 3} {incr i} {
		set e [read-reg ERDA.TAC.Window$i.Enable]
		if {$e == 1} then {lappend activelist 2d.erda$i}
	}
	start-show $activelist
	foreach sp $splist {
		EG "spectrum :name '/$SpectrumServer/$sp' !zero"
		EG "spectrum :name '/$SpectrumServer/$sp' :infonum 3 :info '$infolist'"
	}

	if {[read-reg MEIS.TestMode]} {
		set splist {test.adc-A test.adc-B test.adc-C test.adc-D}
		set cal "linear V 2 0.0 0.001"
		foreach sp $splist {
			set title "ADC [string index $sp 9] - Raw data - Run $erdarun - $time"
			EG "spectrum :name '/$SpectrumServer/$sp' :title '$title'"
#			EG "spectrum :name '/$SpectrumServer/$sp' !zero"
			EG "spectrum :name '/$SpectrumServer/$sp' :infonum 3 :info '$infolist'"
			EG "spectrum :name '/$SpectrumServer/$sp' :dim 1 :cal '$cal'"
		}
	}
}

proc label-online-spectra {} {
	global SpectrumServer VMEserver server dataangle
	global activelist

set server $VMEserver

	if {[read-reg ERDA.Enable]} {
            label-erda-spectra
	    return
        }

	set run [run-num-read]
	set time [date +%d%h%y-%H:%M]
        set beam [read-beam-current]
	set dose [read-reg Integrator.Dose]
        set energy [read-reg Analyser.Energy]
	set angle [read-reg Analyser.Angle]
        set tilt [read-reg Goniometer.Tilt]
        set spin [read-reg Goniometer.Spin]
	set rota [read-reg Goniometer.Rota]
	set height [read-reg Goniometer.Height]

	set dataenergy [expr [first [split $energy k]] * 0.015]
	if {$dataenergy < 0.1} then {set dataenergy 0.1}
	set fullangle [expr $dataangle * 1.40] 
	set fullenergy [expr $dataenergy * 11.0]

	set infolist "{Dose: \"$dose\"} {Current: \"$beam\"} {Energy: \"$energy\"} {Angle: \"$angle\"} {Tilt: \"$tilt\"} {Spin: \"$spin\"} {Rotation: \"$rota\"} {Height: \"$height\"} {Time: \" secs\"}"

	set name "/$SpectrumServer/2d.data"
	set title "MEIS 2D Corrected Data - Run $run - $time"
	set sp [EG "spectrum :name '$name' ?spec"]
	set numch [first [third $sp]].0
	set c [expr {[first $angle] - $dataangle / 2.0}]
	set ma [expr {$dataangle / $numch}]
	set cal1 "linear deg 2 $c $ma"
	set numch [second [third $sp]].0
	set c [expr {[first [split $energy k]] - $dataenergy / 2.0}]
	set me [expr {$dataenergy / $numch}]
	set cal2 "linear keV 2 $c $me"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal1'"
	EG "spectrum :name '$name' :dim 2 :cal '$cal2'"

	set name "/$SpectrumServer/angle"
	set title "MEIS Angle - Run $run - $time"
#	set sp [eg "(spectrum :name '$name' ?spec)"]
	set sp [EG "spectrum :name '$name' ?spec"]
	set numch [third $sp].0
	set c [expr {[first $angle] - $fullangle / 2.0}]
#	set m [expr {$fullangle / ($numch + 1.0)}]
	set cal1 "linear deg 2 $c $ma"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal1'"

	set name "/$SpectrumServer/energy"
	set title "MEIS Energy - Run $run - $time"
#	set sp [eg "(spectrum :name '$name' ?spec)"]
	set sp [EG "spectrum :name '$name' ?spec"]
	set numch [third $sp].0
	set c [expr {[first [split $energy k]] - $fullenergy / 2.0}]
#	set m [expr {$fullenergy / ($maxch + 1.0)}]
	set cal2 "linear keV 2 $c $me"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal2'"

	set name "/$SpectrumServer/2d.full"
	set title "MEIS 2D Data - Run $run - $time"
	EG "spectrum :name '$name' :title '$title'"
	EG "spectrum :name '$name' :dim 1 :cal '$cal1'"
	EG "spectrum :name '$name' :dim 2 :cal '$cal2'"

	set splist {2d.data 2d.full angle energy}
	set activelist [first $splist]
	start-show $activelist
	foreach sp $splist {
		EG "spectrum :name '/$SpectrumServer/$sp' !zero"
		EG "spectrum :name '/$SpectrumServer/$sp' :infonum 3 :info '$infolist'"
	}

	if {[read-reg MEIS.TestMode]} {
		set splist {test.adc-A test.adc-B test.adc-C test.adc-D}
		set cal "linear V 2 0.0 0.001"
		foreach sp $splist {
			set title "ADC [string index $sp 9] - Raw data - Run $run - $time"
			EG "spectrum :name '/$SpectrumServer/$sp' :title '$title'"
			EG "spectrum :name '/$SpectrumServer/$sp' :infonum 3 :info '$infolist'"
			EG "spectrum :name '/$SpectrumServer/$sp' :dim 1 :cal '$cal'"
		}
	}
}

proc write-data-collection-time {} {
   global SpectrumServer VMEserver server

set server $VMEserver

# The time is only valid after taking a dataset
# The DAQ program zeroes it at the start of a run
   if {[read-reg ERDA.Enable]} {
      set splist {2d.erda1 2d.erda2 2d.erda3}
   } else {
      set splist {2d.data 2d.full}
   }
   lappend splist "angle"
   lappend splist "energy"
   set sp [first $splist]

   set z [catch {read-reg MEIS.DataTime} time]
   if {!$z} then {
      set infolist [EG "spectrum :path '/$SpectrumServer' :name '$sp' :infonum 3 ?info"]
      for {set i 0} {$i<[llength $infolist]} {incr i} {
         set info [lindex $infolist $i]
         if {[first $info] == "Time:"} {
            set infolist [lreplace $infolist $i $i "Time: \"$time secs\""]
            break
         }
      }

      foreach sp $splist {
         EG "spectrum :path '/$SpectrumServer' :name '$sp' :infonum 3 :info '$infolist'"
      }
   }
}

proc start-show splist {
   global SpectrumServer
   set a ""

   if {[is-started sd]} then {
      set a [send sd inquire refreshafter]
      send sd stop-refresh
   }

   inform sd "show [prefix-list $splist /$SpectrumServer/]"
   if {$a == ""} then {
      inform sd click-refresh 5
   } else {
      inform sd start-refresh
   }
}

proc click-acquisition v {
	global debug meis_debug RDOUThdwre
	global SpectrumServer VMEserver server directory aqafterid aqsaved going

set server $VMEserver

	if {$v==0} then {
		set stepper ""
		set list [read-regs G*Scan]
		foreach pair $list {
			if {[second $pair] != 0} then {
				set stepper [second [split [first $pair] .]]
#insert-debug "$stepper is scanning"
				set-footer "Stopping $stepper Scan"
				inform gon click-scan Goniometer.$stepper 0 ignore
				break
			}
		}
		if {$stepper != ""} then {
			return 1
		}
		click-da 0
	}
	set go [read-reg MEIS.Go]
	if {$v==$go} then {
		return 1
	}

	if {$v==0} then {
		if {$aqafterid!=""} then {catch {kill-after $aqafterid}}
		inform gon stop-energy-scan
		inform gon stop-rot-scan
		inform gon stop-height-scan
		write-reg MEIS.Go 0
		inform int click-aq-redisplay 0
		redisplay-state
		set-footer "Data Acquisition Stopped"
		if {[is-started sd]} then {send sd refresh-once;send sd stop-refresh;send sd stop-refresh;send sd stop-refresh}
	} else {
		set list [read-regs G*Scan]
		foreach pair $list {
			if {[second $pair] != 0} then {
				display-error-report self "ERROR" "Goniometer Scan in Progress" "Attempting to Start Data Acquisition"
				redisplay-state
				return 0
			}
		}
		set list [read-regs \[An,G\]*Drive]
		foreach pair $list {
			if {[second $pair] != 0} then {
# N.B. Height can be moved on the fly
				set stepper [second [split [first $pair] .]]
				if {$stepper != "Height"} then {
					display-error-report self "ERROR" "Goniometer $stepper Stepping in Progress" "Attempting to Start Data Acquisition"
					redisplay-state
					return 0
				}
			}
		}
                if {[read-register NetVar.DACQ.Sstate] == 2} then {click-da 0}
		set da [click-da 1]
		if {$da==-1} then {return 0}
		inform gon clear-footer
		set run [run-num-incr]
		set aqsaved 0
		label-online-spectra
		write-reg MEIS.Go 1
		refresh 1
 		inform int click-aq-redisplay 1
		set-footer "Data Acquisition Started"
	}
	return 1
}

proc click-meis-setup v {
	write-reg G*Scan 0
	click-setup $v
	click-acquisition 0
	inform erda setup-scale
}

proc click-zero-bad-events {} {
	write-reg MEIS.BadEvent 0
	redisplay-state
}

proc click-list {s} {
   global spectrumlist selection dirlist
   set selection $s
   set spectrumlist {}
   foreach i $s {lappend spectrumlist [lindex $dirlist $i]}
   redisplay-spectra
}

proc list-repaint {} {
   global selection
   foreach i $selection {:modify DA-LIST :select $i}
}

proc click-deselect-all {} {
   global spectrumlist selection
   redisplay-spectra
   :modify DA-LIST :select -1
   set spectrumlist {}
   set selection {}
}

proc click-select-all {} {
   global dirlist spectrumlist selection
   redisplay-spectra
   :modify DA-LIST :select "0 end"
   set spectrumlist $dirlist
   set selection [[:path DA-list].list curselection]
}

proc click-corrections {} {
   inform correct start
}

proc redisplay-state {} {
  global VMEserver server going

set server $VMEserver

  set running [da_state]
  set badevent [read-reg MEIS.BadEvent]
  set erda     [read-reg ERDA.Enable]
  set runnum [run-num-read]

#  :modify DA-HISTOGRAMMER :value $server
  :modify DA-ACQUISITION  :value $running
  :modify DA-BAD-EV       :value $badevent
  if {$erda} then {
    :modify DA-M1 :label "ERDA"
  } else {
    :modify DA-M1 :label "Normal"
  }
  :modify DA-RUN-NUM      :value $runnum
}

proc click-redisplay {} {
   redisplay-state
   redisplay-spectra
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"}  then {click-quit; return}
   if {$c==":Enter"} then {list-repaint; return}
   switch -glob $w {
      AQS-*      {click-aqs $arg}
      AQW-*      {click-aqw $arg}
      DA-REDISPLAY {click-redisplay}
      DA-ZERO      {click-zero}
      DA-SHOW      {click-show}
      DA-SAVE      {click-aqs-start}
      DA-INFO      {click-information}
      DA-HISTOGRAMMER {click-server $v}
      DA-ACQUISITION  {click-acquisition $v}	
      DA-LIST         {click-list [lindex $arg 4]}
      DA-STATISTICS   {inform stats start}
      DA-HELP         {click-help}
      DA-DESELECT-ALL {click-deselect-all}
      DA-SELECT-ALL   {click-select-all}
      DA-BAD-ZERO     {click-zero-bad-events}
      default    {puts stdout $arg}
   }
}

proc start args {
   global started VMEEnable dirlist specdetails
   global SpectrumServer SpectrumServerList env resource
   global VMEServer RegisterServer

   set SpectrumServerList {}
   set histpresent 0
   if {[catch {eg "(experiment ?resources)"} m]} then {
       midas-warning "no experiment connected!!!"
   } else {
       foreach item $m {
          if {[second [fourth $item]] == "spectrum" && [second [fifth $item]] == "hist"} {
              lappend SpectrumServerList [second [first $item]]
          }
       }
   }

   if {[llength $SpectrumServerList] > 0} {
       set SpectrumServer "[first $SpectrumServerList]"
       set env(SHistServer) $SpectrumServer
       set resource $SpectrumServer
       set histpresent 1
   }

   if {$started == 0} then {make-frame} else {open-frame}
   if {$VMEEnable == 1} then {
      starttask vmecontrol
      starttask vmesaverestore
      set RegisterServer [inform vmecontrol inquire Role]
      set VMEServer [inform vmecontrol inquire Role]
   }
   starttask int
   starttask gon
   starttask operator
   starttask erda
   set sdstarted [inform sd inquire started]
   if {$sdstarted == 0} then {
      send sd set spectrum_height 350
   }
   starttask sd
   if {$sdstarted == 0} then {
      send sd position-frame 90 30
      send sd iconize-frame
   }
   set started 1
   set dirlist {}
   click-redisplay
#   if {$dirlist!=[lsort [array names specdetails]]} then {click-meis-setup 1}
   foreach s [array names specdetails] {
      if {[lsearch $dirlist $s] == -1} then {
inform vmecontrol write-vme-register NetVar_HasBeenConfigured 0
inform vmesaverestore write-registers NetVar_HasBeenRestored 0
click-meis-setup 1; break
      }
   }
}


proc make-frame {} {
  global server ERDA

set xvpos 14c

   :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F  \
        :icon-name [self icon-title]

   :enable [self frame-name] Enter

   :panel DA-PANEL1 :padx 5 :pady 5 :layout H
   :layout H; :col-gap 10; :row-gap 10

#   :text DA-HISTOGRAMMER :label "Resource:" :w 14 :valuex $xvpos :value "$server"
   :next-row T
   :text DA-RUN-NUM :label "Run Number:" :w 14 :valuex $xvpos :deaf T
   :next-row T
   :choice DA-ACQUISITION :label "Acquisition:" :valuex $xvpos \
         :strings [list Stop Go]
   :button DA-SETUP  :label "Setup" :command "click-meis-setup 0"
   :next-row T
   :col-gap 2
   :message DA-M0 :label "Data acquisition mode:"
   :message DA-M1 :label "None" :bold T
   :next-row T
   :col-gap 10
   :text DA-BAD-EV :label "Bad Events:" :w 10 :valuex $xvpos :deaf T
   :button DA-BAD-ZERO  :label "Zero"
   :next-row T
   :hskip 32 
:button DA-STATISTICS  :label "Statistics" :w 24
   :next-row T
   :button DA-SELECT-ALL  :label "Select all"
   :button DA-DESELECT-ALL  :label "Deselect all"

   :panel DA-PANEL2 :padx 5 :pady 5 :layout H
   :layout H; :col-gap 10; :row-gap 10

   :list DA-LIST :rows 12 :cols 30 :exclusive F

   :panel DA-BUTTONS :padx 5 :layout B :after FOOTER
   :layout H; :col-gap 10; :row-gap 10

   :button DA-REDISPLAY :label "Redisplay"
   :button DA-ZERO      :label "Zero"
   :button DA-SHOW      :label "Show"
   :button DA-SAVE      :label "Save..."
   :button DA-INFO      :label "Info..."
   :button DA-HELP      :label "Help..."

  :show T

position-frame 660 600

}

# This is a patch replacement for the common code

proc da_state {} {
  global VMEEnable
  global going

#    if any one of the controlled subsystems is in the GO state then the
#    overall state is GO button

  set state 0

  set going [read-reg MEIS.Go]
#  set scanning [read-reg Detector.Scan.Spectrum]
  set scanning 0
  set list [read-regs G*Scan]
  foreach pair $list {
    if {[second $pair] != 0} then {
      set scanning 1; break
    }
  }

  if {$VMEEnable == 1} then {
    if {$going || $scanning} then {set state 1}
  }

  return $state
}
