# Squash the Energy Axis of a 2D Spectrum
# (i.e. Reduce number of bins needed to cover full range)

set frame SS
set frametitle "Perform spectrum squash energy axis"
set icontitle SS

set autoshow 1

set directory ""
set spectrum ""
set result ""
set resultset 0
set comment ""
set factor 2

proc last-n {p n} {
   set i [expr [llength $p]-$n]
   lrange $p $i end
}

proc file-front {n} {return [join [front [split $n "/"]] "/"]}

proc file-last {n} {return [last [split $n "/"]]}

proc list-take {as is} {
   set ll {}
   foreach e $is {lappend ll [lindex $as $e]}
   return $ll
}

#proc click-help {} {inform mb goto {(db)projecting}}

proc get-comment {} {
   global comment directory spectrum
   set z [catch {EG "spectrum :name '$directory/$spectrum' ?label"} m]
   if {$z==0} then {
	set comment $m
	:modify COMMENT :value "$comment"
   }
}

proc click-spectrum {v} {
   global debug spectrum resultset result factor
   if {$debug} then {puts stdout "$v"}
   set spectrum $v
   set result $spectrum.s$factor
   set resultset 0
   get-comment
   repaint-spectrum
   clear-footer
}

proc click-execute {} {
   global  directory spectrum result 
   if {$spectrum=={}} then {set-footer "No spectrum specified"; return}
   if {$result=={}} then {set-footer "No result spectrum specified"; return}
   if {$directory=={}} then {set-footer "No directory specified"; return}
   set z [catch {eg "(spectrum :name '$directory/$spectrum' ?spec)"}]
   if {$z!=0} then {set-footer "$directory/$spectrum not a spectrum"; return}
   set z [catch {eg "(spectrum :name '$directory/$result' ?spec)"}]
   if {$z==0} then {confirm-squash $directory/$result; return}
   squash-continue 2
}

proc click-result {v} {
  global result resultset
  set result $v
  set resultset 1
}

proc click-directory {v} {
  global directory
  set directory $v
}

proc click-comment {v} {
  global comment
  set comment $v
}

proc click-autoshow {v} {
  global autoshow
  set autoshow $v
}

proc click-factor {v} {
  global spectrum factor result resultset
  set factor $v
  if {!$resultset} then {
    set result $spectrum.s$factor
    repaint-spectrum
  }
}

proc squash-continue {v} {
   global directory result comment spectrum factor autoshow

   if {!$v} then {set-footer "squash spectrum cancelled"; return}

   set frompath $directory/$spectrum
   set topath $directory/$result

   if {$v==1} then {
	set-footer "Deleting $topath..."
	catch {EG "spectrum :name '$topath' !delete"}
   }

   set z [catch {eval exec squashspectrum $frompath $topath $factor} m]
   if {$z == 0 && $m == ""} then {
      set-footer "Result spectrum $topath successfully created"
   } else {
      if {$z == 0} then {midas-report $m} \
      else {midas-warning $m; return}
   }

   inform sb "redisplay-if-showing $directory"

   EG "spectrum :name '$topath' :label '$comment'"
   catch {add-info $topath}

   if {$autoshow} then {inform sd show [transparent $topath]}
}

proc add-info s {
   global factor
   set infolist [EG "spectrum :name '$s' :infonum 3 ?info"]
   lappend infolist "Squashed: x$factor"
   EG "spectrum :name '$s' :infonum 3 :info '$infolist'"
}

proc start args {
   global started directory spectrum result factor
   if {[llength $args]==1} then {set args [first $args]}
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   if {[llength $args]>0} then {
	set directory [file-front [first $args]]
	set spectrum  [file-last  [first $args]]
	set result $spectrum.s$factor
	get-comment 
   }
   repaint
   clear-footer
}


proc make-frame {} {
  global autoshow factor

set col1 17c

   :frame [self frame-name] :label "[self frame-title]" :resize F  :show-footer T \
      :icon-name "[self icon-title]"

   :panel P :padx 5 :pady 5
      :layout H; :col-gap 10; :row-gap 10

   :text DIRECTORY :label Directory: :w 32 :valuex $col1
   :next-row T
   :text SPECTRUM  :label Spectrum:  :w 32 :valuex $col1
   :next-row T
   :text RESULT    :label Result:    :w 32 :valuex $col1
   :next-row T
   :text COMMENT   :label Comment:   :w 32 :valuex $col1
   :next-row T
   :number FACTOR   :label "Squashing Factor:"   :w 4 :min 1 :max 250 :valuex $col1 :value $factor
   :next-row T
   :checkbox AUTOSHOW :label "Show Result:" :value $autoshow :valuex $col1
   :button EXECUTE :label Execute
#   :button HELP    :label Help...

  :show T
}   

proc repaint {} {
   global spectrum result directory autoshow factor
   global comment
   :modify DIRECTORY :value $directory
   :modify SPECTRUM  :value "$spectrum"
   :modify RESULT    :value "$result"
   :modify COMMENT   :value "$comment"
   :modify AUTOSHOW  :value $autoshow
   :modify FACTOR :value $factor
}

proc repaint-spectrum {} {
   global directory spectrum result
   :modify DIRECTORY :value $directory
   :modify SPECTRUM  :value "$spectrum"
   :modify RESULT    :value "$result"
}


proc confirm-squash {s} {

   set i [tk_dialog [self frame-name].quitdialog "Ksquared" \
      "spectrum $s already exists\n Please confirm that you want to overwrite it" \
      "warning" 1 "cancel" "overwrite"]
   if {$i} then {squash-continue 1}

}
