# Spectrum Tiling application

# 2-5-95 PHO
#   started by copying add.tcl and modifying

set frame TILE
set frametitle "Tile Spectra"
set icontitle Tile

set autoshow 1

set name ""
set directory ""
set resultname ""
set resultdirectory ""
set resultcomment ""
set defaultdirectory "/disc/specs"
set summandlist {}
set summandstrings {}
set selection -1

proc format-summands l {
   set ll {}
   foreach f $l {lappend ll [format "%-4s %-4s %15s  %-24s" \
       [third $f] [fourth $f] [first $f] [second $f]]}
   return $ll
}

proc clear-summand {} {
   global name directory
   set name ""
   set directory ""
}

proc load-summand n {
   global name directory summandlist
   set summand [lindex $summandlist $n]
   set name [first $summand]
   set directory [second $summand]
}

proc append-spectrum s {
   global summandlist resultname resultinfo resultcomment

   set name [last [split $s "/"]]
   set directory [join [front [split $s "/"]] "/"]
   set list [EG "spectrum :name '$s' :infonum 3 ?info"]
   set energy ""
   set angle ""
   foreach pair $list {
    if {"[first $pair]"=="Energy:"} then {
       set energy [first [split [second $pair] .]]
    }
    if {"[first $pair]"=="Angle:"} then {
       set angle [first [split [second $pair] .]]
    }
   }
   if {[llength $summandlist]==0} then {
    set resultname $name.tile
    set resultinfo $list
    if {$resultcomment==""} then {
       set z [catch {EG "spectrum :name '$s' ?label"} m]
          if {$z==0} then {
        set resultcomment $m
        repaint
          }
    }
   }
   lappend summandlist [list $name $directory $energy $angle]
}

proc paint-summandlist {} {
   global summandlist summandstrings selection
   set summandstrings [format-summands $summandlist]
   :modify LIST :empty T :strings $summandstrings
   if {$selection>=0} then {:modify LIST :select $selection}
}
                 
proc repaint {} {
   global name directory resultname resultdirectory resultcomment autoshow
   :modify NAME             :value "$name"
   :modify DIRECTORY        :value $directory
   :modify AUTOSHOW         :value $autoshow
   :modify RESULT-NAME      :value "$resultname"
   :modify RESULT-DIRECTORY :value $resultdirectory
   :modify RESULT-COMMENT   :value "$resultcomment"
}

proc list-take {as is} {
   set ll {}
   foreach e $is {lappend ll [lindex $as $e]}
   return $ll
}

proc check-summand {} {
   global order units name directory summandlist
   if {[scan $name "%s" name]!=1} then {return 1}
   if {[scan $directory "%s" directory]!=1} then {return 1}
   return 0
}

proc click-append {} {
   global order units name directory summandlist
   if {[check-summand]!=0} then \
      {set-footer "you haven't supplied enough data for a summand!!!"; return}
   append-spectrum $directory/$name
   paint-summandlist
   clear-footer
}

proc click-replace {} {
   global selection order units name directory summandlist
   if {$selection<0} then {
    set-footer "you haven't selected a summand to replace!"
    return
   }
#   set summandlist [lreplace $summandlist $selection $selection \
#                    [list $name $directory]]
   set summandlist [lreplace $summandlist $selection $selection]
   append-spectrum $directory/$name
   set selection -1
   paint-summandlist
   clear-footer
}

proc click-remove {} {
   global selection summandlist
   if {$selection<0} then \
      {set-footer "You haven't selected a summand to remove!"; return}
   set summandlist [lreplace $summandlist $selection $selection]
   set selection -1
   paint-summandlist
   clear-footer
}

proc click-remove-all {} {
   global selection summandlist
   set summandlist {}
   set selection -1
   paint-summandlist
   clear-footer
}

proc make-summand-list {ss} {
   global debug
   set ll {}
   foreach p $ss {append ll "[second $p]/[first $p] "}
   if {$debug} then {puts stdout "$ll"}
   return $ll
}

proc click-start-tiling {} {
   global resultdirectory resultname summandlist
   if {[llength $summandlist]==0} then {
      set-footer "no spectra selected!!!"; return}
   if {$resultname=={}} then {
      set-footer "You haven't given a name for the result!!!"; return}
   if {$resultdirectory=={}} then {
      set-footer "You haven't given a directory for the result!!!"; return}
   set z [catch {eg "(spectrum :name '$resultdirectory/$resultname' ?spec)"}]
   if {$z==0} then {confirm-tiling $resultdirectory/$resultname; return}
   tiling-continue 2
}

proc tiling-continue {v} {
   global resultdirectory resultname resultcomment summandlist autoshow

   if {!$v} then {set-footer "Tiling cancelled"; return}

   set resultspec $resultdirectory/$resultname

   if {$v==1} then {
    set-footer "Deleting $resultspec..."
    catch {EG "spectrum :name '$resultspec' !delete"}
   }
   set-footer "Tiling..."

   set z [catch {eval exec mergespectra \
    [make-summand-list $summandlist]  $resultspec} m]
   clear-footer
   if {$z == 0 && $m == ""} then {
      set-footer "Result spectrum $resultspec successfully created"
   } else {
      if {$z == 0} then {midas-report $m} \
      else {midas-warning $m; return}
   }

   inform sb "redisplay-if-showing $resultdirectory"

   EG "spectrum :name '$resultspec' :label '$resultcomment'"

   if {$autoshow} then {inform sd show [transparent $resultspec]}
}

proc click-help {} {inform mb goto {(db)tileing up}}

proc click-summandlist {s} {
   global selection

   set selection $s
   load-summand $selection
         
   repaint
   clear-footer
}

proc click-redisplay {} {
   paint-summandlist
   repaint
   clear-footer
}

proc click arg {
   global debug
   global name directory autoshow resultname resultdirectory resultcomment
   if {$debug} then {puts stdout "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch $w {
      ELECTRONICS      {click-electronics}
      APPEND           {click-append}
      REPLACE          {click-replace}
      REMOVE           {click-remove}
      REMOVE-ALL       {click-remove-all}
      START            {click-start-tiling}
      LIST             {click-summandlist [lindex $arg 4]}
      NAME             {set name $v}
      DIRECTORY        {set directory $v}
      AUTOSHOW         {set autoshow $v}
      RESULT-NAME      {set resultname $v}
      RESULT-DIRECTORY {set resultdirectory $v}
      RESULT-COMMENT   {set resultcomment $v}
      HELP             {click-help}
      REDISPLAY        {click-redisplay}
      default    {puts stdout $arg}
   }
#   clear-footer
}

proc start args {
   global started env resultdirectory defaultdirectory
   if {[llength $args]==1} then {set args [first $args]}
   if {$resultdirectory==""} then {
    set resultdirectory $defaultdirectory
    if {[catch {set env(SPECTRUMDIR)}]==0} then {
        if {$env(SPECTRUMDIR)!=""} then {set resultdirectory $env(SPECTRUMDIR)}
    }
   }
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1

   foreach s $args {append-spectrum $s}
   paint-summandlist
   repaint
}



proc make-frame {} {
  global autoshow

   set col1 15c
   
  :frame [self frame-name] :label "[self frame-title]" :resize F :show-footer T  \
      :icon-name [self icon-title]

  :panel P1 :layout H :padx 5 :pady 5
  :layout V; :row-gap 10; :col-gap 10

  :text NAME              :label "Name:"      :w 16 :valuex $col1
  :text DIRECTORY         :label "Directory:" :w 16 :valuex $col1
  :checkbox AUTOSHOW      :label "Show Result:" :value $autoshow :valuex $col1
  :text RESULT-NAME       :label "Result:"    :w 16 :valuex $col1
  :text RESULT-DIRECTORY  :label "Directory:" :w 16 :valuex $col1

  :panel P2 :layout H :padx 5
  :layout V; :row-gap 10; :col-gap 10

  :message M1   :label "keV  deg                 name                         directory" :bold T
  :list LIST  :rows 8 :cols 50 :exclusive T

  :panel P3 :padx 5 :layout B :after FOOTER
  :layout H; :row-gap 10; :col-gap 10

  :text RESULT-COMMENT    :label "Comment:"   :w 40 :valuex $col1
  :next-row T

  :button REDISPLAY  :label "Redisplay"
  :button APPEND     :label "Append"
  :button REPLACE    :label "Replace"
  :button REMOVE     :label "Remove"
  :button REMOVE-ALL :label "Remove all"
  :button START      :label "Tile"
  :hskip 50
  :button HELP       :label "Help..."

   :show T
}  

proc confirm-tiling {s} {

   set i [tk_dialog [self frame-name].quitdialog "TILE" \
      "Spectrum $s already exists\n Please confirm that you want to overwrite it" \
      "warning" 1 "cancel" "overwrite"]
   if {$i} then {tiling-continue 1}

}
