# Convert spectrum to ascii format



set frame TOASCII
set frametitle "Convert 1D Spectrum to ASCII Format"
set icontitle toascii


set directory ""
set resdir $env(HOME)/specs
set spectrum ""
set result ""
set resultname ""
set resultresult ""
set format 0
set formatlist {converted y xy 5y}


proc last-n {p n} {
   set i [expr [llength $p]-$n]
   lrange $p $i end
}

proc file-front {n} {return [join [front [split $n "/"]] "/"]}

proc file-last {n} {return [last [split $n "/"]]}

proc click-help {} {inform mb goto {(db)projecting}}

proc click-spectrum {v} {
   global debug spectrum
   if {$debug} then {puts stdout "$v"}
   set spectrum $v
   clear-footer
}

proc click-execute {} {
   global  directory spectrum resdir result 
   if {$spectrum=={}} then {set-footer "No spectrum specified"; return}
   if {$result=={}} then {set-footer "No result spectrum specified"; return}
   if {$directory=={}} then {set-footer "No directory specified"; return}
   set z [catch {eg "(spectrum :name '$directory/$spectrum' ?spec)"}]
   if {$z!=0} then {set-footer "$directory/$spectrum not a spectrum"; return}
   if {[file exists $resdir/$result]} then {
	confirm-toascii $directory/$result; return}
   toascii-continue 2
}

proc toascii-continue {v} {
   global directory resdir result spectrum formatlist format 

   if {!$v} then {set-footer "toascii cancelled"; return}

   set frompath $directory/$spectrum
   set topath $resdir/$result


   set f [lindex $formatlist $format]

   if {$f=="converted"} then {
	set z [catch {exec spec-to-ascii $frompath $topath $format} m]
   } else {
	set z [catch {exec eg-to-ascii -spectrum $frompath -format $f > $topath} m]
   }
   if {$z == 0 && $m == ""} then {
      set-footer "ASCII spectrum $topath successfully created"
   } else {
      if {$z == 0} then {midas-report $m} \
      else {midas-warning $m; return}
   }
}

proc click arg {
   global debug
   global result directory resdir format
   if {$debug} then {puts stdout "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w  {
      SPECTRUM  {click-spectrum $v}
      RESULT    {set result $v}
      DIRECTORY {set directory $v}
      RESDIR    {set resdir $v}
      FORMAT    {set format $v}
      EXECUTE   {click-execute}
      HELP      {click-help}
      default    {puts stdout $arg}
   }
   clear-footer
}

proc start args {
   global started directory spectrum result
   if {[llength $args]==1} then {set args [first $args]}
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1   
   if {[llength $args]>0} then {
	set directory [file-front [first $args]]
	set spectrum  [file-last  [first $args]]
	set result $spectrum.ascii
   }
   repaint
   clear-footer
}


proc make-frame {} {

   set col1 15c
   
  :frame [self frame-name] :label "[self frame-title]" :resize F :show-footer T  \
      :icon-name [self icon-title]

  :panel PANEL :padx 5 :pady 5
  :layout H; :row-gap 10

  :text DIRECTORY :label Directory: :w 32 :valuex $col1
  :next-row T
  :text SPECTRUM  :label Spectrum: :w 32 :valuex $col1
  :next-row T
  :text RESDIR    :label "Result Directory:" :w 32 :valuex $col1
  :next-row T
  :text RESULT    :label Result: :w 32 :valuex $col1
  :next-row T
  :choice-stack FORMAT :label Format: :valuex $col1  \
             :strings [list "Converted" "Ascii Y only" "Ascii X Y" "Ascii 5Y"]
  :button EXECUTE :label Execute
  :button HELP    :label Help...

  :show T
}

proc repaint {} {
   global spectrum result directory resdir format
   :modify DIRECTORY :value "$directory"
   :modify SPECTRUM  :value "$spectrum"
   :modify RESDIR    :value "$resdir"
   :modify RESULT    :value "$result"
   :modify FORMAT    :value "$format"
}

proc confirm-toascii {s} {

   set i [tk_dialog [self frame-name].quitdialog "ASCII" \
      "file $s already exists\n Please confirm that you want to overwrite it" \
      "warning" 1 "cancel" "overwrite"]
   if {$i} then {toascii-continue 2}

}

