# v785n.defn.patch.tcl
# This file modifies the standard display proc to call the MEIS ADC window
# It also defines some registers to be restored/saved with v785n registers
# These registers are associated with the ADC card but are not in hardware
# By pretending they are v785n registers they're handled by vmesaverestore

proc redisplay-adcs {} {
    global w2r
    global modulename module maxadc adcInc

    foreach W {THRESHOLD ENABLE} {
       for {set i 0} {$i < $maxadc} {incr i $adcInc} {
          set w $W$i
          set item $w2r($w)
          set register [first $item]
          set value [eval [second $item] [read-vme-register $modulename#$module.$register]]
          :modify $w :value $value
       }
    }

    inform operator click-redisplay
}


proc read-vme-names {register} {
   global CurrentVMEServer VMEregister EGallow
   DA_pause 0
   set EGallow 0x10004
   set VMEregister /$CurrentVMEServer/$register
   set vl [EG "register :pattern '$VMEregister' ?names"]
   DA_pause 1
   return $vl
}

proc {Add_CAEN V785N_Module} {name server addressdata} {
   global CurrentVMEServer maxadc adcInc

   set CurrentVMEServer  $server

   {Add_CAEN V785_Module} $name $server $addressdata

# The v785n register names are deleted by the "Add_ _Module" call
# so we redefine them as aliases to the existing registers
# (which weren't called v785n#... so haven't been deleted)

   define-vme-register $name alias:Detector.CornerA.Channel CornerA.Channel ""
   define-vme-register $name alias:Detector.CornerB.Channel CornerB.Channel ""
   define-vme-register $name alias:Detector.CornerC.Channel CornerC.Channel ""
   define-vme-register $name alias:Detector.CornerD.Channel CornerD.Channel ""

   define-vme-register $name alias:ERDA.TAC.Channel TAC.Channel ""
   define-vme-register $name alias:ERDA.TAC.Scale TAC.Scale ""
   define-vme-register $name alias:ERDA.TAC.Window1.Enable TAC.Window1.Enable ""
   define-vme-register $name alias:ERDA.TAC.Window1.Base TAC.Window1.Base ""
   define-vme-register $name alias:ERDA.TAC.Window1.Range TAC.Window1.Range ""
   define-vme-register $name alias:ERDA.TAC.Window2.Enable TAC.Window2.Enable ""
   define-vme-register $name alias:ERDA.TAC.Window2.Base TAC.Window2.Base ""
   define-vme-register $name alias:ERDA.TAC.Window2.Range TAC.Window2.Range ""
   define-vme-register $name alias:ERDA.TAC.Window3.Enable TAC.Window3.Enable ""
   define-vme-register $name alias:ERDA.TAC.Window3.Base TAC.Window3.Base ""
   define-vme-register $name alias:ERDA.TAC.Window3.Range TAC.Window3.Range ""
#   define-vme-register $name alias:ERDA.Enable ERDA.Enable ""

   for {set i 0} {$i < $maxadc} {incr i $adcInc} {
      define-vme-register $name alias:Detector.ADC.Pedestal$i Pedestal$i ""
   }
}

proc {XCAEN V785N_DAQ-reset} {args} {
#   V785N
    {CAEN V785_DAQ-reset} [first $args] [second $args]
    global CurrentVMEServer
    set server [second $args]
    if {$server != ""} then {set CurrentVMEServer $server}
    set module [first $args]
#    re-enable overflow suppression
    write-vme-register $module.Bit_Clear_2 0x8
}

proc {XCAEN V785N_DAQ-flush} {args} {
#   V785N
    {CAEN V785_DAQ-flush} [first $args] [second $args]
    global CurrentVMEServer
    set server [second $args]
    if {$server != ""} then {set CurrentVMEServer $server}
    set module [first $args]
#    re-enable overflow suppression
    write-vme-register $module.Bit_Clear_2 0x8
}

proc {XCAEN V785N_DAQ-setup} {args} {
#   V785N
    {CAEN V785_DAQ-setup} [first $args] [second $args] [third $args]
    global CurrentVMEServer
    set server [second $args]
    if {$server != ""} then {set CurrentVMEServer $server}
    set module [first $args]
#    re-enable overflow suppression
    write-vme-register $module.Bit_Clear_2 0x8
}

set Extra_patterns {Pedestal* Corner?.Channel TAC* Bit_Set_2}
foreach p $Extra_patterns {lappend {CAEN V785N_registersavepatterns} $p}
