
loadsource webcam

set frame VC
set frametitle "Videocam"
set icontitle "Videocam"

set started 0
set startedc 0

set open1 0
set open2 0

set captures [list "T" "F"]

set devices [list /dev/video0 /dev/video1]

proc toggle_capture w {
   global captures
    set act [:query $w :inactive]
    set cap [lsearch $captures $act]
    capture $w [expr !$cap]
}

proc mkphoto c {
    global PHOTO
    if {[info exists PHOTO]} {
	incr PHOTO
    } else {
	set PHOTO 1
    }
    set p photo_$PHOTO
    image create photo $p
    set w .$p
    toplevel $w
    wm title $w $p 
    label $w.p -image $p
    pack $w.p -side top -fill both -expand 1
    [:path VID] photo $p
}

proc quit-cam f {
   global startedc
    close_cam 1
    close_cam 2
    if {$startedc} {click-quit $f}
    set startedc 0
}

proc click arg {
    global debug startedc started

   if {$debug} {insert-debug "click with argument \{$arg\}"}

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]

   if {$c == ":quit"} then {
      set f0 [:path $f]
      if {$f0 == "[self frame-name]"} then {
        if {[midas-confirmation "Are you sure you want to quit the video camera windows?" "Yes" "No"] == 0} then {
           quit-cam CAM
           click-quit $f; return
        }
      }
      if {$f == "CAM"} then {
        if {[midas-confirmation "Are you sure you want to quit the video camera window?" "Yes" "No"] == 0} then {
           quit-cam $f; return
        }
      }
   }
}

proc make-frame {} {
    global env devices

    v4lng .v4l;   set devs [.v4l devices]

   :frame [self frame-name] :label "[self frame-title]" :resize F \
      :icon-name "[self icon-title]"
      
     :panel P1 :layout H
       set dev [first $devices]; set avail [lsearch $devs $dev]
       if {$avail != -1} {:button CAM1 :label "Camera 1" :command "camera 1"}
       set dev [second $devices]; set avail [lsearch $devs $dev]
       if {$avail != -1} {:button CAM2 :label "Camera 2" :command "camera 2"}

  :show T
}

proc Video_MakeFrame c {
    global env

  :frame CAM :label "Video Camera $c"   :show-footer T  :w 640 :h 480  :resize F

      :panel PV  :x 0 :y 0
        :layout V
        :webcam VID :w 640 :h 480 :inactive F

  :show T
}

proc capture {w c} {
    global captures
    set act [lindex $captures $c]
    :modify $w :inactive $act
}

proc attr_chg {c w attr args} {
    global ATTR
    set nv [[:path VID] setattr $attr $ATTR($w)]
}

proc setfps {c val} {
    global ATTR
    [:path VID] fps $val
}

proc pfps c {
    global ATTR
    after 1000 pfps
    set ATTR(fps) [format "FPS=%5.2f" [lindex [[:path VID] fps] 1]]
}

proc start {} {
    global started

    if {[set started]} then {
       open-frame
    } else {
       set started 1
       make-frame
    }
#    video 1
}

proc video c {
    global startedc

    if {[set startedc]} then {
       open-frame CAM
    } else {
       Video_MakeFrame $c
    }
    capture VID 0
    close_cam [expr ($c == 1) ? 2 : 1]
    open_cam $c
    capture VID 1
    if {[set startedc]} then {
      after 500
      :modify CAM :label "Video Camera $c"
    }
    set startedc $c
}

proc open_cam c {
    global devices
    global open$c
    if {![set open$c]} {
      set thisdevice [lindex $devices [expr $c - 1]]
      set avail [lsearch [[:path VID] devices] $thisdevice]
      if {$avail == -1} {
         midas-warning "No suitable video device available"
      } else {
         set open$c 1
         set z [catch {[:path VID] open $thisdevice} ]
         [:path VID] fps 5
      }
    }
}

proc close_cam c {
    global open$c
    set open$c 0
    catch {[:path VID] close}
}

proc camera v {
   global startedc

   if {$startedc == $v} {
      toggle_capture VID
   } else {
      video $v
   }
}
